// file: 110-Table.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 //Table - a simple table
Table = function Table() {this.columnNames={}; };
Table.prototype = new DisplayNode();
Table.prototype.constructor = Table;
Table.prototype.getTag = function()
{
	return 'table';
};
Table.prototype.reset = function reset()
{
	DisplayNode.prototype.reset.call(this);
	this.selectedRow = null;
	this.hiddenColumns = {};
	if (this.headerRow)
	{
		var headerCells=this.headerRow.childNodes;
		for (var i=0;i<headerCells.length;i++)
		{
			headerCells[i].style.display='';
		}
	}
}
Table.prototype.createViewNode = function createViewNode()
{
	this.hiddenColumns = {};
    DisplayNode.prototype.createViewNode.call(this);
    this.sort = (this.sharedProperties.sort == 'multiple');
    
	this.setProperties(); // In Mozilla, we have to set the styleclass before creating the body of the table
    this.createCaption();
    this.createHeader();
    this.bodyNode = this.currentWindow.document.createElement('tbody');
    var h = this.sharedProperties.contentHeight;
    if (h && h == ' ')
    	h = null;
    var w = this.sharedProperties.contentWidth;
    if (w && w == ' ')
    	w = null;
 	if (w || h)   
		setScrollableTable(this.bodyNode, w,h); // broswer dependant
    this.viewNode.appendChild(this.bodyNode);
    this.viewNode.id = this.getNodeId();
};
Table.prototype.createCaption = function createCaption()
{
	if ( this.sharedProperties.showCaption == false) // default = true
	{
		return;
	}
	else
		if ( ! this.getRowPrototype())
			return;
	this.captionDOMElement = this.createHTMLElement('caption');
	this.captionDOMElement.innerHTML = '<div>'+tersus.escapeHTML(this.getCaption())+'</div>';
	this.viewNode.appendChild(this.captionDOMElement);
};
Table.prototype.captionChanged = function()
{
	if (this.captionDOMElement)
		this.captionDOMElement.innerHTML = '<div>'+tersus.escapeHTML(this.getCaption())+'</div>';
};

Table.prototype.addHeaderCells = function addHeaderCells(headerRow,node)
/* Adds header cells based on the structure of the given node */
{
	for (var i=0; i< node.displayElements.length; i++)
	{	
		var columnElement = node.displayElements[i];
		var columnNodePrototype = columnElement.getChildConstructor().prototype;
		if (columnNodePrototype.isDisplayGroup)
		{
			this.addHeaderCells(headerRow, columnNodePrototype);
		}
		else
		{
			var labelHTML = columnElement.properties.caption;
			if (!labelHTML || labelHTML == ' ')
				labelHTML = tersus.escapeHTML(columnElement.role)
			labelHTML = this.translate(labelHTML);
			var headerCell = this.createHTMLElement('th');
			headerCell.setAttribute("element_name", columnElement.role);
			var columnName = columnElement.properties.sortColumnName;
			if (!columnName)
				columnName = columnElement.role;
			if (!tersus.useSQLQuotedIdentifiers)
				columnName = tersus.sqlize(columnName);
			headerCell.setAttribute("column_name", columnName);
			this.columnNames[columnElement.role]=tersus.useSQLQuotedIdentifiers ? columnName : columnName.toLowerCase();
			headerCell.innerHTML = labelHTML;
			var col = this.createHTMLChildElement(this.colgroup,'col');
			headerCell.col = col;
			if (this.sort && columnElement.properties.sortable != false)
			{
				headerCell.setAttribute('sort', 'none');
				headerCell.onclick = Table.headerClicked;
			}
			headerCell.onmousedown = Table.mouseDownOnHeader;
			headerCell.onmouseover = Table.updateHeaderCursor;
			headerCell.onmousemove = Table.updateHeaderCursor;
				

			if (headerRow.childNodes.length == 0)
				headerCell.className='first';
			headerRow.appendChild(headerCell);
		}
	}
};
Table.prototype.getRowPrototype = function()
{
	var output = null;
	if (this.subFlowList)
	{
		for (var i=0; i< this.subFlowList.length; i++)
		{
			var subFlow = this.subFlowList[i];
			if (subFlow.isDisplayElement && subFlow.role != '<Header>' && subFlow.role != '<Footer>')
			{
				if (output)
					return null;
				else
					output = subFlow;
			}
		}	
	}
	return output;
};
Table.prototype.createHeader = function createHeader()
{
	if ( this.sharedProperties.showHeadings == false) // default = true
		return;
	var rowElement = this.getRowPrototype();
	if (! rowElement) return;
	var rowPrototype = tersus.repository.get(rowElement.modelId).prototype;
	if (! rowPrototype.subFlowList) return;
	var headerRow = this.createHTMLElement('tr');
	this.colgroup = this.createHTMLChildElement(this.viewNode,'colgroup');
	this.addHeaderCells(headerRow, rowPrototype);
	var tableHeader = this.createHTMLElement('thead');
	tableHeader.appendChild(headerRow);
	this.header = tableHeader;
	this.headerRow = headerRow;
	this.viewNode.appendChild(tableHeader);
};

Table.prototype.removeChildViewNode = function removeChildViewNode(child)
{
	var c = child.viewNode;
	var row = null;
	if (c.tagName == 'TR' )
		row = c;
	else if (c.parentNode && c.parentNode.tagName == 'TR')
		row = c.parentNode;
	else if (c.parentNode && c.parentNode.parentNode && c.parentNode.parentNode.tagName== 'TR')
		row = c.parentNode.parentNode;
	if (row && row.parentNode)
		row.parentNode.removeChild(row);
};
Table.prototype.applyColOrder = function (row)
{
	if (this.colOrder)
	{
		var cells = [];
		for (var i=0;i<row.childNodes.length;i++)
		{
			cells.push(row.childNodes[i]);
		}
		if (cells.length != this.colOrder.length)
		{
			internalError("Failed to apply column order: mismatch between number of cells in row and number of columns in colOrder");
			return;
		}
		for (var i=0;i<cells.length;i++)
		{
			row.removeChild(cells[i]);
		}
		for (var i=0;i<cells.length;i++)
		{
			row.appendChild(cells[this.colOrder[i]]);
		}
	};
};
Table.prototype.getFooter = function()
{
	if (! this.footer)
		this.footer = 	this.header = this.createHTMLChildElement(this.viewNode, 'tfoot');
	return this.footer;
		
};
Table.prototype.appendChildViewNode = function appendChildViewNode(child, childElement)
{
	var element = childElement?childElement:child.element;
	var targetNode = this.bodyNode;
	if (element.role == '<Header>')
		targetNode = this.header;
	if (element.role == '<Footer>')
		targetNode = this.getFooter();
	var row = null;
	if (child.viewNode.tagName == 'TR')
	{
		row = child.viewNode;
	}
	else
	{
		var row = this.createHTMLElement('TR');
		if (child.viewNode.tagName == 'TH' || child.viewNode.tagName == 'TD')
			row.appendChild(child.viewNode);
		else
		{
			var cell = this.createHTMLChildElement(row,'TD');
			cell.appendChild(child.viewNode);
		}
		row.setAttribute('display_order', child.viewNode.getAttribute('display_order'));
	}
	if (this.headerRow && ! element.prop('suppressColumnHiding'))
	{
		var rowCells = row.childNodes;
		var headerCells = this.headerRow.childNodes;
		if (rowCells.length == headerCells.length)
		{
			for (var i=0;i<headerCells.length;i++)
			{
				if (headerCells[i].style.display=='none')
					rowCells[i].style.display= 'none';
			}
		}
	}
		
		 
	if (this.header) // If there is a header, we know we have a single row, so we can directly append the child row and skip display order handling
		targetNode.appendChild(row)
	else
		DisplayNode.appendChildNode(targetNode,row); 
};

Table.prototype['get<Selected Row>'] = function ()
{
	if (this.selectedRow)
		return this.selectedRow;
	else
		return null;
};
Table.prototype['set<Selected Row>'] = Table.prototype.select = function (row)
{
	//TODO(y) - check that row is a row in this table. Report an error if not.
	if (this.selectedRow)
	{
		if (this.pluginVersion>=1)
			$(this.selectedRow.viewNode).removeClass('selected');
		else
			this.selectedRow.viewNode.className = this.selectedRow.baseClassName;
	}
	this.selectedRow = row;
	if (row && row.viewNode)
	{
		if (this.pluginVersion>=1)
			$(row.viewNode).addClass('selected');
		else
			row.viewNode.className = 'selected';
	}
};
Table.prototype.select.requiresNodes = true;
Table.prototype['get<Sort Order>'] = Table.prototype.getSortOrder = function()
{
	var sortOrder = null;
	var headerCells = this.headerRow.childNodes;
	for (var i=0;i<headerCells.length;i++)
	{
		var headerCell = headerCells[i];
		var columnName = headerCell.getAttribute('column_name');
		if (tersus.useSQLQuotedIdentifiers)
			columnName = '"'+columnName+'"';
		var sort = headerCell.getAttribute('sort');
		var sortKey = null;
		if (sort == 'up')
		 	sortKey=' ASC';
		else if (sort == 'down')
			sortKey=' DESC';
		if (sortKey)
		{
			if (sortOrder ==null)
				sortOrder = '';
			else
				sortOrder+=', ';
			sortOrder+=columnName;
			sortOrder+=sortKey;
		}
	}
	return sortOrder;
};
Table.prototype['set<Sort Order>'] = function (sort_order)
{
	//TODO: handle conflicts between column order and sort order
	var headerCells = this.headerRow.childNodes;
	var header_cell_map = {};
	for (var i=0;i<headerCells.length;i++)
	{
		var headerCell = headerCells[i];
		var columnName = headerCell.getAttribute('column_name');
		if (!tersus.useSQLQuotedIdentifers)
			columnName = columnName.toLowerCase();
		this.doSortNoFire(headerCell,'none');
		header_cell_map[columnName]=headerCell;
	}
	var result;
	var re;
	if (sort_order.indexOf('"')>=0)
		re = / *"([^"]+)"( +([a-z]+))? *(,|$)/gi
	else 
		re = / *([^, ]+)( +([a-z]+))? *(,|$)/gi;
	var prevHeaderCell = null;
	while (result = re.exec(sort_order))
	{
		var columnName = result[1];
		if (! tersus.useSQLQuotedIdentifiers)
			columnName = columnName.toLowerCase();
		var dir = result[3];
		dir =  (dir && dir.toLowerCase().charAt(0)=='d') ? 'down' : 'up';
		var headerCell = header_cell_map[columnName];
		this.doSortNoFire(headerCell,dir);
		if (prevHeaderCell)
			this.ensureColumnOrder(prevHeaderCell, headerCell);
		prevHeaderCell = headerCell; 
	};
};
Table.prototype['set<Hidden Columns>'] = function(hidden_cols)
{
	var re;
	if (hidden_cols.indexOf('"')>=0)
		re = / *"([^"]+)" *(,|$)/gi
	else 
		re = / *([^, ]+) *(,|$)/gi;
	var hidden = this.hiddenColumns;
	for (var columnName in hidden)
		hidden[columnName]=0;
	var result;
	while (result = re.exec(hidden_cols))
	{
		var columnName = result[1];
		if (! tersus.useSQLQuotedIdentifiers)
			columnName = columnName.toLowerCase();
		if (hidden[columnName] == null)
		{
			this.hideColumn(columnName);
		}
		else
			hidden[columnName]=1; // Just mark that this column is still hidden
	}
	for (var columnName in hidden)
	{
		if (hidden[columnName] == 0) // was hidden but no longer is
		{
			this.showColumn(columnName);
		}
	}
	this.rebuildColgroup();
};
Table.prototype['remove<Hidden Columns>'] = function()
{
	for (var columnName in this.hiddenColumns)
	{
		this.showColumn(columnName);
	}
	this.rebuildColgroup();

};

Table.prototype['get<Hidden Columns>'] =  function()
{
	var hidden = this.hiddenColumns;
	if (hidden == null)
		hidden = {}; 
	var hiddenCols = null;
	var headerCells = this.headerRow.childNodes;
	for (var i=0;i<headerCells.length;i++)
	{
		var headerCell = headerCells[i];
		var columnName = headerCell.getAttribute('column_name');
		if (hidden[columnName.toLowerCase()])
		{
			if (hiddenCols == null)
				hiddenCols = '';
			else
				hiddenCols+=', ';
			if (tersus.useSQLQuotedIdentifiers)
				columnName = '"'+columnName+'"';	
			hiddenCols+=columnName;
		}
	}
	return hiddenCols;
};
Table.prototype['get<All Columns>'] =  function()
{
	var allCols = null;
	var headerCells = this.headerRow.childNodes;
	for (var i=0;i<headerCells.length;i++)
	{
		var headerCell = headerCells[i];
		var columnName = headerCell.getAttribute('column_name');
		if (tersus.useSQLQuotedIdentifiers)
			columnName = '"'+columnName+'"';
		if (allCols == null)
			allCols = '';
		else
			allCols+=',';
		allCols+=columnName;
	}
	return allCols;
};

Table.prototype.hideColumn = function(columnName)
{
	this.hiddenColumns[columnName] = 1;
	var headerCells = this.headerRow.childNodes;
	var rows = this.viewNode.rows;
	for (var i=0;i<headerCells.length;i++)
	{
		var headerCell = headerCells[i];
		var headerColumnName = headerCell.getAttribute('column_name');
		if (!tersus.useSQLQuotedIdentifiers)
			headerColumnName=headerColumnName.toLowerCase();
		if ( columnName == headerColumnName )
		{
			var cw = getVisibleSize(headerCell).width;
			headerCell.col.style.width = cw + 'px';
			if (this.viewNode.style.width)
			{
				var originalWidth = parseInt(this.viewNode.style.width);
				this.viewNode.style.width = (originalWidth-cw) + 'px';
			}
			headerCell.style.display='none';
			if (this.colgroup == headerCell.col.parentNode)
				this.colgroup.removeChild(headerCell.col);
			for (var j=0;j<rows.length; j++)
			{
				var rowCells = 	rows[j].childNodes;
				rowCells[i].style.display = 'none';
			}
		}
	}
	
};
Table.prototype.showColumn = function(columnName)
{
	delete this.hiddenColumns[columnName];
	var headerCells = this.headerRow.childNodes;
	var rows = this.viewNode.rows;
	for (var i=0;i<headerCells.length;i++)
	{
		var headerCell = headerCells[i];
		var headerColumnName = headerCell.getAttribute('column_name');
		if (!tersus.useSQLQuotedIdentifiers)
			headerColumnName=headerColumnName.toLowerCase();
		if ( columnName == headerColumnName )
		{
			if (this.viewNode.style.width)
			{
				var cw = parseInt(headerCell.col.style.width);
				if (cw)
				{
					var originalWidth = parseInt(this.viewNode.style.width);
					this.viewNode.style.width = (originalWidth+cw) + 'px';
				}
			}
			else
				headerCell.col.style.width='';
			headerCell.style.display='';
			for (var j=0;j<rows.length; j++)
			{
				var rowCells = 	rows[j].childNodes;
				rowCells[i].style.display = '';
			}
		}
	}
}
Table.prototype.setColWidths = function()
{
	var headerCells = this.headerRow.childNodes;
	for (var i=0;i<headerCells.length;i++)
	{
		var headerCell = headerCells[i];
		var c = headerCell.col;
		if (headerCell.style.display!='none')
		{
			c.style.width=getActualStyle(headerCell,'width');
		}
	}

};
Table.prototype.rebuildColgroup = function()
{
	var headerCells = this.headerRow.childNodes;
	var cg = this.colgroup;
	var w= [];
	var cc = [];
	for (var i=0;i<headerCells.length;i++)
	{
		var headerCell = headerCells[i];
		var c = headerCell.col;
		if (cg == c.parentNode)
			cg.removeChild(c);
		if (headerCell.style.display!='none')
		{
			cg.appendChild(c);
		}
	}
//	this.currentWindow.setTimeout(function(){for (var i=0;i<w.length;i++) cc[i].style.width=w[i];},2000);
};	
Table.prototype.ensureColumnOrder = function (cell1, cell2) // ensures that cell2 is not before cell1
{
	var allSiblings = cell1.parentNode.childNodes;
	var i1=-1;
	var i2=-1;
	for (var i=0;i<allSiblings.length;i++)
	{
		if (allSiblings[i] == cell1)
			i1 = i;
		if (allSiblings[i] == cell2)
			i2 = i;
	}
	if (i2 < i1) // rearrage columns in the table so that cell2 appears right after cell1
	{
		this.moveCells(i2,i1+1);
	};
};
Table.isResizeLeft = function(e,headerCell)
{
	var offsets = getScreenPosition(headerCell);
	var win = getParentWindow(headerCell);
	var p = getEventPosition(win,e);
	return (p.x <= offsets.left +5);
};
Table.isResizeRight = function(e,headerCell)
{
	var offsets = getScreenPosition(headerCell);
	var size = getVisibleSize(headerCell);
	var win = getParentWindow(headerCell);
	var p = getEventPosition(win,e);
	return (p.x >= offsets.left + size.width -5);
};
Table.isResize = function(e, headerCell)
{
	return Table.isResizeRight(e,headerCell)
	 || headerCell.previousSibling && Table.isResizeLeft(e,headerCell);
};
Table.headerClicked = function(e)
{
	if (! Table.isResize(e,this))
	{
		var tableViewNode = this.parentNode.parentNode.parentNode;
		var tableNode = DisplayNode.findNodeForDocumentNode(tableViewNode);
		if (!tableNode)
			return;
		var win = tableNode.currentWindow;
		if (! getEventPosition(win,e).equals(tableNode.mouseDownPosition))
			return; // Ignore click if it's part of a drag operation
		var sort = this.getAttribute('sort');
		if (sort == 'up')
			tableNode.doSort(this,'down');
		else if (sort == 'down')
			tableNode.doSort(this,'none');
		else
			tableNode.doSort(this, 'up');
	}
};
Table.prototype.doSortNoFire = function(headerCell,dir)
{
	headerCell.setAttribute('sort',dir);
	headerCell.className = 'sort_'+dir;
};
Table.prototype.doSort = function(headerCell, dir)
{
	this.doSortNoFire(headerCell,dir);
	this.fireOnSort();
}
Table.prototype.fireOnSort = function()
{
	this.mainWindow.engine.handleEvent(this, Events.ON_SORT);
};
Table.mouseDownOnHeader = function(e)
{
	var tableViewNode = this.parentNode.parentNode.parentNode;
	var tableNode = DisplayNode.findNodeForDocumentNode(tableViewNode);
	if (!tableNode)
		return;
	var offsets = getScreenPosition(this);
	var size = getVisibleSize(this);
	var win = tableNode.currentWindow;
	clearSelection(win);
	tableNode.mouseDownPosition  = getEventPosition(win,e);
	if (Table.isResizeLeft(e,this))
	{
		var previousCell = this.previousSibling;
		if (previousCell)
			tableNode.beginResize(e,previousCell);
	}
	else if (Table.isResizeRight(e,this))
		tableNode.beginResize(e,this);
	else
		tableNode.beginReorder(e,this);
};
Table.finishDrag = function (e)
{
	var table=tersus.findNode(this.documentElement.getAttribute('tableId'));
	this.documentElement.setAttribute('tableId', null);
	this.onmouseup = null;
	this.onmousemove = null;
	table.viewNode.className=table.baseClassName;
};
Table.updateHeaderCursor = function (e)
{
	var tableId = this.ownerDocument.documentElement.getAttribute('tableId');
	var table = null;
	if (tableId)
		table=tersus.findNode(tableId);
	if (table && table.reorder)
		this.style.cursor='';
	else
	{
		if (Table.isResize(e,this))
		{
			this.style.cursor='e-resize';
		}
		else if (this.getAttribute('sort'))
			this.style.cursor='pointer';
		else
			this.style.cursor='';
	}
};
Table.performResize = function (e)
{
	var table=tersus.findNode(this.documentElement.getAttribute('tableId'));
	var win = table.currentWindow;
	clearSelection(win);
	var currentPosition = getEventPosition(win,e);
	var delta = {};
	delta.x = currentPosition.x - table.lastPosition.x;
	delta.y = currentPosition.y - table.lastPosition.y;
	table.lastPosition = currentPosition;		
	table.resize(delta);
};
Table.prototype.beginResize = function beginResize(e, headerCell)
{
	this.setColWidths();
	var table = this;
	var win = this.currentWindow;
	clearSelection(win);
	var doc = win.document;
	doc.documentElement.setAttribute('tableId', this.getNodeId());
	table.lastPosition = getEventPosition(win, e);
	table.viewNode.className+=' resizing';
	table.resizingHeaderCell = headerCell;
	table.currentColWidth = getVisibleSize(headerCell).width;
	table.currentTableWidth = getVisibleSize(table.viewNode).width;
	doc.onmousemove = Table.performResize;
	doc.onmouseup = Table.finishDrag;
};
Table.prototype.beginReorder = function beginReorder(e, headerCell)
{
	var table = this;
	var win = this.currentWindow;
	clearSelection(win);
	var doc = win.document;
	var feedback = doc.createElement('div');
	feedback.className='drag_feedback';
	doc.documentElement.setAttribute('tableId', this.getNodeId());
	table.reorder = {};
	table.reorder.feedback =feedback;
	table.reorder.headerCell = headerCell;
	table.reorder.startPosition = getEventPosition(win,e);;
	
	doc.onmousemove = Table.feedbackReorder;
	doc.onmouseup = Table.finishReorder;
};

Table.prototype.cancelReorder = function()
{
	if (! this.reorder)
		return;
	if (this.reorder.feedback.parentNode)
		this.reorder.feedback.parentNode.removeChild(this.reorder.feedback);
	this.reorder = null;
	var doc = this.currentWindow.document;
	doc.documentElement.setAttribute('tableId', null);
	doc.onmouseup = null;
	doc.onmousemove = null;
};
Table.finishReorder = function (e)
{
	var table=tersus.findNode(this.documentElement.getAttribute('tableId'));
	if (! table.reorder)
		return;
	var win = table.currentWindow;
	clearSelection(win);
	var headerCell = table.reorder.headerCell;
	var startX = table.reorder.startPosition.x;
	table.cancelReorder();
	var p = getEventPosition(win,e);
	var fromIndex=-1;
	var toIndex =-1;
	if (p.x != startX)
	{
		var headerCells =headerCell.parentNode.childNodes;
		for (var i=0;i<headerCells.length; i++)
		{
			var cell = headerCells[i];
			if (cell.style.display=='none')
				continue;
			if (cell == headerCell)
				fromIndex = i;
			else if (toIndex < 0)
			{
				var left = getScreenPosition(cell).left;
				var width = getVisibleSize(cell).width;
				if (p.x >= left && p.x <left+width)
				{ 
					if (p.x > left+width/2 )
					{
						toIndex = i+1;
					}
					else
						toIndex = i;
				}
			}
		}
		if (toIndex != -1 && toIndex != fromIndex && toIndex != fromIndex+1)
		{
			var originalSortOrder = table.getSortOrder();
			table.moveCells(fromIndex,toIndex);
			var newSortOrder = table.getSortOrder();
			if (originalSortOrder != newSortOrder)
			{
				table.fireOnSort();
			}
		}
	}
};
Table.move = function (parent, from, to)
{
	var children = parent.childNodes;
	var childFrom = children[from];
	var childTo = null;
	if (to < children.length)
	{
		childTo = children[to];
	}
	parent.insertBefore(childFrom,childTo);
};
Array.prototype.moveItem = function(fromIndex,toIndex)
{
	var item=this.splice(fromIndex,1)[0]; 
	var newIndex = (fromIndex<toIndex) ? toIndex-1: toIndex;
	this.splice(newIndex,0, item);
};
Table.prototype.moveCells = function (fromIndex,toIndex)
{
	this.reorderColGroup(fromIndex, toIndex);
	var rows = this.viewNode.rows;
	for (var i=0;i<rows.length; i++)
		Table.move(rows[i],fromIndex, toIndex);

	//Update the 'colOrder' permutation		
	if (!this.colOrder)
	{
		this.colOrder = [];
		for (var i=0;i<this.headerRow.childNodes.length;i++)
		{
			this.colOrder.push(i);
		}
	}
	this.colOrder.moveItem(fromIndex,toIndex);
};
Table.prototype.reorderColGroup = function(fromIndex, toIndex)
{
	// Since there are no 'col' elements for hidden columns, we need to correct the indices
	var headerCells  = this.headerRow.childNodes;
	var from = fromIndex;
	var to = toIndex;
	for (var i=0;i<headerCells.length;i++)
	{
		var c = headerCells[i];
		if (c.style.display == 'none')
		{
			if (i<=fromIndex)
				--from;
			if (i<=toIndex)
				--to;
		}
	}
	Table.move(this.colgroup, from, to);
};
Table.feedbackReorder = function (e)
{
	var table=tersus.findNode(this.documentElement.getAttribute('tableId'));
	var win = table.currentWindow;
	clearSelection(win);
	var headerCell = table.reorder.headerCell;
	var headerCellSize = getVisibleSize(headerCell);
	var p = getEventPosition(win,e);
	var o = getScreenPosition(table.viewNode);
	var w = getVisibleSize(table.viewNode).width;
	var h = getVisibleSize(table.viewNode).height;
	if( p.y<o.top || p.y > o.top+h || p.x<o.left || p.x > o.left + w)
	{
		table.cancelReorder();
		return;
	}
	
	var feedback = table.reorder.feedback;
	var body = win.document.body;
	if (!feedback.parentNode )
	{
		body.appendChild(feedback);
		feedback.innerHTML = headerCell.innerHTML;
		tersus.async.exec(function()
		{
			feedback.style.width=headerCellSize.width;
			feedback.style.height=headerCellSize.height;
			var d= (window.browser == 'Mozilla')?-2:0;
			feedback.style.top=(getScreenPosition(headerCell).top+d)+body.scrollTop;
			feedback.style.left=p.x+body.scrollLeft;
		}, false /* don't wait */)
	}
	else
	{
		table.reorder.feedback.style.left=p.x+body.scrollLeft;
	}
};
Table.prototype.resize = function(delta)
{
	this.currentColWidth += delta.x;
	this.currentTableWidth += delta.x;
	var table = this;
	setTimeout(function() {table.resizingHeaderCell.col.style.width = table.currentColWidth+'px';table.viewNode.style.width=table.currentTableWidth+'px'},10);
};
// file: 115-Page.js
(function( $, undefined ) {
	var setter=tersus.setter;
	

tersus.Widget('Page','HTMLTag',{
	defaultTag:'div'
	,defaultWrapperTags:'div'
	,isPage:true
	,prepareForHTML:function(e,parent)
	{
		$(this.displayElements).each(
			function(i) {
				var dataRole = {'<Header>':'header','<Footer>':'footer','<Content>':content}[this.role];
				if (dataRole)
					this.properties = $.extend({'data-role':dataRole},this.properties);
				});
		return tersus.HTMLTag.prototype.prepareForHTML.call(this,e,parent);
	}
	,getScrollContainer:function()
	{
		if (!this.scrollContainer)
		{
			var $v=$(this.viewNode);
			var $s=$v.wrapInner('<div class="tersus-scroll-container"><div class="tersus-scroll-content"/></div>').find('.tersus-scroll-container');
			this.scrollContainer  = $s[0];
		}
		return this.scrollContainer;
	}
	,onBeforeScrollStart:function(e) /*iScroll Hook to detect cases where we don't want to scroll*/
	{
		if (   e.target.tagName == "SELECT"
            || e.target.tagName == "INPUT"
            || e.target.tagName == "BUTTON"
            || e.target.tagName == "TEXTAREA") {
            return true;
        }
		e.preventDefault();
		
	}
	,attach:function()
	{
		if (window.iScroll && this.sprop('scroll')!=false)
		{
			var sc=this.getScrollContainer();
			if (sc)
			{
				var self = this;
				$(sc).find('input').bind('vclick',function(){self.onResize();});
	//			this.iscroll = new iScroll(sc,{vScrollbar:false});
				this.iscroll = new iScroll(sc,{vScrollbar:false, onBeforeScrollStart:this.onBeforeScrollStart});
				var id=this.getNodeId();
				$(window).bind('ter-process-end', function(){setTimeout(
				function()
				{
					var n = tersus.findNode(id);
					if (n && n.iscroll)
					{
						n.iscroll.refresh();
					}
				}
				,1000)});
			}
		}
		
	}
	,refresh:function()
	{
		tersus.HTMLTag.prototype.refresh.call(this);
		$(this.viewNode).pageUpdate();		
		if (this == this.parent.selectedPage)
			$(this.wrapperNode).addClass('ter-tp-active');
		
	}
	,destroy:function()
	{
//		if (this.iscroll)
//			this.iscroll.destroy();
		tersus.HTMLTag.prototype.destroy.call(this);
	}
	,onResize:function()
	{	
		//Set min-height on page content, based on explicit min-height on viewNode or wrapperNode
		var $v = $(this.viewNode);
		
		var c = this.wrapperNode || this.viewNode;
		var h;
/*		while (c.style && ! c.style.height)
			c = c.parentNode;
		if (c.style && c.style.height)
			h = parseInt(c.style.height);*/
		while (! c.style.height)
			c = c.parentNode;
		h = parseInt(c.style.height);

		if (!h)
			h = $v.outerHeight(); // In certain dialogs the relevant height is on viewNode and not wrapperNode	
		var $c = $v.find('.ui-content');
		var bh = $c.outerHeight(true)-$c.height();
		var fh = 0; $v.find('.ui-footer,.ui-header,.ter-footer,.ter-header').each(function(index){ fh+=$(this).outerHeight();});
		$c.css('min-height',h-bh-fh);
		var s = this.iscroll;
		if (s)
		{
			var sc = this.getScrollContainer();
			$(sc).height(h);
			setTimeout(function(){s.refresh();},100);
		}
			
	}
});

})(jQuery);
// file: 120-Row.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function Row() {};
Row.prototype = new DisplayNode();
Row.prototype.constructor = Row;
Row.prototype.defaultTag = 'tr';
Row.prototype.attachViewNode = function Row_createViewNode()
{
	this.setDisplayOrder();
	if (this.parent.viewNode.tagName != 'TABLE')
	{
		this.tableNode  = this.createHTMLElement('table');
		this.tbodyNode = this.createHTMLElement('tbody');
		this.tableNode.setAttribute('display_order',this.viewNode.getAttribute('display_order'));
		this.tableNode.appendChild(this.tbodyNode);
		this.tbodyNode.appendChild(this.viewNode);
		if(this.sharedProperties.width != undefined && this.sharedProperties.width != ' ')
			this.tableNode.style.width = this.sharedProperties.width;
		if (this.sharedProperties.styleClass != undefined && this.sharedProperties.styleClass != ' ')
			this.tableNode.className = this.sharedProperties.styleClass;
		else if (this.defaultStyleClass)
			this.tableNode.className = this.defaultStyleClass;
			
		var rowNode = this.viewNode;
		
		// We temporarily set the view node to the table node, because this is the node
		// we want to append to the parent
		
		// This is not very clean, and it would have probably have been more elegant
		// if we distinguished between 'contentNode' and 'viewNode', but this refactoring is currently 
		// out of scope, especially because we'll have to decide what to do with
		// things like style class etc. (which currently apply to the content node)
		// We have to be careful with backward compatability.
		
		// We DON'T simply append our table to the parent view node, because the parent node
		// may have special logic in appendChildViewNode - specifically - we want to retain the behavior of DisplayGroup.

		this.viewNode = this.tableNode;
		this.parent.appendChildViewNode(this);
		this.viewNode = rowNode;		
	}
	else
	{
		this.parent.appendChildViewNode(this);
	}
	if (this.parent.sharedProperties.zebraTable == true)
	{
		var v = this.viewNode;
		var parity = v.parentNode.childNodes.length & 1;
		var cn = v.className;
		cn = cn ? cn : '';
		this.extraClassName =  (parity ? ' odd':' even');
		v.className = cn + this.extraClassName; 
	}
}
Row.prototype.setEventHandlers = function Row_setEventHandlers()
{
	DisplayNode.prototype.setEventHandlers.call(this);

	if (this.isSelectable())	
		this.registerEventHandler(Events.ON_CLICK);	
};
Row.prototype.isSelectable = function()
{
	return this.sharedProperties.selectable==true || this.parent.sharedProperties.selectable==true; 
}
Row.prototype.select = function()
{
	if (this.isSelectable())
	{
		var table = this.parent;
		table.select(this);
	}
};
Row.prototype.onClick = function (domNode, callback)
{
	this.select();
	DisplayNode.prototype.onClick.call(this,domNode, callback);
};
Row.prototype.onChildFocus = function (child)
{
	this.select();	
};
Row.prototype.onDelete = function ()
{
	DisplayNode.prototype.onDelete.call(this);
	if (this.parent.selectedRow == this)
		this.parent.selectedRow = null;
};
Row.prototype.removeChildViewNode = function removeChildViewNode(child)
{
	var c = child.viewNode;
	var p = this.viewNode;
	if (c.parentNode == p) // c is a cell (TD or TH)
		p.removeChild(c);
	else if (p == c.parentNode.parentNode) // c is wrapped in a cell
		p.removeChild(c.parentNode);
};

Row.prototype.appendChildViewNode = function appendChildViewNode(child, childElement)
{
	if (!childElement)
		childElement = child.element;
	var cellNode;
	if (child.viewNode.tagName == 'TD' || child.viewNode.tagName == 'TH')
		cellNode = child.viewNode;
	else
	{
		var cellNode = this.createHTMLElement('td');
		cellNode.setAttribute('display_order',child.viewNode.getAttribute('display_order'));
		cellNode.appendChild(child.viewNode);
	}
	if (child.element && child.element.properties)
	{
		var elementProperties = child.element.properties;
		if (tersus.isNumber(elementProperties.colSpan))
			cellNode.colSpan = elementProperties.colSpan;
		if (tersus.isNumber(elementProperties.rowSpan))
			cellNode.rowSpan = elementProperties.rowSpan;
		if (tersus.notEmpty(elementProperties.cellWidth))
			cellNode.style.width=elementProperties.cellWidth;
		if (elementProperties.cellStyleClass && elementProperties.cellStyleClass != ' ')
			cellNode.className=elementProperties.cellStyleClass;
	}
	if (child.colSpan)
		cellNode.colSpan = child.colSpan;
	DisplayNode.appendChildNode(this.viewNode,cellNode);
	if (cellNode.parentNode.childNodes.length == 1)
	{
		var c = cellNode.className;
		c = c + ' first';
		cellNode.className=c;
		if (cellNode == child.viewNode)
		{
			child.baseClassName = cellNode.className;
			child.extraClassName = ' first';
		}
	}
	var p = this.parent;
	if (p.applyColOrder && p.headerRow && this.viewNode.childNodes.length == p.headerRow.childNodes.length)
	{
		p.applyColOrder(this.viewNode);
	}
	if (p.hiddenColumns != null && p.hiddenColumns[p.columnNames[childElement.role]] != null)
		cellNode.style.display='none';
};

Row.prototype.deleteMe = function()
{
	DisplayNode.prototype.deleteMe.call(this);
	if (this.parent && this.parent.selectedRow == this)
		this.parent.selectedRow = null;
};// file: 130-Popup.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
//Popup - opens a pop-up window

function Popup() {};
Popup.prototype = new DisplayNode();
Popup.prototype.constructor = Popup;
Popup.prototype.create = function create_popup()
{
	alert('Popup '+this.element.role+' ('+this.modelId+') created');
	ikkk();
};
Popup.WAITING_TO_POP_UP = 'Waiting to pop-up';
Popup.POPPED_UP = 'Popped-up';
Popup.prototype.isPopup = true;
Popup.prototype.start = function()
{
	this.popupStatus = Popup.WAITING_TO_POP_UP;
	tersus.popupNodes[this.getNodeId()]=this;
	this.pause(); // Processing will start once the pop up loads
	this.currentWindow = this.createWindow();
	tersus.registerWindow(this.currentWindow);
	if (! this.currentWindow)
	{
		alert('Your browser blocks pop-up windows.\nTo use this application, please configure the browser to allow this site to open pop-up windows.');
		return;
	}
	this.currentWindow.focus();
}
Popup.prototype.findById = function popup_FindById(id, alreadyScanned)
{
	//Traverse the popup only if its window is open
	if (this.currentWindow && ! this.currentWindow.closed)
		return tersus.Node.prototype.findById.call(this,id, alreadyScanned);
	else
		return null;
};

// resume() is overriden to support the initial pause while waiting for the popup to pop)
Popup.prototype.resume = function resumePopup()
{
	if (this.popupStatus == Popup.WAITING_TO_POP_UP)
	{
		this.popupStatus = Popup.POPPED_UP;
		this.currentWindow.currentRootDisplayNode = this;
		this.currentWindow.focus();
		tersus.createEventDispatchers(this.currentWindow);
		this.setViewNode();
		this.createChildren();
		FlowNode.prototype.start.call(this);
		var node = this;
		this.currentWindow.setTimeout(function(){node.focusOnFirstField();}, 100);
	}
	else
	{
		this.runLoop();
	}
};
Popup.prototype.getPopupTitle = function getPopupTitle()
{
	return this.getCaption();
};

Popup.prototype.setTitle = function()
{
	try
	{
		if (this.currentWindow && ! this.currentWindow.closed)
		{
			this.currentWindow.document.title = this.getPopupTitle();
			if (this.header)
				this.header.innerHTML = tersus.escapeHTML(this.getPopupTitle());
		}	
	}
	catch (e)
	{
	}
};
Popup.prototype.captionChanged = Popup.prototype.setTitle;
Popup.prototype.setViewNode = function setViewNode()
{
	this.viewNode = this.currentWindow.document.getElementById('dialogBody');	
	if (this.viewNode)
	{
		this.header = this.currentWindow.document.getElementById('dialogHeader');
		this.footer = this.currentWindow.document.getElementById('dialogFooter');
	}
	else
	{
		this.viewNode = this.currentWindow.document.getElementById('root');	
		if (! this.viewNode)
			this.viewNode = this.currentWindow.document.body;
	}
	this.setTitle();
	var nodeId = this.getNodeId();
	this.isOpen = true;
	this.onResize();
};
Popup.prototype.templateURL = document.compatMode == 'BackCompat' ? 'popup.quirksmode.html' : 'popup.html';
Popup.prototype.createWindow = function Popup_createWindow(){
	if (this.sprop('templateURL'))
		this.templateURL = this.sprop('templateURL');
	
	var options = this.sharedProperties.windowOptions;
	if (! options)
		options = 'toolbar=no,location=no,status=no,menubar=no,scrollbars=yes,resizable=yes';
	var w = this.sharedProperties.windowWidth;
	if ( ! w || w == ' ')
		w = 350;
	var h = this.sharedProperties.windowHeight;
	if (! h || h == ' ')
		h = 265;
	var leftPx = parseInt((screen.availWidth-w-10)/2);
	var topPx = parseInt((screen.availHeight-h-30)/2);
	if (leftPx<0) leftPx=0;
	if (topPx<0) topPx=0;
	var targetWindow = this.getTargetWindow();
	var skeletonURL = tersus.rootURL+this.templateURL+'?nodeId='+this.getNodeId();
	this.currentWindow.mainTersusWindow = this.mainWindow;
	var popup = this.currentWindow.open(skeletonURL,targetWindow,"top="+topPx+",left="+leftPx+",width=" + w + ",height=" + h+","+options); 
	return popup;
};

Popup.prototype.getTargetWindow = function getTargetWindow()
{
	//TODO support mutltipe popup windows
	if (this.sharedProperties.targetWindow && this.sharedProperties.targetWindow!=' ')
		return this.sharedProperties.targetWindow;
	else if (this.currentWindow == this.mainWindow)
		return "popup";
	else
		return this.currentWindow.name+"_popup";
};
Popup.prototype.destroy = function()
{
	if (! this.isDestroyed)
	{
		this.isDestroyed = true;
		DisplayNode.prototype.destroy.call(this);
		this.doClose();
	}
};
Popup.prototype.closeWindow = function() 
{
	this.destroyHierarchy();
};
Popup.prototype.doClose = function ()
{
	delete tersus.popupNodes[this.getNodeId()];
	tersus.unregisterWindow(this.currentWindow);
	if (this.currentWindow && ! this.currentWindow.closed)
		this.currentWindow.close();
	this.isOpen = false;
	this.viewNode = null;
	this.footer = null;
	this.header = null;
};
Popup.prototype.onResize = function()
{
	if (! (this.isOpen && this.currentWindow && ! this.currentWindow.closed))
		return;
	var d = this.viewNode.ownerDocument;
	var s = getViewportSize(d);
	if (this.lastSize && this.lastSize.height == s.height && this.lastSize.width == s.width)
		return; // no change in size - nothing to do
	this.lastSize = s;
	var height = s.height;	 //client height of body
	var delta = d.getElementById('root').offsetHeight - this.viewNode.clientHeight;

	//Because clientHeight includes padding, but style.height doesn't include padding, we need to add padding to delta
	delta += getMeasure(this.viewNode,'padding-bottom');
	delta += getMeasure(this.viewNode,'padding-top');

	
	
	//Similarly, we need to remove body padding from the available height
	height -= parseInt(getActualStyle(d.body,'margin-bottom'));
	height -= parseInt(getActualStyle(d.body,'margin-top'));
//	height-=1;
	//alert(s.height+' ;'+height);
	var newHeight = height-delta;
	if (newHeight>0 && newHeight < 3000)
	{
		this.viewNode.style.height = newHeight+'px';
	}
	this.onResizeChildren();
};
Popup.prototype.getValidationContext = function Popup_getValidationContext()
{
	return this;
};
Popup.prototype.computedReadOnly = function Popup_computedReadOnly()
{
	return this.explicitReadOnly;
};
Popup.prototype.parentReadOnlyChanged = function Popup_parentReadOnlyChanged(readOnly)
{
};
// file: 131-Popin.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.PopIn = function tersus_PopIn(){};
tersus.PopIn.prototype = new Popup();
tersus.PopIn.prototype.constructor = tersus.PopIn;
tersus.PopIn.prototype.isPopIn = true;
tersus.PopIn.prototype.templateURL='popin.html';
tersus.PopIn.prototype.onload = function()
{
	Popup.prototype.onload.call(this);
	if (this.sprop('templateURL'))
		this.templateURL = this.sprop('templateURL');
	tersus.repository.loadHTMLTemplate(this.templateURL);
};

tersus.PopIn.prototype.create = function () {
	alert('Pop-in created');
}
tersus.PopIn.prototype.getCurrentRootViewNode = function()
{
	var root;
	var doc = this.currentWindow.document;
	root = doc.getElementById('root');
	if (! root)
		root = doc.getElementById('tersus.content');
	return root;
};
tersus.PopIn.prototype.createViewNode = function() 
{
	var w = this.currentWindow;
	var doc = w.document;
	clearSelection(w);
	this.originalTitle = doc.title;
	this.originalRoot = this.getCurrentRootViewNode();
	this.originalRootDisplayNode = w.currentRootDisplayNode;
	w.currentRootDisplayNode = this;
	var template = tersus.repository.getHTMLTemplate(this.templateURL);
	var newRoot = importNode(doc,template);
	newRoot.id = 'root';
	this.originalRoot.parentNode.replaceChild(newRoot, this.originalRoot);
	this.originalBodyStyle = doc.body.className;
	if (this.originalBodyStyle == '')
		doc.body.className = 'popin';
	else if (! this.originalBodyStyle.match(/popin/))
		doc.body.className='popin '+this.originalBodyStyle;
	this.setViewNode();
};
tersus.PopIn.prototype.disableNavigation = function()
{
	this.previousFixedURL  = tersus.history.fixedURL;
	if (this.sharedProperties.disableNavigation != false)
	{
		tersus.history.fixedURL = window.location.href;
	}
	else
		tersus.history.fixedURL = null;
	
	
};

tersus.PopIn.prototype.resumeNavigation = function()
{
	tersus.history.fixedURL = this.previousFixedURL;
};

tersus.PopIn.prototype.doClose = function()
{
	delete tersus.popupNodes[this.getNodeId()];
	
	if (this.isOpen)
	{
		this.resumeNavigation();
		var doc = this.currentWindow.document;
		var currentRoot = doc.getElementById('root');
		currentRoot.parentNode.replaceChild(this.originalRoot, currentRoot);
		doc.title = this.originalTitle;
		doc.body.className=this.originalBodyStyle;
		this.currentWindow.currentRootDisplayNode = this.originalRootDisplayNode;
		this.viewNode = null;
		this.footer = null;
		this.header = null;
		
	}
	this.isOpen = false;
};

tersus.PopIn.prototype.start = function()
{
	tersus.popupNodes[this.getNodeId()]=this;
	this.disableNavigation();
	this.createViewNode();
	this.initViewNode();
	if (!this.initElement)
		this.createChildren();
	this.isOpen = true;
	FlowNode.prototype.start.call(this);
	var node = this;
	if (tersus.isIPhone)
		window.scrollTo(0,1);
	var activeElement = this.currentWindow.document.activeElement;

	// Remove focus to prevent selection from "smearing" into popin.  Do not blur BODY as in IE7 it causes the browser window to be pushed back
	if (activeElement && activeElement.blur && activeElement.tagName != 'BODY')
		activeElement.blur();
	this.currentWindow.setTimeout(function(){node.focusOnFirstField();}, 100);
	
};
tersus.PopIn.prototype.findById = function tersus_popIn_FindById(id, alreadyScanned)
{
	//Traverse the popup only if its window is open
	if (this.isOpen)
		return tersus.Node.prototype.findById.call(this,id, alreadyScanned);
	else
		return null;
};
// file: 140-Pane.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function Pane() {};
Pane.prototype = new DisplayNode();
Pane.prototype.constructor = Pane;
Pane.prototype.defaultTag = 'div';
Pane.prototype.minHeight = 50;
Pane.prototype.setHeight = function(newHeight)
{
	this.viewNode.style.height = newHeight;
};
Pane.prototype.adjustBottom = function()
{
	var d = this.viewNode.ownerDocument;
	var body = this.viewNode.ownerDocument.body;
	var height = getVisibleSize(body).height;
	var margins = parseInt(getActualStyle(body,'margin-top'))+parseInt(getActualStyle(body,'margin-bottom'));
	height-=margins;

	var currentHeight = getVisibleSize(this.viewNode).height;
	var currentBottom = getOffsets(this.viewNode).top + currentHeight;
	if (window.browser != 'IE')
	{
		var paddingBottom = parseInt(getActualStyle(this.viewNode,'padding-bottom'));
		var paddingTop = parseInt(getActualStyle(this.viewNode,'padding-top'));
		currentBottom += paddingBottom + paddingTop;
	}
	
	var bottomMargin = tersus.isNumber(this.sharedProperties.adjustBottomMargin) ? parseInt(this.sharedProperties.adjustBottomMargin) :0;
	var delta = height-currentBottom - bottomMargin;
	
	if (delta+currentHeight >= this.minHeight)
		this.adjustHeight(delta);
};
Pane.prototype.adjustHeight = function(delta)
{
	tersus.adjustHeight(this.viewNode, delta);
};

Pane.prototype.setProperties = function()
{
	DisplayNode.prototype.setProperties.call(this);
	var pane = this;
	if (this.sharedProperties.adjustBottom == true)
		tersus.async.exec(function() {pane.onResize();}, false /* don't wait */);
};
Pane.prototype.onResize = function()
{
	if (this.sharedProperties.adjustBottom == true)
		this.adjustBottom();
	this.onResizeChildren();
};// file: 150-DisplayGroup.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.DisplayGroup = function () {};
tersus.DisplayGroup.prototype = new DisplayNode();
tersus.DisplayGroup.prototype.constructor = tersus.DisplayGroup;
tersus.DisplayGroup.prototype.isDisplayGroup = true;
tersus.DisplayGroup.prototype.createViewNode = function ()
{
	
};
tersus.DisplayGroup.prototype.initViewNode = function ()
{
	
};
tersus.DisplayGroup.prototype.attachViewNode = function ()
{
	this.viewNode = this.parent.viewNode;
};
tersus.DisplayGroup.prototype.removeChildViewNode = function (child)
{
	if (this.parent)
		this.parent.removeChildViewNode(child);
};
tersus.DisplayGroup.prototype.addChildHTMLPrefix = function (htmlV, child)
{
	if (this.parent)
		this.parent.addChildHTMLPrefix(htmlV,child);
};
tersus.DisplayGroup.prototype.addChildHTMLSuffix = function (htmlV, child)
{
	if (this.parent)
		this.parent.addChildHTMLSuffix(htmlV,child);
}
tersus.DisplayGroup.prototype.appendChildViewNode = function (child)
{
	var displayOrder = this.element.displayOrder + child.viewNode.getAttribute('display_order')*0.01;
	child.viewNode.setAttribute('display_order',displayOrder);
	this.parent.appendChildViewNode(child,this.element);
	child.refreshVisibility();
};
tersus.DisplayGroup.prototype.refreshVisibility = function ()
{
	this.forDisplayChildren( function(child) {child.refreshVisibility();});
};

tersus.DisplayGroup.prototype.isGroup = true;
tersus.DisplayGroup.prototype.onDelete = function()
{
	this.forDisplayChildren( function(child) {child.onDelete();});
};
tersus.DisplayGroup.prototype.applyColOrder = function(row)
{
	if (this.parent.applyColOrder)
		this.parent.applyColOrder(row);
};
// file: 160-PushButton.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.PushButton = function (){};
tersus.PushButton.prototype = new DisplayNode();
tersus.PushButton.prototype.defaultTag = 'span';
tersus.PushButton.prototype.getButtonName = function ()
{
	return null;
};
tersus.PushButton.prototype.initViewNode = function tersus_PushButton_initViewNode()
{
    DisplayNode.prototype.initViewNode.call(this);
    this.updateSelfReadOnly(this.computedReadOnly());
};

tersus.PushButton.prototype.createViewNode = function ()
{
    var name = this.getButtonName();
    var $button=$('<input>').attr('type',this.buttonType).addClass(this.buttonType);
    if (name != null)
    	$button.attr('name',name);
    this.button = $button[0];
    if (this.getElement('<Caption>'))
    {
    	DisplayNode.prototype.createViewNode.call(this);
    	this.viewNode.appendChild(this.button);
    	this.label = this.createHTMLElement('span',this.buttonType+'Label');
    	this.label.innerHTML = tersus.escapeHTML(this.getCaption());
    	this.viewNode.appendChild(this.label);
    }
    else
    	this.viewNode = this.button;
};
tersus.PushButton.prototype.getChecked = tersus.PushButton.prototype['get<Checked>'] = function ()
{
	if (this.viewNode)
		return this.button.checked;
};
tersus.PushButton.prototype.setChecked = tersus.PushButton.prototype['set<Checked>'] = function (value)
{
	if (this.viewNode)
		this.button.checked = value;
	if (window.browser=='IE')
	{
		// In IE 6.0, setting 'checked' before the button is rendered 
		// has no visual effect
		var button = this.button;
		tersus.async.exec(function(){ button.checked = value;}, false /* don't wait */);
	}
};
tersus.PushButton.prototype['remove<Checked>'] = function ()
{
	if (this.viewNode)
		this.button.checked = false;
};

tersus.PushButton.prototype.getClickTarget = function()
{
	return this.button;
};
tersus.PushButton.prototype.setEventHandlers = function()
{
	this.registerEventHandler(Events.ON_CLICK, this.button);
	if (this.label)
		this.registerEventHandler(Events.ON_CLICK, this.label);
};

tersus.PushButton.prototype.onClick = function(domNode, callback)
{
	if (domNode == this.button)
	{
		DisplayNode.prototype.onClick.call(this, domNode, callback);
		this.onChange(domNode, callback);
	}
	else if (domNode == this.label)
		this.labelClicked(domNode, callback);
};

tersus.PushButton.prototype.captionChanged = function()
{
	if (this.label)
		this.label.innerHTML = tersus.escapeHTML(this.getCaption());
};
tersus.PushButton.prototype.updateSelfReadOnly = function tersus_PushButton_updateSelfOnly(readOnly)
{
	if (this.currentReadOnly == readOnly)
		return; // no change
	this.currentReadOnly = readOnly;
	if (this.button)
	{
		if (readOnly)
		{
			this.button.disabled = true;
		}
		else
			this.button.disabled = false;
	}
};
// file: 170-SplitPane.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function SplitPane() {};
SplitPane.prototype = new Pane();
SplitPane.prototype.constructor = SplitPane;
SplitPane.prototype.defaultTag = 'div';
SplitPane.prototype.defaultStyleClass = 'splitPane';
SplitPane.prototype.height=400;
SplitPane.prototype.minWidth=50;
SplitPane.prototype.leftWidth = 200;
SplitPane.prototype.borderWidth=1;
SplitPane.prototype.paddingWidth=5;
SplitPane.prototype.dividerThickness=5;
SplitPane.prototype.contentPaneStyle = "splitPaneContent";
SplitPane.prototype.dividerStyle = "splitPaneDivider";
SplitPane.prototype.setDimensions = function()
{
	if (this.sharedProperties.height)
		this.height =parseInt(this.sharedProperties.height);
	this.setHeight(this.height);
	this.divider.style.width = this.dividerThickness + 'px';
	if (this.sharedProperties.leftPaneWidth)
	{
		this.leftWidth=parseInt(this.sharedProperties.leftPaneWidth);
	}
	
};
SplitPane.prototype.setHeight = function(height)
{
	this.height = height;
	this.viewNode.style.height = '';
	this.pane1.style.height = '';
	this.pane2.style.height = '';
	this.viewNode.style.height = this.height+ 'px';
	this.setPaneHeight(this.pane1,this.height);
	this.setPaneHeight(this.pane2,this.height);
	this.divider.style.height = this.height+ 'px';
};
SplitPane.prototype.createViewNode = function createViewNode()
{
	DisplayNode.prototype.createViewNode.call(this);
	if (this.sharedProperties.paddingWidth)
		this.paddingWidth =parseInt(this.sharedProperties.paddingWidth);
	if (this.sharedProperties.borderWidth)
		this.borderWidth =parseInt(this.sharedProperties.borderWidth);
	this.viewNode.style.borderWidth = this.borderWidth;
	
	this.pane1 = this.createHTMLElement('div');
	this.pane2 = this.createHTMLElement('div');
	this.divider = this.createHTMLElement('div');
	this.direction = getActualStyle(this.currentWindow.document.body, 'direction');
	this.setPaneProperties(this.pane1);
	this.setPaneProperties(this.pane2);
	this.divider.className = this.dividerStyle;
	if (this.direction == 'rtl')
		this.divider.className+=' rtl';
	
	this.divider.setAttribute('SplitPaneId', this.getNodeId());
	this.setDimensions();
	this.attachContent();
	var splitPane = this;
	this.divider.onmousedown = SplitPane.beginDrag;
	tersus.async.exec( function() {splitPane.onResize();}, false /* don't wait */);
};

SplitPane.prototype.reset = function reset()
{
	DisplayNode.prototype.reset.call(this);
	this.pane1 = null;
	this.pane2 = null;
	this.viewNode.innerHTML='';
};
SplitPane.prototype.appendChildViewNode = function appendChildViewNode(child)
{
	if (! this.leftPaneContent)
	{
		this.leftPaneContent = child.viewNode;
		this.pane1.appendChild(this.leftPaneContent);
	}
	else if (! this.rightPaneConent)
	{
		this.rightPaneContent = child.viewNode;
		this.pane2.appendChild(this.rightPaneContent);
	}
	else
		modelError('More than 2 child nodes in '+this.modelId + ' (path = '+this.getPath()+')');
	
};
SplitPane.prototype.setPaneProperties = function setPaneProperties(pane, width)
{
		pane.className=this.contentPaneStyle;
		if (this.direction == 'rtl')
			pane.className+=' rtl';
		
		pane.style.padding = this.paddingWidth + 'px';

};
SplitPane.prototype.detachContent = function detachContent()
{
	if (this.contentAttached)
	{
		this.contentAttached = false;
		if (this.pane1.parentNode == this.viewNode)
			this.viewNode.removeChild(this.pane1);
		if (this.divider.parentNode == this.viewNode)
			this.viewNode.removeChild(this.divider);
		if (this.pane2.parentNode == this.viewNode)
			this.viewNode.removeChild(this.pane2);
	}
};
SplitPane.prototype.attachContent = function attachContent()
{
	if (! this.contentAttached)
	{
		this.contentAttached = true;
		this.viewNode.appendChild(this.pane1);
		this.viewNode.appendChild(this.divider);
		this.viewNode.appendChild(this.pane2);
	}
};

SplitPane.prototype.onResize = function onResize()
{
	if (this.contentAttached && ! this.resizing && this.viewNode.offsetWidth > 0)
	{
		this.resizing = true;
		if (this.sharedProperties.height)
			this.setHeight(parseInt(this.sharedProperties.height));
		else
		{
			var node = this.viewNode.parentNode;
			while (node && node.style)
			{
				var parentHeight = node.clientHeight;
				if (parentHeight)
				{
					this.setHeight(parentHeight-2*this.borderWidth);
					break;
				}
				node = node.parentNode;
			}
		}
		var parentWidth = 0;
		
		// Detach content so that the parent width is exactly the available width
		this.detachContent();
		// Clear explicit widths - to be recalculated
		this.viewNode.style.width=null;
		this.pane1.style.width=null;
		this.pane2.style.width = null;
		// Calculating the available width is browser specific
			parentWidth=this.viewNode.offsetWidth;
		if (window.browser == 'Mozilla')
		{
			//In mozilla (as of 2005), we need to set the width property of the parent div in order to make the initial rendering correct
			this.viewNode.style.width=(parentWidth-2*this.borderWidth)+'px';
		}
		
		this.availableWidth = parentWidth - 2*this.borderWidth-this.dividerThickness;
		
		if (this.availableWidth <2*this.minWidth)
		{
			this.leftWidth = Math.round(this.availableWidth/2);
		}
		else if (this.leftWidth + this.minWidth > this.availableWidth)
		{
			this.leftWidth = this.availableWidth - this.minWidth;
		}
		//set explicit widths
		this.rightWidth= this.availableWidth - this.leftWidth;
		this.setPaneWidth(this.pane1, this.leftWidth);
		this.setPaneWidth(this.pane2, this.rightWidth);
		if (this.sharedProperties.adjustBottom == true)
		{
			this.adjustBottom();
		};
		//re-attach content
		this.attachContent();
		this.onResizeChildren();
		this.resizing = false;
	}
};

SplitPane.prototype.setPaneWidth = function setPaneWidth(pane,width)
{
		if (window.browser == 'IE' && ieLevel<7) // Traditional box model - width includes padding 
		{
			pane.style.width=width+'px';
		}
		else
		{
			pane.style.width=(width-2*this.paddingWidth)+'px';
		}
};

SplitPane.prototype.setPaneHeight = function setPaneHeight(pane,height)
{
		if (window.browser == 'IE' && ieLevel <7 )
		{
			// In IE, the hight include padding
			pane.style.height=height + 'px';
		}
		else
		{
			pane.style.height=(height -2*this.paddingWidth)+'px';
		}
};
SplitPane.beginDrag = function(e)
{
	var splitPane = tersus.findNode(this.getAttribute('SplitPaneId'));
	splitPane.beginDrag(e);
};
SplitPane.finishDrag = function (e)
{
	var splitPane=tersus.findNode(this.documentElement.getAttribute('SplitPaneId'));
	this.onmouseup = null;
	this.onmousemove = null;
	splitPane.viewNode.className=splitPane.baseClassName;
	
};
SplitPane.performDrag = function (e)
{
	var splitPane=tersus.findNode(this.documentElement.getAttribute('SplitPaneId'));
	var win = splitPane.currentWindow;
	var currentPosition = getEventPosition(win,e);
	var delta = {};
	delta.x = currentPosition.x - splitPane.lastPosition.x;
	delta.y = currentPosition.y - splitPane.lastPosition.y;
	splitPane.lastPosition = currentPosition;		
	splitPane.resize(delta);
};
SplitPane.prototype.beginDrag = function beginDrag(e)
{
	var splitPane = this;
	var win = this.currentWindow;
	var doc = win.document;
	doc.documentElement.setAttribute('SplitPaneId', this.getNodeId());
	splitPane.lastPosition = getEventPosition(win, e);
	splitPane.viewNode.className+=' resizing';
	doc.onmousemove =SplitPane.performDrag;
	doc.onmouseup=SplitPane.finishDrag;
};

SplitPane.prototype.resize = function resize(delta)
{
	if (this.leftWidth+delta.x >= this.minWidth && this.rightWidth-delta.x >= this.minWidth)
	{
		if (this.direction == 'ltr')
		{
			this.leftWidth += delta.x;
			this.rightWidth -= delta.x;
		}
		else
		{
			this.leftWidth -= delta.x;
			this.rightWidth += delta.x;
		}
			
		if (delta.x >0)
		{
			this.setPaneWidth(this.pane2, this.rightWidth);
			this.setPaneWidth(this.pane1, this.leftWidth);
		}
		else
		{
			this.setPaneWidth(this.pane1, this.leftWidth);
			this.setPaneWidth(this.pane2, this.rightWidth);
		}
		
		this.onResizeChildren();
		
	}
};
// file: 180-CallJavaApplet.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
tersus.appletName = 'tersus.applets.Base';
tersus.appletArchives = 'applets.jar';
tersus.CallJavaApplet = ActionNode.__subclass('CallJavaApplet');
tersus.CallJavaApplet.separateJVM=true;
tersus.CallJavaApplet.prototype.start = function()
{
	var appletName = this.getLeafChild('<Applet Class Name>');
	if (appletName == null)
		appletName = tersus.appletName;
	var archives = this.getLeafChild('<Archives>');
	if (archives == null)
		archives = tersus.appletArchives;
	var methodName = this.getLeafChild('<Method Name>');
	var args = this.prepareArguments();
	this.separateJVM = this.getLeafChild('<Separate JVM>');
	if (this.separateJVM == null)
		this.separateJVM = tersus.CallJavaApplet.separateJVM;
	
	this.waitForApplet(appletName,methodName,args, archives);
};
tersus.CallJavaApplet.prototype.waitForApplet = function waitForApplet(appletName,methodName,args, archives)
{
	if (document.applets[appletName])
	{
		var output =  this.callApplet(appletName,methodName, args);
		if (output == '__ASYNC')
			this.pause();
		else 
		{
			if (output != null)
				this.handleOutput(output);
			delete FlowNode.map[this.nodeId];
		}
			
	}
	else
	{
		this.pause();
		var rootContext = (this.getExecutionContext() == window.rootDisplayNode);
		if (rootContext)
		{
			tersus.progress.set('Loading applet '+appletName);
		}
		var node = this;
		setTimeout(function()
		{
			node.createApplet(appletName,archives);
			node.startTime = new Date();
			node.called=false;
			node.timerId = setInterval(function ()
			{
				var active = false;
				
				try 
				{
					active = document.applets[appletName] && document.applets[appletName].isActive();
				}
				catch ( e)
				{
					active = false;
				}
					
				if (((new Date())-node.startTime) > 60000 || active)
				{
					clearInterval(node.timerId);
					if (! node.called) // Prevent duplicate calling
					{
						node.called=true;
						var output = node.callApplet(appletName,methodName, args);
						if (output != '__ASYNC')
							node.handleResponse(output);
					}
					// Note that sometimes the timer is invoked one extra time to clearing the timer is not enough
				} 
			},100);
		},10);
	}
};
tersus.CallJavaApplet.prototype.createApplet = function(appletName,archives)
{
	if (!archives)
		archives = '';
	var applet = document.createElement('applet');
	applet.codebase='.';
	applet.width=1;
	applet.height=1;
	applet.setAttribute('id',appletName);
	applet.setAttribute('name',appletName);
	applet.setAttribute('mayscript','true');
	applet.setAttribute('separate_jvm',this.separateJVM);
	applet.code=appletName;
	applet.archive=archives;
	var c = document.getElementById('applet_container');
	if (!c)
	{
		c = document.createElement('div');
		c.id='applet_container';
		document.body.appendChild(c);
	}
	c.appendChild(applet);
};

tersus.CallJavaApplet.prototype.prepareArguments = function()
{
	var st = this.triggers.concat([]);
	var args = [];
	st.sort(function(a,b){ if (a.role < b.role) return -1; else if (a.role == b.role) return 0; else return 1;});
	for (var i=0;i<st.length;i++)
	{
		var t = st[i];
		if (t.role.charAt(0) != '<' && t.modelId != BuiltinModels.NOTHING_ID  )
		{
			if (t.isRepetitive)
			{
				var a = [];
				var l =t.getChildren(this);
				if (l)
				{
					for (var j=0;j<l.length; j++)
					{
						var v = l[j];
						if (v!= null &&  v.leafValue != null)
							a.push(v.leafValue);
						 else
						  	a.push(v);
					}
				}
				args.push(a);
			}
			else
			{
				var v = t.getChild(this);
				if (v!= null && v.leafValue != null)
					args.push(v.leafValue);
				else
					args.push(v);
			}
		}
	}	
	return args;
};
tersus.CallJavaApplet.prototype.callApplet = function(appletName,methodName,args)
{
	var rootContext = (this.getExecutionContext() == window.rootDisplayNode);
	if (! document.applets[appletName])
	{
		if (rootContext)
			tersus.progress.clear();
		modelExecutionError('Applet '+appletName + ' not loaded',this);
		return;
	}
	var applet = document.applets[appletName];
	if (rootContext)
		tersus.progress.set("Calling "+methodName);
	window.tersus_CallJavaApplet_caller = this;
	window.tersus_CallJavaApplet_caller_id = this.getNodeId();
	if (window.browser == 'IE')
	{
		var invocation = 'applet.'+methodName+'(';
		for (i=0;i<args.length;i++)
		{
			if (i>0)
				invocation += ',';
			invocation+='args['+i+']';
		}
		invocation += ')';
		var output = eval(invocation);
	}
	else
		var output = applet[methodName].apply(applet,args);
	window.tersus_CallJavaApplet_caller = null;
	window.tersus_CallJavaApplet_caller_id = null;
	
	return output;
	
};
tersus.CallJavaApplet.prototype.isJavaException = function (e)
{
	if ( typeof e === 'object' &&  'getMessage' in e && 'getClass' in e )
	{
		var n = e.getClass().getName()+'';
		return n.match(/Exception|Error/) ? true: false;
	}
	else
		return false;
};
tersus_handle_applet_response = function (nodeId, output, exitName)
{
	if (tlog)
		tlog('tersus_handle_applet_response id='+nodeId);
	var node = tersus.findNode(nodeId);
	if (!node)
	{
		alert('Applet node #'+nodeId+ ' not found');
		return;
	}
	if (tlog)
		tlog('Processing applet response for '+node.modelName+' ['+nodeId+']');
	node.handleResponse(output,exitName);
};
tersus_handle_applet_error = function (nodeId, error)
{
	error = ''+error; //convert to JS string
	if (tlog)
		tlog('tersus_handle_applet_error id='+nodeId);
	var node = tersus.findNode(nodeId);
	if (!node)
	{
		alert('Applet node #'+nodeId+ ' not found');
		return;
	}
	if (tlog)
		tlog('Processing applet response for '+node.modelName+' ['+nodeId+']');
	
	if (error.match(/^{/))
		error = eval('var e='+error+';e;'); //TODO use string JSON parse
	node.appletError = error;
	node.handleResponse(null,null);
};
tersus_charge_applet_leaf_exit = function (nodeId, exitName, value)
{
	var node = tersus.findNode(nodeId);
	if (!node)
	{
		alert('Applet node #'+nodeId+ ' not found');
		return;
	}
	node.chargeLeafExit(exitName, value);
};
tersus.CallJavaApplet.prototype.handleOutput = function(output, exitName)
{
	if (!exitName)
		exitName = '<Output>';
	if (output != null)
		this.chargeLeafExit(exitName,output);	
}
tersus.CallJavaApplet.prototype.handleResponse = function(output,exitName)
{
	var executionContext = this.getExecutionContext();
	if (executionContext == window.rootDisplayNode)
		tersus.progress.set('Processing response');
	if (output)
	{
		try
		{
			if (this.isJavaException(output))
			{
				this.appletError = output;
			}
			else
			{
				this.handleOutput(output,exitName);
			}
		}
		catch (e)
		{
        	if (!this.appletError)
        		this.appletError = e;
        }
	}
	this.continueExecution();

		
	this.destroy(); // Important - removes node from the node map
	this.children=null; //To be on the safe side in case an applet holds a reference to the node
	this.parent = null; //To be on the safe side ..	
};
tersus.CallJavaApplet.prototype.resume = function()
{
	if (this.appletError)
		throw this.appletError;
}
// file: 180-ScrollPane.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function ScrollPane() {};
ScrollPane.prototype = new Pane();
ScrollPane.prototype.constructor = ScrollPane;
ScrollPane.prototype.defaultStyleClass = 'scrollPane';
// file: 181-ScrollPane.iphone.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at http://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
ScrollPane.prototype = 
function Scrollable() {};
ScrollPane.prototype = new tersus.HTMLTag();
ScrollPane.prototype.constructor = ScrollPane;
ScrollPane.prototype.defaultTag = 'div';
ScrollPane.prototype.defaultClass  = 'scrollPane';
ScrollPane.prototype.defaultWrapperTags='div';
ScrollPane.prototype.defaultWrapperStyleClasses='scrollPaneWrapper';


ScrollPane.preventDefaultSet = false;

ScrollPane.prototype.onResize = function()
{
	this.setDefaultHeight();
	if (this.iScroll)
		this.iScroll.refresh();
	this.onResizeChildren();
}
ScrollPane.prototype.initIScroll = function()
{
	var node = this;

	tersus.async.exec(function()
	{
		node.setDefaultHeight();
		node.iScroll = new iScroll(node.wrapperNode);
	});
};
ScrollPane.prototype.destroy = function()
{
	if (this.iScroll && this.iScroll.destory)
	{
		this.iScroll.destory(true);
		this.iScroll=null;
	}
	tersus.HTMLTag.prototype.destroy.call(this);
};
ScrollPane.prototype.setDefaultHeight = function()
{
	var e = $(this.wrapperNode);
	while (e.length>0)
	{
		if (e.css('overflow-y')=='hidden')
		{
			$(this.wrapperNode).height(e.height());
			return;
		}
		e = e.parent();
	}
	this.iScroll.refresh();
};
ScrollPane.prototype.copyValueOpt = function(value, nodeMap, htmlV, legacyNodes)
{
	tersus.HTMLTag.prototype.copyValueOpt.call(this,value, nodeMap, htmlV, legacyNodes)
	var node = this;
	if (window.iScroll)tersus.async.exec(function(){tersus.async.exec(function() {node.initIScroll();});});
};

// file: 181-ShowFile.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.ShowFile = function ShowFile() {};
tersus.ShowFile.useIFrame=true;
tersus.ShowFile.prototype = new ServiceNode();
tersus.ShowFile.prototype.constructor = tersus.ShowFile;
tersus.ShowFile.prototype.start = function startService()
{	
	if (tersus.alertMode.interactive)
	this.callService();
};
tersus.ShowFile.prototype.getTraceFileName = function()
{
	return this.traceFileName;
};
tersus.ShowFile.iframe = null;
tersus.ShowFile.prototype.callService = function callService()
{
	if (window.trace)
	{
		this.traceFileName = window.trace.openTraceFile();
		window.trace.link(this,this.traceFileName, "contains the trace of the server side process");
	}
	var form = this.prepareForm();
	this.addTersusParameters(form);

	form.action = 'File';
	form.method = 'POST';
	if (tersus.ShowFile.useIFrame)
	{
		if (tersus.ShowFile.iframe == null)
			tersus.ShowFile.iframe = createAnIFrame(window,'_showFile');
		form.target='_showFile';
	}
	
	setEncType(form,'multipart/form-data'); 
	var sigField = addHiddenField(form, '_sig', 'END');
	form.appendChild(sigField);
	this.nodesToRemove.push(sigField);
	form.submit();
	for (var i=0; i< this.nodesToRemove.length; i++)
	{
		var node = this.nodesToRemove[i];
		node.parentNode.removeChild(node);
	};
};
// file: 200-FieldList.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
tersus.Field = function Field(element, path)
{
	if (element != null)
	{
		this.name = element.role;
		this.sqlName = element.columnName ? element.columnName : tersus.sqlize(element.role);
		this.path = path;
		this.element = element;
	}
};
tersus.Field.prototype.constructor = tersus.Field;
tersus.Field.prototype.setValue = function (obj, value)
{
	obj.setValue(this.path, value, Operation.REPLACE);
};
tersus.Field.prototype.setLeafValue = function (obj, leafValue)
{
	var value = this.element.createChildInstance();
	value.setLeaf(leafValue);
	obj.setValue(this.path, value, Operation.REPLACE);
};
tersus.Field.prototype.getSQLDef = function()
{
	var childP = this.element.getChildConstructor().prototype;
	return this.sqlName+ ' ' + childP.sqlType;
};
tersus.Field.prototype.getSQLValue = function (obj)
{
	var values = obj.getValues(this.path);
	if (values && values.length == 1)
	{
		var v = values[0];
		if (v == null)
		  	return v;
		return v.isDateAndTime ? v.serialize() : v.leafValue; // SQLite doesn't support date and time - need to serialize
	}
	else 
		return null;
};

tersus.MapField = tersus.Field.__subclass('MapField', function(sqlName)
{
	this.name = this.sqlName=sqlName;
});
tersus.MapField.prototype.getSQLValue = function (obj)
{
	if (obj == null)
		return null;
	var map = obj.leafValue;
	if (map == null)
		return null;
	var v = map[this.sqlName];
	if (v == null)
		  return v;
	return v.isDateAndTime ? v.serialize() : v.leafValue; // SQLite doesn't support date and time - need to serialize
};

tersus.FieldList = function(model)
{
	this.initialize(model);
};
tersus.FieldList.prototype.constructor = tersus.FieldList;
tersus.FieldList.prototype.initialize = function(model)
{
	this.fields = [];
	this.primaryKeyFields=[];
	this.fieldsByName={};
	if (model)
	{
		this.addFields(model);
		for (var i=0; i<this.fields.length;i++)
		{
			var f = this.fields[i];
			if (f.element.isPrimaryKey)
			{
				this.primaryKeyFields.push(f);
				f.isPrimaryKey = true;
			}
		}
		if (this.primaryKeyFields.length == 0 && this.fields.length >0)
		{
			this.primaryKeyFields.push(this.fields[0]); // Todo - remove this sometime in the future?
			this.primaryKeyFields[0].isPrimaryKey=true;
		}
	}
};
tersus.FieldList.prototype.addMapFields = function(obj)
{
	if (obj == null)
		return;
	var map = obj.leafValue;
	if (map == null)
		return;
	for (var name in map)
		this.addMapField(name);
};
tersus.FieldList.prototype.getField = function getField(name)
{
	return this.fieldsByName[tersus.sqlize(name)];
};

tersus.FieldList.prototype.addMapField = function(name)
{
	if (this.getField(name))
		return;
	this.addField(new tersus.MapField(name));
}
tersus.FieldList.prototype.addField = function(f)
{
	this.fields.push(f);
	if (this.fieldsByName[f.sqlName] != null)
			modelError('Ambiguous column name '+f.sqlName, this);	
	this.fieldsByName[f.sqlName] = f;
};
tersus.FieldList.prototype.addFields = function addFields(root,prefixPath)
{	
	if (prefixPath == null)
		prefixPath = new Path('');
	var elements = root.elementList;
	if (!elements)
		return;
	var n=elements.length;
	for (var i=0;i<n;i++)
	{
		var e = elements[i];
		var c = e.getChildConstructor().prototype;
		var p = prefixPath.clone();
		p.addSegment(e.role);
		if (c.isLeaf)
			this.addField(new tersus.Field(e, p));
		else
			this.addFields(c,p);
	} 
};
// file: 201-DatabaseAction.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.databases = {};
tersus.maxDatabaseSize=5000000;
tersus.DatabaseAction = ActionNode.__subclass('tersus.DatabaseAction');
tersus.DatabaseAction.prototype.getDB = function()
{
	var datasource = this.getLeafChild('<Data Source>');
	if (!datasource)
		datasource = 'default';
	if (datasource.charAt(0) != '/')
	{
		var m = tersus.rootURL.match(/^(?:https?:\/\/[^/]+\/|file:\/+)([^/]+)\/$/);
		if (m && m.length > 0)
			datasource=m[1]+'_'+datasource;
	}
	
	var db = tersus.databases[datasource];
	if (! db)
	{
		db = openDatabase(datasource, 1.0, datasource, tersus.maxDatabaseSize);
		if (db == null) // Workaround for phonegap issue with quota update
		{
			for (var i=0;i<=tersus.maxDatabaseSize/1000000;i++)
			{
				db = openDatabase(datasource, 1.0, datasource, tersus.maxDatabaseSize);
				if (db != null)
					break;
			}
		}
		tersus.databases[datasource]=db;
	}
	return db;
};

tersus.DatabaseAction.prototype.getSQLStatememt = function()
{
	var sql = this.getLeafChild('<SQL Statement>');
	if (!sql)
		modelError('Missing <SQL Statement>',this);
	return sql;
};

tersus.DatabaseAction.prototype.analyzeParameterizedSQL = function(str)
{
	var t = str;
	var i=0;
	var l = t.length;
	var out = [];
	var keys = [];
	while (i<l&& i>= 0)
	{
		var n = t.indexOf('${',i);
		if (n == -1)
		{
			out.push(t.substring(i));
			i = n;
		}
		else
		{
			out.push(t.substring(i,n));
			var b = n+ 2;
			var e=t.indexOf('}', b);
			if (e<0)
			{
				modelExecutionError("Invalid template: missing '}' for '${'.  Position="
							+ n
							+ " Text="
							+ t.substring(n, n + 20));
				return null;
			}
			var key=t.substring(b,e);
			out.push('?');
			keys.push(key);
			i = e+1;
		}

	}
	var result = {};
	result.sqlText = out.join('');
	result.parameters = keys;
	return result;
};

tersus.DatabaseAction.prototype.getFieldList = function(model)
{
	if (!model.fieldList)
	{
		model.fieldList = new tersus.FieldList(model);
	}	
	return model.fieldList;
};
tersus.DatabaseAction.prototype.getOutputColumnsExit = function getOutputColumnsExit()
{
	return this.getExit('<Output Columns>');
};
tersus.DatabaseAction.prototype.outputColumns = function outputColumns(colMap)
{
	var exit = this.getOutputColumnsExit();
	if (exit)
	{
		for (var name in colMap)
		{
			var col = exit.createChildInstance();
			col.setLeafChild('Name',name, Operation.REPLACE);
			col.setLeafChild('Type',colMap[name], Operation.REPLACE);
			this.accumulateExitValue(exit, col);		
		}
	}
};
tersus.DatabaseAction.prototype.outputResultAsMap = function outputResultAsMap(exit, row, columns)
{
	var out = exit.createChildInstance();
	var map = {};
	out.leafValue = map;
	for (var key in row)
	{
		var value = row[key];
		if (value == null)
		{
			if (columns[key] == null)
				columns[key] = '?';
			continue;
		}
		var type =	typeof(value);
		var c;
		if (type == 'string')
		{
			c = tersus.repository.get(BuiltinModels.TEXT_ID);
		}
		else if (type == 'number')
		{
			c = tersus.repository.get(BuiltinModels.NUMBER_ID);
		}
		else
			throw {message:'Unexpected database return type '+type};
		var node = new c();
		node.prototype = c.prototype;
		node.setLeaf(value);
		map[key]=node;
		if (columns)
		{
			if (columns[key] == null || columns[key]=='?')
				columns[key] = c.prototype.modelName;
			else if (columns[key] != c.prototype.modelName)
				columns[key] = 'Anything';
		}
				
	}
	if (exit.isRepetitive)		
		this.accumulateExitValue(exit, out);
	else
		this.chargeExit(exit,out);
	
};

tersus.DatabaseAction.prototype.getTableNameOfModel = function(record)
{
	return record.tableName  ? record.tableName : tersus.sqlize(record.modelName);
};
tersus.DatabaseAction.prototype.getTableName = function getTableName(record)
{
	var n = this.getLeafChild('<Table Name>');
	if (n == null)
		n = this.getTableNameOfModel(record);
	return n;
};

tersus.DatabaseAction.prototype.addPrimaryKeyWhereClause = function createPrimaryKeyFilter(record,fieldList,sqla, values)
{
	sqla.push("WHERE");
	var pk = fieldList.primaryKeyFields;
	var npk = pk.length;
	for (var i=0;i<npk;i++)
	{
		if (i>0)
			sqla.push('AND');
		var f = pk[i];
		var v = f.getSQLValue(record);
		sqla.push(f.sqlName);
		if (v == null)
			sqla.push('IS NULL');
		else
		{
			sqla.push('= ?');
			values.push(v);
		}
	}

};
tersus.DatabaseAction.prototype.addSelectFieldsSQL = function addSelectFieldsSQL(sqla, tableName, fieldList, explicitColumns)
{
	sqla.push('SELECT');
	if (explicitColumns)
		sqla.push(explicitColumns)
	else
	{
		var fields = fieldList.fields;
		var n = fields.length;
		for (var i=0;i<n; i++)
		{
			if (i>0)
				sqla.push(',');
			sqla.push(fields[i].sqlName);
		}
	}
	sqla.push('FROM');
	sqla.push(tableName);
};
tersus.DatabaseAction.prototype.outputResults = function(results, exit, maxRows, reverse)
{
	var nRows = 0;
	var rows = results.rows;
	if (rows)
		nRows =  rows.length;
	if (nRows == 0)
	{
		this.chargeEmptyExit('<None>');
		return;
	}
	if (maxRows && nRows > maxRows)
		nRows = maxRows;
	var outputColumnsExit = this.getOutputColumnsExit();
	var outputPrototype = exit.getChildConstructor().prototype;
	if (outputPrototype.isMap)
	{
		var colMap=null;
		if (outputColumnsExit)
			colMap = {};
		for (var i=0;i<nRows; i++)
		{
			this.outputResultAsMap(exit, rows.item(i), colMap);
		}
		if (outputColumnsExit)
		{
			this.outputColumns(colMap);
		}
	}
	else
	{
		var fields = this.getFieldList(outputPrototype).fields;
		var nFields = fields.length;
		if (reverse)
		{
			for (var i=nRows -1; i>=0; i--)
				this.outputResult(rows.item(i), fields, nFields, exit);
		}
		else
		{
			for (var i=0;i<nRows; i++)
				this.outputResult(rows.item(i), fields, nFields, exit);
		}
	}
	
};
tersus.DatabaseAction.prototype.outputResult = function(item, fields, nfields, exit)
{
	var out = exit.createChildInstance();
	if (out.isLeaf)
	{
		for (var k in item)
			out.setLeaf(item[k]);
	}
	else // composite
	{
		for (var j=0;j<nfields;j++)
		{
			var field = fields[j];
			var v = item[field.sqlName];
			if (v != null)
				field.setLeafValue(out,v);
		}
	} 
	if (exit.isRepetitive)		
		this.accumulateExitValue(exit, out);
	else
		this.chargeExit(exit,out);
};

tersus.DatabaseAction.prototype.start = function()
{
	this.pause();
	if (this.tableOK)
	{
		this.run();
		return;
	}
	this.tableOK = true;
	var p = this.getTablePrototype();
	if (!p || p.tableOK || p.isMap)
	{
		this.run();
	}
	else
	{
		p.tableOK = true;
		this.getTableDef(this.getTableNameOfModel(p), this.updateTableAndRun, this);
	}
};
tersus.DatabaseAction.prototype.updateTableAndRun = function (tableDef)
{
	var p = this.getTablePrototype();
	var fieldList = this.getFieldList(p);
	var fields = fieldList.fields;
	var nfields = fields.length;
	var batch = [];
	if (tableDef == null)
	{
		var sql = 'CREATE TABLE '+this.getTableNameOfModel(p) + ' (';
		for (var i=0;i<nfields; i++)
		{
			var field = fields[i];
			if (i>0)
				sql += ', ';
			sql+=field.getSQLDef();
		}
		var pk = fieldList.primaryKeyFields;
		if (pk.length > 0)
		{
			sql += ', PRIMARY KEY (';
			for (var i=0;i<pk.length;i++)
			{
				if (i>0)
					sql += ',';
				sql += pk[i].sqlName;
			}
			sql+=')';
		}
		sql+=')';
		batch.push(sql);
	}
	else
	{
		for (var i=0;i<nfields;i++)
		{
			var field = fields[i];
			if (tableDef.columns[field.sqlName] == null)
				batch.push('ALTER TABLE '+p.tableName+' ADD COLUMN '+field.getSQLDef());
		}
	}
	if (batch.length > 0)
		this.execSqlBatch(batch, function(){this.run();});
	else
		this.run();			
};

tersus.DatabaseAction.prototype.resumeContext = function()
{
	this.continueExecution();
};
tersus.DatabaseAction.prototype.execSql = function (sql, values, callback)
{
	if (window.trace)
	{
		window.trace.sql(this, sql, values);
	}
	//alert(sql);
	var errorHandler = function(transaction,error)
	{
		alert('Error: '+error.message+' ['+error.code+']');
		return true;	
	};
	var node = this;
	var dataHandler = function(transaction, results)
	{
		callback.call(node,transaction, results);
	};
	this.getDB().transaction(function(transaction){ transaction.executeSql(sql, values, dataHandler, errorHandler);});	
};
tersus.DatabaseAction.prototype.execSqlBatch = function (commands, callback, index)
{
	if (index == null)
		index = 0;
	if (commands.length != null && index >= commands.length)
	{
		alert('Invalid index in execSqlBatch');
	}
	this.execSql(commands[index], [], function()
	{ 
		if (index+1==commands.length)
			callback.call(this);
		else
			this.execSqlBatch(commands, callback, index+1);
	});
};
tersus.DatabaseAction.prototype.getTableDef = function(tableName, callback, obj)
{
	var errorHandler = function(transaction,error)
	{
		alert('Error: '+error.message+' ['+error.code+']');
		return true;	
	};
	var dataHandler = function(transaction, results)
	{
		if (!results || !results.rows || results.rows.length == 0)
		{
			callback.call(obj,null);
			return;
		}
		
		var re0 = /create table [^(]*\((.*)\)$/i;
		var re1 = /\s*(?:(?:(\w+)(?:\s+(\w+))?(?:\s*\(\s*(\d+)\s*\)\s*)?)|(?:primary\s*key\s*\(\s*([^)]*)\s*\)))((?:\s|\w)*)(?:,|$)/ig;
		var row = results.rows.item(0);
		var tableName = row.name;
		var sql = row.sql;
		var tableSpec= re0.exec(sql)[1];
		var tableDef = {name:tableName,columns:{}};
		do
		{
			var m = re1.exec(tableSpec);
			if (m!= null)
			{
				if (m[1] != null) // Column definition
				{
					var col = {name:m[1], type:m[2], size:m[3]};
					if (m[5] && /primary\s+key/i.test(m[5]))
						col.isPrimaryKey=true;
					tableDef.columns[col.name]=col;
				}
				else if (m[4] != null) // Primary key definition
				{
					var pk = m[4].split(/\s*,\s*/);
					for (var j=0; j<pk.length;j++)
					{
						var c = pk[j];
						if (c && tableDef.columns[c])
							tableDef.columns[c].isPrimaryKey=true;
					}
				}
					
			}
		} while (m!= null);
		callback.call(obj, tableDef);
	};
	this.getDB().transaction(function(transaction) {
		transaction.executeSql("select name, sql from sqlite_master where type=? and name=?",['table', tableName],  dataHandler, errorHandler);
	});
};
tersus.DatabaseAction.prototype.getSQLValue = function getSQLValue(role)
{
	  var v = this.getChild(role);
	  if (v == null)
	  	return v;
	  return v.isDateAndTime ? v.serialize() : v.leafValue; // SQLite doesn't support date and time - need to serialize

};// file: 300-touchend-onclick.iphone.non-simulator.non-symbian.js
/*if (tersus.simulateClickByTouch != false)
{
	DisplayNode.prototype.registerEventHandler = function(eventType, domNode)
	{
		if (!domNode)
			domNode = this.viewNode;
		domNode.setAttribute('lid',this.getNodeId());
		if (!eventType.jsName)
		{
			internalError('No event dispatcher defined for '+eventType.elementName);
			return;
		}
		var dispatcher = this.currentWindow.tersus_dispatchers[eventType.jsName];
		if (dispatcher == null)
			internalError('No event dispatcher defined for '+eventType.jsName);
		else
		{
			if (eventType === Events.ON_CLICK)	
			{
				domNode[Events.ON_TOUCHEND.jsName]=dispatcher;
				domNode[Events.ON_TOUCHSTART.jsName]=tersus.noop;
			}
			else
				domNode[eventType.jsName] = dispatcher;
		}	
	};
	function getEventAttributes(e)
	{
		var atts = {};
		atts.type = e.type;
		if (e.type.substring(0,3)=='key')
		{
			if (e.keyCode)
				atts.keyCode = e.keyCode;
			else 
				atts.keyCode = e.which;
		}
		var buttonCode = e.button;
		if (buttonCode == 1)
			atts.button = 'Left';
		else if (buttonCode == 4)
			atts.button = 'Middle';
		else if (buttonCode == 2)
			atts.button = 'Right';
		if (e.changedTouches && e.changedTouches.length>0)
		{
			var t = e.changedTouches[0];
			atts.clientX  = t.clientX;  
			atts.clientY = t.clientY;
		}
		else
		{
			atts.clientX  = e.clientX;  
			atts.clientY = e.clientY;
		}
		
		atts.targetElement = e.target ? e.target : e.srcElement;
		return atts;
	};	
	
}*/
// file: 301-orientation-change.iphone.non-simulator.js
window.onorientationchange = function(ev)
{
	if (ev)
		tersus.currentEventAttributes = getEventAttributes(ev);
	if (window.rootDisplayNode)
		window.rootDisplayNode.queueEvent(Events.ON_ORIENTATION_CHANGE,{});
	
}// file: 301-orientation-change.js
tersus.currentOrientation=window.orientation;
window.onorientationchange = function(ev)
{
	if (tersus.currentOrientation === window.orientation)
		return;
	tersus.currentOrientation=window.orientation;
	if (ev)
		tersus.currentEventAttributes = getEventAttributes(ev);
	if (window.rootDisplayNode)
		window.rootDisplayNode.queueEvent(Events.ON_ORIENTATION_CHANGE,{});
	
};

tersus.simulateOrientationChange = function()
{
	if (window.orientation==0 || window.orientation==180)
		window.orientation=90;
	else
		window.orientation=0;
	tersus.invokeAction('<On Orientation Change>',null,null,window.rootDisplayNode);
};
// file: AbsoluteNumber.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('AbsoluteNumber', function()
{
	var x = this.getLeafChild('<X>');
	if (x != null)
	{
		var absValue = Math.abs(x);
		this.chargeLeafExit('<Absolute Value>', absValue);
	}
});
// file: ActionDialog.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.isIE6 = navigator.userAgent.indexOf('MSIE 6') >=0;

 //tersus.ActionDialog = function tersus_ActionDialog() {};
	tersus.ActionDialog = tersus.PopIn.__subclass('Dialog');
	tersus.ActionDialog.prototype.templateURL='actiondialog.html';
	tersus.ActionDialog.prototype.createViewNode = function()
	{
		// create mask (this gives modality by blocking UI access to existing content)
		clearSelection(this.currentWindow); // sometimes the original selection is extended to newly created elements
		var d = this.currentWindow.document;
		var r = this.getCurrentRootViewNode();
		//var ml = this.createHTMLChildElement(r,'div');
		
		// create dialog div
		var template = tersus.repository.getHTMLTemplate(this.templateURL);
		var actioDialogHtml = importNode(d,template);
		this.dialogLayer = actioDialogHtml;
		actioDialogHtml.className='confirm_screenopen';
		this.viewNode = tersus.findChildNodesByClassName(actioDialogHtml,'action_dialog_body')[0];
		this.viewNode = actioDialogHtml;
		this.isOpen = true;
		r.appendChild(actioDialogHtml);
		this.header = tersus.findChildNodesByClassName(actioDialogHtml,'action_dialog_title')[0];
		if (this.header)
		  this.header.innerHTML = tersus.escapeHTML(this.getPopupTitle());
		  
		this.onResize();
		
	};
	tersus.ActionDialog.prototype.onResize = function()
	{
		if (! (this.isOpen && this.currentWindow && ! this.currentWindow.closed && this.viewNode))
			return;
				this.onResizeChildren();
	}
	tersus.ActionDialog.prototype.doClose = function() 
	{
		delete tersus.popupNodes[this.getNodeId()];
	
		if (this.isOpen)
		{
			this.resumeNavigation();
			this.viewNode = null;
			this.header = null;
			var dl = this.dialogLayer;
			//var ml = this.maskLayer;
			this.dialogLayer=null;
			//this.maskLayer=null;
			dl.parentNode.removeChild(dl);
			//ml.parentNode.removeChild(ml);
		}
		this.isOpen = false;
	
	};
// file: AddStyleClass.js
/*********************************************************************************************************
 * Copyright (c) 2003-2011 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/

tersus.newAction('AddStyleClass',function()
{	
	var element = this.getChild('<Element>');
	var value = this.getLeafChild('<Style Class>');
	if (!element || !value)
		return;
	var elementClass = element.getLeafChild('<Style Class>');
	if (!elementClass)
		elementClass = value;
	else
	{
		var orig = ' ' + elementClass + ' ';
		var classNames = value.split(/\s+/);
		for (var i=0;i<classNames.length;i++)
		{
			var c=classNames[i];
			if (orig.indexOf(' '+c+' ') < 0)
				elementClass += ' '+c;
		}
	}
	element.setLeafChild('<Style Class>',$.trim(elementClass), Operation.REPLACE);
});
// file: AdvancedFind.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.AdvancedFind =  tersus.DatabaseAction.__subclass('tersus.AdvancedFind');
tersus.AdvancedFind.prototype.NO_N_MATCHES = 'No Number of Matches';
tersus.AdvancedFind.prototype.NO_POSITION = 'No Position';
tersus.AdvancedFind.prototype.getTablePrototype = function()
{
	var e = this.getExit('<Records>');
	if (e == null)
		e = this.getTrigger('<From>');
	if (e == null)
		e = this.getTrigger('<To>');
	if (e == null)
		e = this.getTrigger('<Start From>');
	if (e == null)
		e = this.getTrigger('<Match>');
	if (e == null)
		return null;
	return e.getChildConstructor().prototype;
};
tersus.AdvancedFind.prototype.run = function()
{
	var ctx =
	{
		condition:[],
		values:[],
		from:this.getChild('<From>'),
		to:this.getChild('<To>'),
		match: this.getChild('<Match>'),
		startFrom:this.getChild('<Start From>'),
		numberOfRecords:this.getChild('<Number of Records>'),
		offset:this.getLeafChild('<Offset>'),
		backward:(true == this.getLeafChild('<Backward>')),
		table:this.getLeafChild('<Table>'),
		columns:this.getLeafChild('<Columns>'),
		filter:this.getLeafChild('<Filter>'),
		givenOrderBy:this.getLeafChild('<Order By>'),
		recordsExit:this.getExit('<Records>'),
		columns:this.getLeafChild('<Columns>'),
		moreExit:this.getExit('<More>'),
		positionExit:this.getExit('<Position>'),
		tablePrototype:this.getTablePrototype(),
		numberOfMatchesExit:this.getExit('<Number of Matches>')
	};
	if (!ctx.offset)
		ctx.offset = 0;
	
	this.setOptimization(ctx);
	this.setFieldList(ctx);
	this.addTable(ctx);
	this.addDynamicFilterConditions(ctx);
	this.addRangeConditions(ctx);
    this.addMatchConditions(ctx);	
	this.setOrderBy(ctx, ctx.backward);
	ctx.needsMatches = ctx.numberOfMatchesExit	 && !ctx.noMatches;
	ctx.needsPosition = ctx.positionExit!=null && !ctx.noPosition;

	if (ctx.needsMatches 
	 || ctx.backward && ctx.needsPosition && ctx.startFrom == null)
    {
    	this.countMatches(ctx, function(ctx,count)
    	{
    		ctx.nMatches = count;
    		if (ctx.needsMatches)
    		{
    			this.chargeLeafExit(ctx.numberOfMatchesExit, count);
    		}
    		if (ctx.recordsExit)
    			this.executeQuery(ctx);
    		else
    			this.resumeContext();
    		
    	});
   	}
    else
  		if (ctx.recordsExit)
   			this.executeQuery(ctx);
   		else
   			this.resumeContext();
   		
};
tersus.AdvancedFind.prototype.setOptimization = function(ctx)
{
	ctx.noPosition = false;
	ctx.noMatches = false;
	var trigger = this.getTrigger('<Optimization>');
	if (!trigger)
		return;
	
	var list = trigger.getChildren(this);
    if (list)
    {
    	for (var i=0;i<list.length;i++)
    	{
    		var opt = list[i].leafValue;
    		if (opt == this.NO_POSITION)
    			ctx.noPosition=true;
    		else if (opt == this.NO_N_MATCHES)
    			ctx.noMatches = true;
    	}
    }
};
tersus.AdvancedFind.prototype.countMatches = function countMatches(ctx, callback)
{
	var sqla = ['SELECT COUNT(1) AS count FROM', ctx.tableSql];
	if (ctx.condition.length > 0)
	{
		sqla.push('WHERE');
		tersus.addAll(sqla, ctx.condition);
	}
	this.execSql(sqla.join(' '),ctx.values, function(transaction, results) 
	{
		var count = results.rows.item(0).count;
		callback.call(this, ctx, count);
	});
};
tersus.AdvancedFind.prototype.setFieldList = function setFieldList (ctx)
{
	if (ctx.tablePrototype != null)
	{
		if (ctx.tablePrototype.isMap)
		{
			ctx.fieldList = new tersus.FieldList();
			ctx.fieldList.addMapFields(ctx.from);
			ctx.fieldList.addMapFields(ctx.to);
			ctx.fieldList.addMapFields(ctx.match);
			ctx.fieldList.addMapFields(ctx.startFrom);
			if (ctx.columns != null)
			{
				tersus.foreach(ctx.columns.split(/ *, */),
				function (name)
				{
						ctx.fieldList.addMapField(name);
				});
			}
			if (ctx.givenOrderBy != null)
			{
				tersus.foreach(ctx.givenOrderBy.split(/\s*,\s*/),
				function (clause)
				{
        			var pp = clause.split(/\s+/);
        			if (pp.length >= 1)
        			{
        				var name = pp[0];
        				ctx.fieldList.addMapField(name);
        			}
        		});
        	}			
		}
		else
			ctx.fieldList = this.getFieldList(ctx.tablePrototype);
	}
};
tersus.AdvancedFind.prototype.addTable = function addTable(ctx)
{
	var rawSql = ctx.table? ctx.table : this.getTableName(ctx.tablePrototype);
	var q = this.analyzeParameterizedSQL(rawSql);
	ctx.tableSql = q.sqlText;
	for (var i=0; i<q.parameters.length; i++)
	{
		ctx.values.push(this.getSQLValue(q.parameters[i]));
	}
};
tersus.AdvancedFind.prototype.addDynamicFilterConditions = function(ctx)
{
	if (! ctx.filter)
		return;
	ctx.condition.push('(');
	var q = this.analyzeParameterizedSQL(ctx.filter);
	ctx.condition.push(q.sqlText);
	ctx.condition.push(')');
	for (var i=0; i<q.parameters.length; i++)
	{
		ctx.values.push(this.getSQLValue(q.parameters[i]));
	}
};



tersus.AdvancedFind.prototype.addMatchConditions = function(ctx)
{
  if (ctx.match == null)
        return;
	var columns = ctx.fieldList.fields;
    for (var i = 0; i < columns.length; i++)
    {
		var column = columns[i];
        var matchValue =  column.getSQLValue(ctx.match);
		if (matchValue != null)
        {
			if (matchValue.indexOf && matchValue.indexOf('%') >= 0)
	            this.addCondition(ctx, column, ' like ', matchValue);
	        else
	            this.addCondition(ctx, column, ' = ', matchValue);
	    }
	}
};       
				

tersus.AdvancedFind.prototype.addRangeConditions = function(ctx)
{
  if (ctx.from == null && ctx.to == null)
        return;

	var columns = ctx.fieldList.fields;
    for (var i = 0; i < columns.length; i++)
    {
		var column = columns[i];
        var fromValue = null;
        var toValue = null;
        if (ctx.from != null)
			fromValue = column.getSQLValue(ctx.from);
		if (ctx.to != null)
			toValue = column.getSQLValue(ctx.to);
		if (fromValue == null && toValue != null)
        {
			if (ctx.condition.length > 0)
				ctx.condition.push("AND");
            ctx.condition.push("(");
            this.addCondition(ctx, column, '<=', toValue);
            this.appendOrIsNull(ctx.condition, column.sqlName);
        }
        else
        {
            if (fromValue != null)
                this.addCondition(ctx, column, '>=', fromValue);
            if (toValue != null)
                this.addCondition(ctx, column, '<=', toValue);
        }
    }	

};
tersus.AdvancedFind.prototype.addCondition = function addCondition(ctx, column, operator, value)
{
	var c = ctx.condition;
    if (c.length > 0 && c[c.length - 1] != '(' )
    {
        c.push(" AND ");
    }
    c.push(column.sqlName);
   	c.push(operator);
    c.push('?');
    //types.add(column.getValueHandler());
    ctx.values.push(value);

};

tersus.AdvancedFind.prototype.appendOrIsNull = function(sqla, columnName)
{
    sqla.push('OR');
    sqla.push(columnName);
    sqla.push(' IS NULL )');
};

tersus.AdvancedFind.prototype.setOrderBy = function setOrderBy(ctx, reverse)
{
    ctx.columnOrder = {};
    ctx.orderedColumns = [];
    if (ctx.recordsExit == null)
    	return;

    this.handleExplicitOrderBy(ctx,reverse);
    if (ctx.fieldList != null)
        this.addPrimaryKeyToSortOrder(ctx,reverse);
};

tersus.AdvancedFind.prototype.handleExplicitOrderBy = function handleExplicitOrderBy (ctx, reverse)
{
   	ctx.columnOrder = {};
   	ctx.orderedColumns=[];
   	ctx.orderBy = [];
    if (ctx.givenOrderBy)
    {
    	var p = ctx.givenOrderBy.split(/\s*,\s*/);
		for (var i=0;i<p.length;i++)
        {
        	var clause = p[i];
        	var pp = clause.split(/\s+/);
            if (pp.length == 0 || pp.length > 2)
            {
            	this.modelExecutionError("Invalid <Order By>: '" + ctx.givenOrderBy + "'", this);
            	return;
            }
            var columnName = pp[0];
            var column = ctx.fieldList.getField(columnName);
            if (column == null)
            {
            	if (ctx.tablePrototype && ctx.tablePrototype.isMap)
            	{	
            		column = new tersus.MapField(columnName);
            		ctx.fieldList.addField(column);	
            	}
            	else
            	{
                	modelExecutionError("Invalid <Order By> ('" + ctx.givenOrderBy
                        + "': unknown column '" + columnName + "'");
                	return;
                }
            }
	
			var desc = pp.length > 1 && pp[1].search(/^desc/i) >= 0;
			if (reverse)
				desc = !desc;
			this.addColumnToSortOrder(ctx, column, desc)
        }
    }
};
tersus.AdvancedFind.prototype.addColumnToSortOrder = function addColumnToSortOrder(ctx, column, desc)
{
    var dir = desc ? 'DESC' : 'ASC';
	ctx.columnOrder[column.sqlName]=desc;
    ctx.orderedColumns.push(column);
    if (ctx.orderBy.length > 0)
    	ctx.orderBy.push(',');
    ctx.orderBy.push(column.sqlName);
    if (desc)
		ctx.orderBy.push(dir);
};
tersus.AdvancedFind.prototype.addPrimaryKeyToSortOrder = function addPrimaryKeyToSortOrder(ctx, reverse)
{
    var pkColumns = ctx.fieldList.primaryKeyFields;
    for (var i = 0; i < pkColumns.length; i++)
    {
        var column = pkColumns[i];
        var columnName = column.sqlName;
    	if (ctx.columnOrder[columnName] == null)
        	this.addColumnToSortOrder(ctx, column, reverse);
    }
};
tersus.AdvancedFind.prototype.addStartFromConditions = function addStartFromConditions(ctx)
{
    if (ctx.startFrom == null)
        return;
    var startFromCondition = [];
    var previousTermsValues = [];
    //var  previousTermsTypes = [];
    var nTerms = 0;
    var previousTermsClause = [];
    for (var i = 0; i < ctx.orderedColumns.length; i++)
    {
        var column =  ctx.orderedColumns[i];
        var columnName = column.sqlName;
        var desc = ctx.columnOrder[columnName];
        if (desc == null)
            continue;
        var value = column.getSQLValue(ctx.startFrom);
        var last = (i == ctx.orderedColumns.length - 1);
        if (value == null)
        {
            if (last)
            {
                if (desc) // If the value of the last column is null, we only care about it if it's sorted desc
                {
                    if (previousTermsClause.length > 0)
                    {
                        previousTermsClause.push("AND");
                        previousTermsClause.push(columnName);
                        previousTermsClause.push("IS NULL");
                    }

                }
                if (startFromCondition.length > 0 && previousTermsClause.length > 0)
                {
                    startFromCondition.push("OR (");
                    tersus.addAll(startFromCondition,previousTermsClause);
                    startFromCondition.push(")");
                }
                else if (previousTermsClause.length > 0)
                {
                	
                    tersus.addAll(startFromCondition,previousTermsClause);
                }
                tersus.addAll(ctx.values,previousTermsValues);
//                tersus.addAll(types,previousTermsTypes);
                ++nTerms;
            }
            else
            {
                if (!desc)
                {
                    if (startFromCondition.length == 0)
                    {
                        startFromCondition.push("(");
                    }
                    else
                    {
                        startFromCondition.push('OR (');
                    }
                    if (previousTermsClause.length > 0)
                    {
                        tersus.addAll(startFromCondition,previousTermsClause);
		                tersus.addAll(ctx.values,previousTermsValues);
	//	                tersus.addAll(types,previousTermsTypes);
                        startFromCondition.push("AND");
                    }
                    startFromCondition.push(columnName);
                    startFromCondition.push("IS NOT NULL )");
                    ++nTerms;
                }
                if (previousTermsClause.length > 0)
                {
                    previousTermsClause.push("AND");
                }
                previousTermsClause.push(columnName);
                previousTermsClause.push(" IS NULL");

            }
        }
        else
        {
            if (startFromCondition.length > 0 && previousTermsClause.length > 0)
            {
                startFromCondition.push("OR (");
                tersus.addAll(startFromCondition,previousTermsClause);
                tersus.addAll(ctx.values,previousTermsValues);
//                tersus.addAll(types,previousTermsTypes);
                startFromCondition.push("AND (");
            }
            else if (previousTermsClause.length > 0)
            {
 				tersus.addAll(startFromCondition,previousTermsClause); 
 				startFromCondition.push("AND ((");
 				tersus.addAll(ctx.values,previousTermsValues);
//              tersus.addAll(types,previousTermsTypes);
 			}         	
            else
                startFromCondition.push("((");
            startFromCondition.push(columnName);

            ctx.values.push(value);
            //types.push(column.getValueHandler());
            if (last)
            {
                if (!desc)
                    startFromCondition.push(">= ? ))");
                else
                {
                    startFromCondition.push("<= ?");
                    // we need to explicitly include null values
                    this.appendOrIsNull(startFromCondition, columnName);
                    startFromCondition.push(')');
                }
            }
            else
            {
                if (!desc)
                    startFromCondition.push("> ? ))");
                else
                {
                    startFromCondition.push("< ?");
                    // we need to explicitly include null values
                    this.appendOrIsNull(startFromCondition, columnName);
                    startFromCondition.push(')');
                }

                if (previousTermsClause.length > 0)
                    previousTermsClause.push("AND");
                previousTermsValues.push(value);
                previousTermsClause.push(columnName);
                previousTermsClause.push("= ?");
                //previousTermsTypes.push(column.getValueHandler());
            }
            ++nTerms;
        }
    }
    if (nTerms == 0)
        return;
    if (ctx.condition.length == 0)
        ctx.condition = startFromCondition;
    else
    {
        ctx.condition.push('AND (');
        tersus.addAll(ctx.condition, startFromCondition);
        ctx.condition.push(')');
    }

};
tersus.AdvancedFind.prototype.executeQuery = function(ctx)
{
	this.addStartFromConditions(ctx);
	var sqla = [];
	var explicitColumns = ctx.columns == null && ctx.tablePrototype.isMap ? '*' : ctx.columns;
	this.addSelectFieldsSQL(sqla, ctx.tableSql, ctx.fieldList, explicitColumns);
		
	if (ctx.condition.length > 0)
	{
		sqla.push('WHERE');
		tersus.addAll(sqla, ctx.condition);
	}
	 
	if (ctx.orderBy.length > 0)
	{
		sqla.push('ORDER BY');
		tersus.addAll(sqla,ctx.orderBy);
	}
	var extra = 0; 
	if (ctx.numberOfRecords > 0)
	{
		if (ctx.moreExit != null)
			extra = 1;
		sqla.push('LIMIT');
		sqla.push(extra + parseInt(ctx.numberOfRecords));
		sqla.push('OFFSET')
		sqla.push(ctx.offset);
	}
	var sql = sqla.join(' ');
	this.execSql(sql, ctx.values, function(transaction,results)
	{
		var nrows = results.rows.length;
		var nrecs = (ctx.numberOfRecords > 0 && nrows> ctx.numberOfRecords) ? ctx.numberOfRecords : nrows;
		if (ctx.moreExit && nrows > nrecs)
			this.chargeEmptyExit(ctx.moreExit);
		this.chargeLeafExit('<Number of Records Returned>', nrecs);
		if (ctx.recordsExit != null)
			this.outputResults(results, ctx.recordsExit, nrecs, ctx.backward);
		this.getPosition(ctx);
	});
};
tersus.AdvancedFind.prototype.getPosition = function(ctx)
{
	if (! ctx.needsPosition)
	{
		this.resumeContext();
		return;
	}
	if (ctx.startFrom == null)
	{
		var pos = ctx.backward ? ctx.nMatches - ctx.numberOfRecords-ctx.offset : ctx.offset;
		this.chargeLeafExit(ctx.positionExit,pos);
		this.resumeContext();
		return;
	}

    ctx.condition=[];
    ctx.values=[];
    this.addDynamicFilterConditions(ctx);
    this.addRangeConditions(ctx);
    this.addMatchConditions(ctx);
    this.setOrderBy(ctx, true); // Regardless of <Backward>, we count all records before our own record
    this.addStartFromConditions(ctx);
    this.countMatches(ctx, function(ctx, count)
    {
    	var startFromPosition = count - 1; // If <Start From> is the first record we have count=1 and startFromPosition = 0);
    	var pos = ctx.backward ? 
    						startFromPosition+1-ctx.numberOfRecords-ctx.offset :
    						startFromPosition + ctx.offset;
		this.chargeLeafExit(ctx.positionExit,pos);
		this.resumeContext();
		return;
    });
};


 // file: Alert.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 //Alert - displays something

Alert = function Alert() {};
Alert.prototype = new ActionNode();
Alert.prototype.constructor = Alert;

Alert.prototype.start = function startAlert()
{
	if (this.children && this.children['<Message>'] && tersus.alertMode.interactive)
	{
		var message =  tersus.unescapeString(''+this.getLeafChild('<Message>'));
		if (navigator.notification)
		{
			//Phonegap implementation
			this.pause();
			var self = this;
			var callback = function()
			{
				self.resume = function(){ this.finished();};
				self.continueExecution();
			};
			navigator.notification.alert(message, callback);			
		}
		else
		{
			//Native browser 'alert'
			this.currentWindow.alert(this.translate(message));
		}
	}
};
// file: Appears.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Appears = function Appears(){};
tersus.Appears.prototype = new ActionNode();
tersus.Appears.prototype.constructor = tersus.Appears;
tersus.Appears.prototype.start = function()
{
	var list = this.getTriggerValues('<Item>');
	var item = this.getChild('<Item>');
	var found = false;
	for (var i=0; i<list.length; i++)
	{
		var v = list[i];
		if (compareValues(v, item))
		{
			found = true;
			break;
		}
	}
	if (found)
		this.chargeExit('<Yes>', item);
	else
		this.chargeExit('<No>', item); 
		
};
// file: Arrow.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
function DisclosureArrow() {};
DisclosureArrow.prototype = new DisplayNode();
DisclosureArrow.prototype.constructor = DisclosureArrow;
DisclosureArrow.prototype.defaultTag = 'span';
// file: Average.js
/*********************************************************************************************************
 * Copyright (c) 2003-2010 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
tersus.newAction('Average', function()
{
	var numbers = this.getTriggerValues();
	
	if (numbers&& numbers.length > 0)
	{
		var sum = 0;
		for (var i=0; i<numbers.length; i++)
		{
			var value = numbers[i];
			if (value.leafValue != null)
			{
				sum += parseFloat(value.leafValue);
			}
		}
		this.chargeLeafExit('<Mean>', sum/numbers.length);
	}
});// file: Back.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Back = function tersus_GetElmentName() {};
tersus.Back.prototype = new ActionNode();
tersus.Back.prototype.constructor = tersus.Back;
tersus.Back.prototype.start = function ()
{
	setTimeout(function () {window.history.back();},0);
};
// file: BackgroundAction.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.BackgroundAction = function tersus_BackgroundAction() {};
tersus.BackgroundAction.prototype = new ActionNode();
tersus.BackgroundAction.prototype.constructor = tersus.BackgroundAction;
tersus.BackgroundAction.prototype.start = function tersus_BackgroundAction_start()
{
	if (this.parent.isBackground)
	{
		// This is the "final" invocation (in the background)
		ActionNode.prototype.start.call(this);
	}
	else
	{
		// This is the "original" invocation that should start a background process
		var process = new BackgroundProcess(this.parent, this.element);
		if (this.triggers)
			for (var i = 0; i < this.triggers.length; i++)
			{
				var trigger = this.triggers[i];
			
				if (trigger.isRepetitive)
				{
					internalError("Repetitive triggers not supported in Background Action");
				}
				else
				{
					var value = this.getChild(trigger.role);
					if (value)
						process.setTriggerValue(trigger.role, value);
				}
			}
		setTimeout (function(){
		tersus.async.exec(function (){process.start()}, false /* don't wait */);},0);// Using setTimeout to ensure we're not continuing in same 'burst' of Javascript
	}
};
// file: Branch.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function Branch() {};
Branch.prototype = new ActionNode();
Branch.prototype.constructor = Branch;
Branch.prototype.start = function ()
{
	var dataTrigger = this.getElement('<Data>');
	var selectorTrigger = this.getElement('<Selector>');
	if (! selectorTrigger) // For backward compatability
		selectorTrigger = this.getElement('<Value>');
	var selectorValue = selectorTrigger.getChild(this);
	var dataValue;
	if (dataTrigger)
		dataValue = dataTrigger.getChild(this);
	else
		dataValue = selectorValue;
	var defaultExit = this.getElement('<Other>');
	for (var i=0; i<this.exits.length; i++)
	{
		var exit = this.exits[i];
		if (exit != defaultExit)
		{
			var valueStr = exit.valueStr;
			if (valueStr == undefined)
				valueStr = exit.role;
			valueStr = valueStr.replace(/\\t/g, '\t');
			valueStr = valueStr.replace(/\\n/g, '\n');
			valueStr = valueStr.replace(/\\r/g, '\r');
			valueStr = valueStr.replace(/\\\\/g, '\\');

			if (selectorValue.toString() == valueStr)
			{
				this.chargeExit(exit, dataValue);
				return;
			}
			
		}
	}
	if (defaultExit)
		this.chargeExit(defaultExit, dataValue);
};// file: BranchByType.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.BranchByType = function BranchByType(){};
tersus.BranchByType.prototype = new ActionNode();
tersus.BranchByType.prototype.constructor = tersus.BranchByType;
tersus.BranchByType.prototype.start = function()
{
	var input = this.getChild('<Input>');
	var otherExit = null;
	if (input)
	{
		for (var i=0; i<this.exits.length; i++)
		{
			var exit = this.exits[i];
			if (exit.role == '<Other>')
				otherExit = exit;
			else
			{
				var c = exit.getChildConstructor();
				if (c && c.prototype.isInstance(input))
				{
					this.chargeExit(exit, input);
					return;
				}
			}
		}
		if (otherExit)
			this.chargeExit(otherExit, input);
	}
};// file: Button.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function Button(){};
Button.prototype = new DisplayNode();
Button.prototype.constructor = Button;
Button.prototype.type = 'Button';
Button.prototype.defaultTag = 'button';
Button.prototype.isButton = true;
Button.prototype.destroy = function()
{
	this.isDestroyed = true;
	DisplayNode.prototype.destroy.call(this);
};
Button.prototype.createChildren = function Button_createChildren()
{
	// we don't create the sub-hierarchy for a button.
};
Button.prototype.createViewNode = function Button_createViewNode()
{
	DisplayNode.prototype.createViewNode.call(this);
	this.viewNode.innerHTML = tersus.escapeHTML(this.getCaption());
};
Button.prototype.captionChanged = function()
{
	if (this.viewNode)
		this.viewNode.innerHTML = tersus.escapeHTML(this.getCaption());
};
Button.prototype.setEventHandlers = function Button_setEventHandlers()
{
	DisplayNode.prototype.setEventHandlers.call(this);
	this.registerEventHandler(Events.ON_CLICK);
}
Button.prototype.doStart = function Button_start() 
{
	// We override doStart and not start because we don't want to trace 'Started' events for buttons
	this.waiting();
};
Button.prototype.onClick = function(domNode, callback)
{
	queueEvent(this, this.handleClick, [], new Date(), "Button clicked ["+this.getPath().str+"]", callback);
}; 
Button.prototype.handleClick = function tersus_Button_handleClick() 
{
	if (this.isDestroyed || !this.currentWindow || this.currentWindow.closed || this.status != FlowStatus.WAITING_FOR_INPUT)
	{
		return;
	}
	
	var validate = this.sharedProperties.validate != false; // null considered true
	var validationContext = this.getValidationContext();
	var valid = true;
	if (validate)
	{
		if (tlog) tlog('button.onclick:validating');
		valid = validationContext.validate();
	}
	if (valid)
	{
		if (tlog) tlog('button.onclick: starting handler');
		this.children = null;
		this.readyToResume();
		this.clicked = true;
		resumeRoot();
		if (tlog) tlog('button.onclick: handler done');
	}
	else
		if (tlog) tlog('button.onclick: not valid');
	
};

Button.prototype.validate = function validate()
{
	return true; // Do not traverse children
};
Button.prototype.resume = function resumeButton()
{
	if (this.clicked)
	{
		this.clicked = false;
		this.started();
		FlowNode.prototype.start.call(this);
	}
	else 
		FlowNode.prototype.resume.call(this);
};
Button.prototype['get<Disabled>'] = function ()
{
	return this.disabled;
};

Button.prototype['set<Disabled>'] = function (disabled)
{
	this.disabled = disabled?true:false;
	if (this.viewNode)
	{
		this.viewNode.disabled=this.disabled;
		if (window.browser == 'IE') // IE 6.0 doesn't support the :disable CSS pseudoclass, so we use an explicit style as a workaround
		{
			if (disabled && !this.viewNode.className)
				this.viewNode.className = 'disabled';
			if (!disabled && this.viewNode.className=='disabled')
				this.viewNode.className = '';
		}
	}
};
// file: CancelTimer.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.CancelTimer = function tersus_CancelTimer() {};
tersus.CancelTimer.prototype = new ActionNode();
tersus.CancelTimer.prototype.constructor = tersus.CancelTimer;
tersus.CancelTimer.prototype.start = function()
{
	var timerId = this.getLeafChild("<Timer Id>");
	if (! timerId)
	{
		modelError("Missing <Timer Id> in \"Cancel Timer\" "+this.modelId);
		return;
	}
	if (tlog) tlog("Cancelled timer "+timerId);
	clearInterval(timerId);
};
// file: Checkbox.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Checkbox = function tersus_Checkbox () {};
tersus.Checkbox.prototype = new tersus.PushButton();
tersus.Checkbox.prototype.constructor = tersus.Checkbox;
tersus.Checkbox.prototype.buttonType='checkbox';
// <Value> property: for backwards compatability
tersus.Checkbox.prototype['get<Value>'] = tersus.PushButton.prototype['get<Checked>'];
tersus.Checkbox.prototype['set<Value>'] = tersus.PushButton.prototype['set<Checked>'];
tersus.Checkbox.prototype['remove<Value>'] = tersus.PushButton.prototype['remove<Checked>'];
tersus.Checkbox.prototype.labelClicked = function(domNode, callback)
{
	if (this.button && ! this.button.disabled)
	{
		this.button.checked = ! this.button.checked;
		this.onChange(domNode, callback);
	}
}
// file: Click.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Click = function Click() {};
tersus.Click.prototype = new ActionNode();
tersus.Click.prototype.constructor = tersus.Click;
tersus.Click.prototype.start = function ()
{
	var element = this.getChild('<Element>');
	var queue = this.getLeafChild('<Queue>');
	var action = this;
	var targetNode;
	if (element && element.getClickTarget)
		targetNode = element.getClickTarget();
	else if (element & element.eventTargets && element.eventTargers.onClick)
		targetNode= element.eventTargets.onClick;
	else if (element && element.viewNode)
	{
		targetNode = element.viewNode;
	}
	else 
	{
		targetNode = this.getLeafChild('<View Node>');
	}
	if (targetNode)
	{
		this.pause();
		var resume = function()
		{
		  	action.continueExecution();
		}
		var click = function() {
/*			if (targetNode.click &&
			 ! (targetNode.tagName == 'INPUT' && targetNode.type=='text'))
			 	// click() on text input fields doesn't  work
				targetNode.click();
			else if (targetNode.onclick)
			{
				var e = {};
				e.type = 'mouseclick';
				targetNode.onclick(e);
			}*/
			$(targetNode).click();
		 	tersus.async.exec(resume, false /* don't wait */);
		};
		tersus.async.exec(click, false /* don't wait  */); // not sure this behavior is correct
		/*
		 * Proposed improvement:
		 *  - If 'Click' runs in main thread it exits immediately and target is clicked later.
		    - If 'Click' runs in background, it pauses, and target is clicked when root contet is not busy ('wait=true') 
		  */
	}
}
tersus.Click.prototype.resume = function()
{
};
// file: CloseWindow.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 

//CloseWindow - closes the currentWindow

function CloseWindow(){};
CloseWindow.prototype = new ActionNode();
CloseWindow.prototype.constructor = CloseWindow();
CloseWindow.prototype.start = function()
{
	// Find the containing node that has a closeWindow method
	var node = this.getChild('<Dialog>');
	if (!node)
		node = this;
	while (node && !node.closeWindow) 
		node = node.parent;
	if (node && node.closeWindow)
		node.closeWindow();
};
// file: Collapse.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function Collapse() {};
Collapse.prototype = new ActionNode();
Collapse.prototype.constructor = Collapse;
Collapse.prototype.start = function ()
{
	var item = this.getChild('<Item>');
	if (item.collapse)
		item.collapse();
};
// file: Compare.js
/*********************************************************************************************************
 * Copyright (c) 2003-2010 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
tersus.newAction('Compare', function()
{
	var x = this.getLeafChild('<X>');
	var y = this.getLeafChild('<Y>');
	
	if (x<y)
		this.chargeLeafExit('<X<Y>',x);
	if (x>y)
		this.chargeLeafExit('<X>Y>',x);
	if (x==y)
		this.chargeLeafExit('<X=Y>',x);
});// file: Concatenate.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('Concatenate', function()
{
	var st = this.triggers.concat([]);
	var values = [];
	st.sort(function(a,b){ if (a.role < b.role) return -1; else if (a.role == b.role) return 0; else return 1;});
	var sep = this.getLeafChild('<Separator>');
	if (sep == null)
		sep = '';
	for (var i=0;i<st.length;i++)
	{
		var t = st[i];
		if (t.role != '<Separator>' && t.modelId != BuiltinModels.NOTHING_ID  )
		{
			if (t.isRepetitive)
			{
				var l =t.getChildren(this);
				if (l)
				{
					for (var j=0;j<l.length; j++)
					{
						var v = l[j];
						if (v!= null && v.leafValue != null)
							values.push(v.leafValue);
					}
				}
						
			}
			else
			{
				var v = t.getChild(this);
				if (v!= null && v.leafValue != null)
					values.push(v.leafValue);
			}
			
		}
	}
	var out = values.join(sep);
	this.chargeLeafExit('<Concatenation>',out);	
});

// file: Confirm.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
//Confirm - displays a confirmation message and retrieves user input
function Confirm() {};
Confirm.prototype = new ActionNode();
Confirm.prototype.constructor = Confirm;

Confirm.prototype.start = function startConfirm()
{
	var message =  tersus.unescapeString('' +this.getLeafChild('<Message>'));
	var response;
	if (tersus.alertMode.interactive)
		response = this.currentWindow.confirm(this.translate(message));
	else
		response = tersus.alertMode.confirm;
	if (response)
		this.chargeLeafExit('<OK>', message);
	else
		this.chargeLeafExit('<Cancel>', message);
};// file: ConstructDate.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('ConstructDate', function()
{
	var d = new tersus.DateValue();
	d.year = this.getLeafChild('<Year>');
	d.month = this.getLeafChild('<Month>');
	d.day = this.getLeafChild('<Day>');
	this.chargeLeafExit('<Date>',d);	
});

// file: ConstructDateAndTime.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('ConstructDateAndTime', function()
{
	var y = this.getLeafChild('<Year>');
	var m = this.getLeafChild('<Month>');
	var d = this.getLeafChild('<Day>');
	var h = this.getLeafChild('<Hour>');
	var mi = this.getLeafChild('<Minute>');
	var s =  this.getLeafChild('<Second>');
	var ms =  this.getLeafChild('<Millisecond>');
	
	var dt = new Date(y,m-1, d, h, mi, s, ms);
	this.chargeLeafExit('<Date and Time>',dt);
});

// file: ConvertNumberToText.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.numberFormats = {};
tersus.newAction('ConvertNumberToText', function()
{
	var number = this.getLeafChild('<Number>');
	if (number == null)
		return;
	var formatStr = this.getLeafChild('<Format>');
	if (formatStr == null)
		this.chargeLeafExit ('<Text>',new String(number));
	else
		this.chargeLeafExit ('<Text>',gwtFormatNumber(number,formatStr));
});

// file: ConvertTextToNumber.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('ConvertTextToNumber', function()
{
	var text = this.getLeafChild('<Text>');
    if (text != null)
    {
    	if (tersus.isNumber(text))
    	{
    		var f = parseFloat(text);
    		if (! isNaN(f))
    		{
    			this.chargeLeafExit('<Number>',f);
    			return;
    		}
    	}
    	var nanExit = this.getExit('<Not a Number>');
    	if (nanExit != null)
    		this.chargeLeafExit(nanExit.role,text);
    	else
    		throw new tersus.Exception('Invalid number format '+text);
    }
});

// file: Count.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Count = function tersus_Count(){};
tersus.Count.prototype = new ActionNode();
tersus.Count.prototype.constructor = tersus.Count;
tersus.Count.prototype.start = function ()
{
	var values = this.getTriggerValues();
	this.chargeLeafExit('<Occurrences>',values.length);
};
// file: Create.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Create = function tersus_Create() {};
tersus.Create.prototype = new ActionNode();
tersus.Create.prototype.constructor = tersus.Create;
tersus.Create.prototype.start = function ()
{
	var modelId = this.getLeafChild('<Model Id>');
	var exit = this.getElement("<Object>");
	if (exit == null)
		return; // This can happen if user doesn't have permission for target type
	var modelNotFoundExit = this.getElement("<Model Not Found>");

	if (modelId != null)
	{
		var	constructor = tersus.repository.get(modelId);
		if (constructor!=null)
		{
			var value = new constructor();
			this.chargeExit(exit, value);
		}
		else
		{
			if (modelNotFoundExit!=null)
				this.chargeLeafExit(modelNotFoundExit.role,modelId);
			else
				modelExecutionError("Model not found:"+modelId,this);
				
		}
	}
	else
	{
		var value = exit.createChildInstance();
		this.chargeExit(exit, value);
	}
};
// file: CreateFromTemplate.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.CreateFromTemplate = function() {};
tersus.CreateFromTemplate.prototype = new ActionNode();
tersus.CreateFromTemplate.prototype.constructor = tersus.CreateFromTemplate;
tersus.CreateFromTemplate.prototype.start = function()
{
	var t = this.getLeafChild('<Template>');
	var i=0;
	var l = t.length;
	var out = [];
	while (i<l&& i>= 0)
	{
		var n = t.indexOf('${',i);
		if (n == -1)
		{
			out.push(t.substring(i));
			i = n;
		}
		else
		{
			out.push(t.substring(i,n));
			var b = n+ 2;
			var e=t.indexOf('}', b);
			if (e<0)
			{
				modelExecutionError("Invalid template: missing '}' for '${'.  Position="
							+ n
							+ " Text="
							+ t.substring(n, n + 20));
				return null;
			}
			var key=t.substring(b,e);
			out.push(this.getLeafChild(key));
			i = e+1;
		}

	}
	this.chargeLeafExit('<Text>',out.join(''));
}

// file: CreateJSON.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('CreateJSON', function()
{
	var value = this.getChild('<Value>');
	var serializer = new Serializer();
	var text = serializer.serialize(value); 
	this.chargeLeafExit('<JSON Text>', text);
});// file: CreateLocation.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.CreateLocation = function tersus_CreateLocation() {};
tersus.CreateLocation.prototype = new ActionNode();
tersus.CreateLocation.prototype.RELATIVE='<Relative>';
tersus.CreateLocation.prototype.BASE_URL='<Base URL>';
tersus.CreateLocation.prototype.constructor = tersus.CreateLocation;
tersus.CreateLocation.prototype.start = function ()
{
	var relative = this.getTriggerLeafValue(this.RELATIVE);
	var baseURL = this.getTriggerLeafValue(this.BASE_URL);
	if (relative && baseURL)
	{
		modelExecutionError(this.RELATIVE + ' and '+ this.BASE_URL+'should not be used together in Create Location',this);
		relative = false;
	}
	var params;
	if (relative)
		params = tersus.getInternalParameters();
	else
		params = {};
	if (this.triggers)
	{
		for (var i=0; i< this.triggers.length; i++)
		{
			var trigger = this.triggers[i];
			if (trigger.role != this.RELATIVE && trigger.modelId != BuiltinModels.NOTHING_ID)
			{
				var value = trigger.getChild(this);
				if (value != null)
				{
					if (value.serialize)
						value = value.serialize();
					else
						value = value.leafValue;
				}
				params[trigger.role]= value;
			}
		}
	}
	
	var URL;
	if (baseURL)
	{
		URL = baseURL;
		if (URL.indexOf('#') < 0 && URL.indexOf('?')<0)
			URL+='?';
	}
	else
	{
		URL = tersus.getCurrentViewURL();
		if (URL.indexOf('#') <0)
			URL += '#';
	}
	
	var lastChar = URL.charAt(URL.length-1);
	var first =  lastChar == '#' || lastChar == '?';
	for (var p in params)
	{
		var v = params[p];
		if (v!=undefined || v != null)
		{
			if (! first)
				URL +='&';
			URL += encodeURIComponent(p)+'='+encodeURIComponent(v);
			first = false;
		}
	}
	this.chargeLeafExit('<URL>',URL);		
};// file: DatabaseQuery.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.DatabaseQuery = tersus.DatabaseAction.__subclass('tersus.DatabaseQuery');
tersus.DatabaseQuery.prototype.start = function()
{
	var sql = this.getSQLStatememt();
	var q = this.analyzeParameterizedSQL(sql);
	var values = [];
	for (var i=0; i<q.parameters.length; i++)
	{
		values.push(this.getSQLValue(q.parameters[i]));
	}
	this.pause();
	this.execSql(q.sqlText, values, function(transaction,results)
	{
		this.outputResults(results, this.getExit('<Results>'));
		this.resumeContext();
	});
};
// file: DatabaseUpdate.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.DatabaseUpdate = tersus.DatabaseAction.__subclass('tersus.DatabaseUpdate');
tersus.DatabaseUpdate.prototype.start = function()
{
	var sql = this.getSQLStatememt();
	var q = this.analyzeParameterizedSQL(sql);
	var values = [];
	for (var i=0; i<q.parameters.length; i++)
	{
		values.push(this.getSQLValue(q.parameters[i]));
	}
	this.pause();
	this.execSql(q.sqlText, values, function(transaction,results)
	{
		this.handleResults(results);
		this.resumeContext();
	});
};
tersus.DatabaseUpdate.prototype.handleResults = function(results)
{
	if (results)
	{
		if (results.rowsAffected != null)
			this.chargeLeafExit('<Number of Records Affected>',results.rowsAffected);
//		if (results.insertId != null)
//			this.chargeLeafExit('<Insert Id>', results.insertId);
		}
};// file: DateField.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.DateField = function tersus_DateField() {};
tersus.DateField.prototype = new tersus.GenericField();
tersus.DateField.prototype.constructor = tersus.DateField;
tersus.DateField.prototype.isEditable = true;
tersus.DateField.popCalendar = function()
{
	var documentNode = this;
	var node = DisplayNode.findNodeForDocumentNode(documentNode);
	if (!node)
		return;
	var valueHandler = function valueHandler(year, month, day)
	{
		node.onChange();
	};
	node.currentWindow.popUpCalendar(this, this, valueHandler, DATE_FORMAT);
};
tersus.DateField.prototype.initWritable = function ()
{
	tersus.GenericField.prototype.initWritable.call(this);
	this.inputField.id = this.getNodeId();
	this.inputField.onclick = tersus.DateField.popCalendar;
	if (!tersus.isNumber(this.sharedProperties.size))
		this.inputField.size = 10;
};

tersus.DateField.prototype.onChange = function()
{
	tersus.GenericField.prototype.onChange.call(this);
	hideCalendar();
};
tersus.DateInputField = DateInputField = tersus.DateField; // Backward compatability

// file: Delete.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Delete = tersus.DatabaseAction.__subclass('tersus.Delete');
tersus.Delete.prototype.getTablePrototype = function()
{
	return this.getTrigger('<Record>').getChildConstructor().prototype;
};
tersus.Delete.prototype.run = function()
{
	var record = this.getChild('<Record>');
	var tableName = this.getTableName(record);
	var fieldList = this.getFieldList(record.constructor.prototype);
	var sqla = []; // 'buffer' for creating the sql
	var values = []; // values
	sqla.push('DELETE FROM ');
	sqla.push(tableName);
	this.addPrimaryKeyWhereClause(record,fieldList,sqla, values);
	var sql = sqla.join(' ');
	this.execSql(sql, values, function(transaction,results)
	{
		this.handleResults(results,record,tableName);
		this.resumeContext();
	});
			
};
tersus.Delete.prototype.handleResults = function(results, record, tableName)
{
	var count = results.rowsAffected;
	if (count == 1)
		this.chargeExit('<Deleted>',record);
	else if (count == 0)
		this.chargeExit('<Not Found>',record);
	else
		modelExecutionError("Delete resulted in deleting multiple records in table "
                                    + tableName
                                    + ". Check primary key settings", this);
                                    
};// file: DeleteElement.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.DeleteElement = function DeleteElement(){};
tersus.DeleteElement.prototype = new ActionNode();
tersus.DeleteElement.prototype.constructor = tersus.DeleteElement;
tersus.DeleteElement.prototype.start = function()
{
	var e = this.getChild('<Element>');
	if (e && e.deleteMe)
		e.deleteMe();
	else
		internalError("Failed to delete child in "+this.getPath());
};
// file: Dialog.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.isIE6 = navigator.userAgent.indexOf('MSIE 6') >=0;
if ( tersus.isIPhone)
{
	tersus.Dialog = tersus.PopIn; // On iPhone (and iPhone simulation) we use a full pop-in for instead of a Dialog
}
else
{
	tersus.Dialog = tersus.PopIn.__subclass('Dialog');
	tersus.Dialog.prototype.setStyleClass = function(value)
	{
		if (this.pluginVersion>0)
			DisplayNode.prototype.setStyleClass.call(this,value);
	};
	
	tersus.Dialog.prototype.createViewNode = function()
	{
		// create mask (this gives modality by blocking UI access to existing content)
		clearSelection(this.currentWindow); // sometimes the original selection is extended to newly created elements
		var d = this.currentWindow.document;
		var r = this.getCurrentRootViewNode();
		var ml = this.createHTMLChildElement(r,'div');
		ml.className='dialog_mask';
		$(ml).css('z-index',++ tersus.topZindex);
		this.maskLayer = ml;
		// create dialog div
		var template = tersus.repository.getHTMLTemplate(this.templateURL);
		var dialog = importNode(d,template);
		var pStyle=this.sharedProperties.dialogPosition;
		if (pStyle)
		{
			$(dialog).attr('style',pStyle);
		}
		this.dialogLayer = dialog;
		dialog.style.position='absolute';
		var c = this.sprop('styleClass');
		dialog.className='dialog';
		
		this.viewNode = tersus.findChildNodesByClassName(dialog,'dialogBody')[0];
		if (this.viewNode)
		{
			this.header = tersus.findChildNodesByClassName(dialog,'dialogHeader')[0];
			this.footer = tersus.findChildNodesByClassName(dialog,'dialogFooter')[0];
		}
		else
			this.viewNode = dialog;
		this.isOpen = true;
		r.appendChild(dialog);
		$(dialog).css('z-index',++ tersus.topZindex);
		if (this.header)
		  this.header.innerHTML = tersus.escapeHTML(this.getPopupTitle());
		this.onResize();
		this.hideSelectNodes();
	};
	tersus.Dialog.prototype.onResize = function()
	{
		if (! (this.isOpen && this.currentWindow && ! this.currentWindow.closed && this.viewNode))
			return;
		var d = this.viewNode.ownerDocument;
	
		var s = getViewportSize(d);
		// Prevent infinite loop in Internet Explorer ...
		if (this.lastSize && this.lastSize.height == s.height && this.lastSize.width == s.width)
			return; // no change in size - nothing to do
		this.lastSize = s;
		var th = s.height - 30;
		var tw = s.width - 30;
	
		var w = this.sharedProperties.windowWidth;
		if ( ! w || w == ' ')
			w = 350;
		var h = this.sharedProperties.windowHeight;
		if (! h || h == ' ')
			h = 265;
		w = Math.min(w,tw);
		h = Math.min(h,th);
		var dl = this.dialogLayer;
		dl.style.width = w+'px';
		var pStyle=this.sharedProperties.dialogPosition;
		if (!pStyle)
		{
			var leftPx = parseInt((tw-w)/2);
			var topPx = parseInt((th-h)/2);
			var scroll = getWindowScroll(this.currentWindow);
			dl.style.top = (topPx + scroll.top )+'px';
			dl.style.left = (leftPx + scroll.left)+'px';
		}
		var ml = this.maskLayer;
		ml.style.height = s.height + 'px';
		ml.style.width = '100%';
		ml.style.position = 'absolute';
		ml.style.top = '0px';
		ml.style.left = '0px';

		var delta = 0;
		if (this.header)
			delta += getHeight(this.header);
		if (this.footer)
			delta += getHeight(this.footer);
		if (h-delta>20)
			this.viewNode.style.height = (h-delta)+'px';
		this.onResizeChildren();
	}
	tersus.Dialog.prototype.doClose = function() 
	{
		delete tersus.popupNodes[this.getNodeId()];
	
		if (this.isOpen)
		{
			this.resumeNavigation();
			this.viewNode = null;
			this.header = null;
			var dl = this.dialogLayer;
			var ml = this.maskLayer;
			this.dialogLayer=null;
			this.maskLayer=null;
			dl.parentNode.removeChild(dl);
			ml.parentNode.removeChild(ml);
		}
		this.isOpen = false;
		this.restoreSelectNodes();
	};
	tersus.Dialog.prototype.hideSelectNodes = function()
	{
	   	if (!tersus.isIE6)
	   		return;
		var l = this.currentWindow.document.body.getElementsByTagName('select');
		this.selectNodes = [];
		for (var i=0;i<l.length;i++)
			this.selectNodes.push(l[i]);
		for (var i=0;i<this.selectNodes.length;i++)
		{
			var n = this.selectNodes[i];
			var placeHolder = this.createHTMLElement('span');
			n.placeHolder=placeHolder;
			n.parentNode.replaceChild(placeHolder,n);			
		}
	
	};
 	tersus.Dialog.prototype.restoreSelectNodes = function()
	{
		if (this.selectNodes)
		{
			for (var i =0; i<this.selectNodes.length; i++)
			{
				var n =	this.selectNodes[i];
				var placeHolder = n.placeHolder;
				placeHolder.parentNode.replaceChild(n, placeHolder);
				n.placeHolder = null;
			}
		};
	};
	tersus.Dialog.prototype.setEventHandlers = function()
	{
		DisplayNode.prototype.setEventHandlers.call(this);
		this.dragNode = this.header ? this.header : this.viewNode;
	
		this.registerEventHandler(Events.ON_MOUSEDOWN, this.dragNode);
	};
	tersus.Dialog.prototype.onMouseDown = function(domNode, callback)
	{
		var atts = tersus.currentEventAttributes;
		if (!atts) return;
		callback();
		if (this.checkTargetElement(atts.targetElement))
		{
			var p = getScreenPosition(this.dialogLayer);
			this.dialogLayer.style.position='absolute';
			this.dialogLayer.style.top  = (p.top - getScroll(this.dialogLayer).top)+'px';
			this.dialogLayer.style.left = (p.left  - getScroll(this.dialogLayer).left)+'px';
			this.baseP = p;
			this.baseMouseP = {x:atts.clientX,y:atts.clientY};
			var node = this;
			var b = this.viewNode.ownerDocument.body;
			b.style.cursor = 'pointer';
			b.onmouseup = function(e)
			{
				node.onDrag(e);
				node.endDrag();
				return false; // disable default action (image drag) 
			};
			b.onmousemove = function(e)
			{
				node.onDrag(e);
				return false; //disable default action (image drag)
			}
			return false; //disable default action (image drag)
		}
		return true;// Don't disable default action
	};

	// Checks whether an element is a "good" drag source (doesn't have its own event handling logic)
	tersus.Dialog.prototype.checkTargetElement = function (targetElement)
	{
		if (targetElement == this.dragNode)
			return true;
		var n = targetElement;
		while (n && n != this.dragNode)
		{
			if (n.onclick || n.onmousedown || n.onmouseup || n.onmousemove || n.ondoubleclick)
				return false;
			if (n.tagName == 'INPUT' || n.tagName == 'TEXTAREA' || n.tagName == 'SELECT' || n.tagName == 'OPTION')
				return false;
			n = n.parentNode;
		}
		return true;
	};
	tersus.Dialog.prototype.endDrag = function()
	{
		var b = this.viewNode.ownerDocument.body;
		b.onmousemove = null;
		b.onmouseup = null;
		b.style.cursor = 'default';
	
	
	};
	tersus.Dialog.prototype.onDrag = function(e)
	{
		var atts;
		if (e)
			atts = getEventAttributes(e);
		else
			atts = getEventAttributes(this.currentWindow.event);
		if (!atts) return;
		this.dialogLayer.style.top = (this.baseP.top + atts.clientY - this.baseMouseP.y - getScroll(this.dialogLayer).top)+'px';
		this.dialogLayer.style.left = (this.baseP.left + atts.clientX - this.baseMouseP.x - getScroll(this.dialogLayer).left)+'px';
	
	}
}
// file: Dialog2.js
$.fn.tersus = function(options)
{
	return this.map(function(ix, e) {return DisplayNode.findNodeForDocumentNode(e);});
};
//animation/transition end callback
$.fn.animationEnd = function( callback )
{
    if($.support.cssTransitions)
    {
    	var proxy = $.proxy(callback,function(event)
    	{
    		$(this).unbind( "webkitAnimationEnd", proxy );
    		$(this).unbind( "webkitTransitionEnd", proxy );
            return callback.apply( $(this), arguments );
        });
    	
        return this.bind({webkitAnimationEnd:proxy,webkitTransitionEnd:proxy});
    }
    else
    {
        callback();
        return this;
    }
};
tersus.Widget('Dialog2','Page', {
	baseProperties:{'data-role':'page'}
	,isDialog:true
	
	,attachViewNode:function()
	{
		this.mode = 'View';
		if (this.scope && this.scope.isText)
		{
			this.mode = this.scope.leafValue;
			if (this.mode == 'Page')
			{
				this.scope=this.parent;
				//Find containing page 
				while (this.scope && this.scope.plugin != 'Tersus/Widgets/Page')
					this.scope = this.scope.parent;
			}
		}
		else if (this.scope == window.rootDisplayNode)
			this.mode = 'View';
		else if (this.scope != null)
			this.mode = 'Embedded';

		if (this.mode == 'View')
		{
			$.mobile.pageContainer.add('#tersus\\.content').last().append(this.wrapperNode);
			this.previousPage = $.mobile.activePage;
		}
		else
		{
			var $self=$(this.wrapperNode);
			if (this.mode == 'Floating')
			{
				var theme = null;
				$.mobile.pageContainer.add('#tersus\\.content').last().append($self.addClass('ter-dialog ter-overlay'));
			}
			else
			{
				$(this.scope.wrapperNode).addClass('ter-dialog-container').append($(this.wrapperNode).addClass('ter-dialog'));;
			}
			var $h=$self.find('[data-role=header]');
			if ($h.length>0)
				$h.addClass( "ui-header ui-bar-" + (theme || $self.parent('[data-role=page]').data( "theme" ) || "a") ).find('h1').addClass('ui-title');
			else
				$self.addClass('ter-noheader');
			$self.find('[data-role=content]').addClass( 'ui-content');;
		}
		
	}
	,getScrollContainer:function()
	{
		if (! this.scrollContainer)
		{
			if (this.mode == 'View')
			{
				$(this.viewNode).wrapInner('<div>');
				this.scrollContainer =  this.viewNode;
			}
			else
				this.scrollContainer  = this.wrapperNode;
		}
		return this.scrollContainer;		
	}
	,isPopup:true
	,isPopin:true
	,start: function()
	{
		this.scope = this.getChild('<Scope>');
		if (this.element && ! this.element.prepareForHTML)
			this.element.prepareForHTML = DisplayElement.prototype.prepareForHTML;
		var w = this.currentWindow;
		var doc = w.document;
		clearSelection(w);
		this.create();
		FlowNode.prototype.start.call(this);
		this.isOpen = true;
		w.scrollTo(0,1);
		var transition;
		
		if (this.mode === 'View' )
		{
			$(this.viewNode).page({addBackBtn:false});
			transition = this.replacePage;
		}
		else
		{
			if (this.mode == 'Page')
			{
				$(this.viewNode).find('[data-role=header]').addClass('ui-header ui-bar ui-bar-a').find('h1').addClass('ui-title');
				$(this.viewNode).find('[data-role=content]').addClass('ui-content');
			}
			transition = this.mode == 'Floating' ?this.popDialog : this.slideIn ;
		}	
		var self = this;
		tersus.popupNodes[this.getNodeId()]=this;
		setTimeout(function(){transition.call(self);},200);
		
		// Make scope the DOM Parent
		// Set style on scope and self
	}
	,enhanceForms:function()
	{
		var $self = $(this.wrapperNode);
		$self
			.find( "input, textarea" )
            .not( "[type='radio'], [type='checkbox'], button, [type='button'], [type='submit'], [type='reset'], [type='image']" )
            .textinput();
	    $self
            .find( "input, select" )
            .filter( "[data-role='slider'], [data-type='range']" )
            .slider();

  		/*$self
            .find( "select:not([data-role='slider'])" )
            .selectmenu();*/
        $self
            .find( "[type='radio'], [type='checkbox']" )
            .checkboxradio();
        $self
	        .find( "button, [type='button'], [type='submit'], [type='reset'], [type='image']" ).not("[data-role='none']")
            .button();
	}
	,popDialog:function()
	{
		var $self = $(this.wrapperNode);
		this.enhanceForms();
           // enchance form controls
		$self.wrapInner('<div>');
		$self.addClass('ter-dialog-active pop in');


        
		this.onResize();
		$self.animationEnd(function() {
			$self.removeClass("pop out in reverse");
		});
	}
	,onResize: function()
	{
		var $self = $(this.wrapperNode);
		var $inner =$(this.viewNode);
		var th = tersus.availableHeight();
		if (this.mode == 'Floating')
		{
			var dh = $self.outerHeight();
			$self.css('top',(th-dh)/2);
			this.onResizeChildren();
		}
		else
		{
			if (this.mode=='View')
				$inner.css('height',th);
			else
			{
				$self.css('height',$(this.scope.wrapperNode).outerHeight());
				if (this.mode != 'Page')
					$self.css('width',$(this.scope.viewNode).innerWidth());
			}
			tersus.Page.prototype.onResize.call(this);
		}		
	}
	,slideIn:function()
	{
		var self=this;
		var $self = $(this.wrapperNode);
		this.enhanceForms();
		$self.addClass('ter-dialog-active ter-slide right');
		setTimeout(function()
		{
			$self.addClass('in');
			$self.animationEnd(function() {
			self.onResize();
			//Todo: at the end of the animation we probably want to clear these styles 
				//$self.removeClass("slide out in reverse");
			});
		},10);
	
	}
	,changePage: function(from, to, transition, back)
	{
		from.addClass( transition + " out " + ( back ? "reverse" : "" ) );
		to.addClass( $.mobile.activePageClass + " " + transition +
			" in " + ( back ? "reverse" : "" ) );
					// callback - remove classes, etc
		to.animationEnd(function() {
			from.add( to ).removeClass("out in reverse " + transition );
			from.removeClass( $.mobile.activePageClass );
			loadComplete();
			removeContainerClasses();
		});
	
	}
	
	,replacePage:function()
	{
		var self = this;
		var $self  = $(this.viewNode).page();
		var $prev = this.previousPage;
		$self.addClass('ter-slide right ui-page-active');
		if ($prev != null)
			$prev.addClass('ter-slide left ui-page-active in');
		setTimeout(function()
		{
			$self.addClass('in');
			self.onResize();
			if ($prev)
			{
				$self.animationEnd(function()
				{
					$prev.removeClass('ui-page-active').add($self).removeClass('ter-slide right left in');
				});
				$prev.removeClass('in');
			}
		},10);
	}
	,destroy: function()
	{
		if (! this.isDestroyed)
		{
			this.isDestroyed = true;
			this.supr('destroy');
			this.doClose();
		}
	}
	,closeWindow: function() 
	{
		this.destroyHierarchy();
	}
	,refresh: function()
	{
		var wasactive = $(this.viewNode).hasClass('ui-page-active');
		tersus.Page.prototype.refresh.call(this);
		if (wasactive)
			$(this.viewNode).addClass('ui-page-active');
		
	}
	,doClose: function()
	{
		delete tersus.popupNodes[this.getNodeId()];
		var self=this;
		var $self = $(this.viewNode);
		var $wrapper = $(this.wrapperNode);
		if (this.isOpen)
		{
			if (this.mode == 'View')
			{
				var doc = this.currentWindow.document;
				var $prev=this.previousPage;
				
				if ($prev)
					$prev.addClass('ui-page-active ter-slide left');
				$self.addClass('ui-page-active ter-slide right in');
				setTimeout(function()
				{
					$self.animationEnd(function(){
							$wrapper.remove();
							$.fixedToolbars.show(true);
						});
					$self.removeClass('in');
					if ($prev)
						$prev.addClass('in');
					
				},0);
				

			}
			else if (this.mode == 'Floating')
			{
				$(this.wrapperNode).addClass('pop out reverse').animationEnd(function()
				{
					$wrapper.remove();
				});
			
			}
			else
			{
				var $scope = $(this.scope.viewNode);
				$wrapper.addClass('ter-slide right in');
				setTimeout(function()
				{
					$wrapper.removeClass('in').animationEnd(function()
					{
						$wrapper.remove();
						$scope.removeClass('ter-dialog-container');		
					});
				},0);
			}
			this.viewNode = null;
			this.rootNode = null;
		}
		this.isOpen = false;
	}
});
// file: DialogFooter.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.DialogFooter = function tersus_DialogFooter() {};
tersus.DialogFooter.prototype = new Row();
tersus.DialogFooter.prototype.constructor = tersus.DialogFooter;
tersus.DialogFooter.prototype.attachViewNode = function DialogFooter_attachViewNode()
{
	this.tableNode  = this.createHTMLElement('table');
	this.tbodyNode = this.createHTMLElement('tbody');
	this.tableNode.appendChild(this.tbodyNode);
	this.tbodyNode.appendChild(this.viewNode);
	if (this.parent&& this.parent.footer)
		this.parent.footer.appendChild(this.tableNode);

}
PopupFooter=tersus.DialogFooter; // For backward compatability// file: Divide.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Divide = function tersus_Divide() {};
tersus.Divide.prototype = new ActionNode();
tersus.Divide.prototype.constructor = tersus.Divide;
tersus.Divide.prototype.start = function ()
{
	var x = this.getLeafChild('<X>');
	var y = this.getLeafChild('<Y>');
	if (this.getElement('<Ratio>'))
		this.chargeLeafExit('<Ratio>', x/y);
	if (this.getElement('<Quotient>'))
		this.chargeLeafExit('<Quotient>',parseInt(x/y));
	if (this.getElement('<Remainder>'))
		this.chargeLeafExit('<Remainder>',x%y);
};
// file: DualDisplay.js
// This is not really a plugin - just a set of functions that implement flipping
tersus.DualDisplay = {

flip:function flip_dual_display(node)
{
	var parent = node.parent.parent.viewNode;

	parent.mode = 3 - parent.mode;
	tersus.DualDisplay.set(parent);
},
init: function init_dual_display(node)
{
	var parent = node.parent.parent.viewNode;
	parent.mode = 1;
	var s = parent.lastChild.style;
	s.display='none';
	setTimeout(function(){tersus.DualDisplay.set(parent);s.display='';},10);
},

set: function set_dual_display(parent)
{
	var option1 = parent.firstChild;
	var option2 = option1.nextSibling;
	var h = getVisibleSize(parent).height;
	if (parent.mode == 1)
	{
		option1.style['-webkit-transform']='none';
		option2.style['-webkit-transform']='translateY('+h+'px)';
		$(option1).css('-moz-transform','none');
		$(option2).css('-moz-transform','translate(0px,'+h+'px)');
	}
	else
	{
		option1.style['-webkit-transform']='translateY(-'+h+'px)';
		option2.style['-webkit-transform']='none';
		$(option1).css('-moz-transform','translate(0px,-'+h+'px)');
		$(option2).css('-moz-transform','none');
	}
}
};
// file: DynamicDisplay.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.DynamicDisplay = function tersus_DynamicDisplay() {};
tersus.DynamicDisplay.prototype = new DisplayNode();
tersus.DynamicDisplay.prototype.constructor = tersus.DynamicDisplay;
tersus.DynamicDisplay.prototype.defaultTag = 'span';
tersus.DynamicDisplay.prototype.content = null;
tersus.DynamicDisplay.prototype.initViewNode = function()
{
	this.viewNode.style.display='none';
};
// file: DynamicInvocation.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.DynamicInvocation = function tersus_DynamicInvocation() {};
tersus.DynamicInvocation.prototype = new FlowNode();
tersus.DynamicInvocation.prototype.constructor = tersus.DynamicInvocation;
tersus.DynamicInvocation.prototype.doStart = function ()
{
	this.started();
	var modelId = null;
	modelId = this.getLeafChild('<Model Id>');
	if (! modelId)
	{
		modelExecutionError('<Model Id> is missing',this);
		return;
	}
	/* We create a process of the given model, copy trigger values on to it, and then invoke it
	 * (including outgoing links).
	 *
	 */
	
	var	constructor = tersus.repository.get(modelId);
	var process = new constructor();
	process.element = this.element;
	process.parent = this.parent;
	process.mainWindow = this.mainWindow;
	process.currentWindow = this.currentWindow;
	if (process.triggers)
	{
		for (var i = 0; i < process.triggers.length; i++)
		{
			var trigger = process.triggers[i];
			var dTrigger = this.getElement(trigger.role);
			if (dTrigger)
			{
				if (trigger.isRepetitive)
				{
					var values = dTrigger.getChildren(this);
					dTrigger.removeChildren(this);
					for (var j=0; j< values.length; j++)
					{
						var value = values[j];
						trigger.addAChild(process,value)
					}
				}		
				else
				{
					var value = dTrigger.getChild(this);
					if (value)
						trigger.replaceChild(process,value);
				}
			}
		}
	}
	var node = this;
	process.whenDone = function()
	{
		node.finished();
	};
	process.setStatus(FlowStatus.READY_TO_START);
	process.invoke();
};
// file: Earlier.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Earlier = function tersus_Earlier() {};
tersus.Earlier.prototype = new ActionNode();
tersus.Earlier.prototype.constructor = tersus.Earlier;
tersus.Earlier.prototype.start = function ()
{
	var time1 = this.getChild('<Time 1>');
	var time2 = this.getChild('<Time 2>');
	if (tersus.getDate(time1) < tersus.getDate(time2))
		this.chargeExit('<Yes>', time1);
	else
		this.chargeExit('<No>', time1);
};

// file: EarlierOrEqual.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.EarlierOrEqual = function tersus_EarlierOrEqual() {};
tersus.EarlierOrEqual.prototype = new ActionNode();
tersus.EarlierOrEqual.prototype.constructor = tersus.EarlierOrEqual;
tersus.EarlierOrEqual.prototype.start = function ()
{
	var time1 = this.getChild('<Time 1>');
	var time2 = this.getChild('<Time 2>');
	if (tersus.getDate(time1) <= tersus.getDate(time2))
		this.chargeExit('<Yes>', time1);
	else
		this.chargeExit('<No>', time1);

};

// file: Earliest.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Earliest = function tersus_Earliest() {};
tersus.Earliest.prototype = new ActionNode();
tersus.Earliest.prototype.constructor = tersus.Earliest;
tersus.Earliest.prototype.start = function ()
{
	var trigger = this.getElement('<Times>');
	var list = trigger.getChildren(this);
	if (list&& list.length > 0)
	{
		var earliest = list[0];
		for (var i=1;i<list.length;i++)
		{
			var next = list[i];
			if (tersus.getDate(next) < tersus.getDate(earliest))
				earliest = next;
		}
		this.chargeExit('<Earliest>', earliest);
	}
};

// file: EmbeddedHTML.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.EmbeddedHTML = function tersus_EmbeddedHTML(){};
tersus.EmbeddedHTML.prototype = new DisplayNode();
tersus.EmbeddedHTML.prototype.constructor = tersus.EmbeddedHTML;
tersus.EmbeddedHTML.prototype.defaultTag = 'div';
tersus.EmbeddedHTML.prototype.onload = function()
{
	tersus.repository.loadHTMLTemplate(tersus.translateResource(this.sharedProperties.templatePath));
};
tersus.EmbeddedHTML.prototype.createViewNode = function()
{
	var template = tersus.repository.getHTMLTemplate(tersus.translateResource(this.sharedProperties.templatePath));
	this.viewNode = importNode(this.currentWindow.document,template);
};
tersus.EmbeddedHTML.prototype.createChildren = function EmbeddedHTML_createChildren()
{
	if (this.displayElements)
	{
		/* Prepare a hash table with the nodes that correspond to different elements*/
		this.childNodeMap = {};
		tersus.EmbeddedHTML.addChildrenToMap(this.viewNode, this.childNodeMap);
		for (var i=0; i<this.displayElements.length; i++)
		{
			var e = this.displayElements[i];
			if ( !e.isRepetitive && ! e.isMethodElement)
			{
				var child = e.createChildWithoutVisuals(this);
				var childViewNode = this.childNodeMap[e.role];
				if (childViewNode)
				{
					child.setViewNode(childViewNode);
					childViewNode.setAttribute('element_name',null);
					child.setEventHandlers();
					child.createChildren();
				}
				else
					alert('Missing element '+e.role+ " in template (path="+this.getPath()+")");
				
			}
		}
	}
	
};
tersus.EmbeddedHTML.addChildrenToMap = function(scope, map)
{
	//Not reusing createNodeMap because in this case we don't traverse the whole hierarchy
	var children = scope.childNodes;
	for (var i=0; i< children.length; i++)
	{
		var childNode = children[i];
		if (childNode.nodeType == 1 /*Node.ELEMENT_NODE*/ )
		{
			var childName = childNode.getAttribute('element_name');
			if (childName != null)
				map[childName]=childNode;
			else					
				tersus.EmbeddedHTML.addChildrenToMap(childNode, map);
		}
	}
};tersus.EmbeddedElement = function EmbeddedElement(){};
tersus.EmbeddedElement.prototype = new tersus.EmbeddedHTML();
tersus.EmbeddedElement.prototype.constructor = tersus.EmbeddedElement;
tersus.EmbeddedElement.prototype.onload = function()
{
};

tersus.EmbeddedElement.prototype.createViewNode = function tersus_EmbeddedElement_createViewNode()
{
	var parentViewNode = this.parent.viewNode;
	this.templateNode = this.parent.childNodeMap[this.element.role];
	if (!this.templateNode)
		modelError("Failed to locate template node for "+this.getPath());
	this.viewNode = this.templateNode.cloneNode(this.templateNode,true);
	this.viewNode.setAttribute('element_name',null);
	this.viewNode.style.display='';
};
tersus.EmbeddedElement.prototype.attachViewNode = function tersus_EmbeddedElement_attachViewNode()
{
	this.templateNode.parentNode.insertBefore(this.viewNode,this.templateNode);
};
// file: EndsWith.js
/*********************************************************************************************************
 * Copyright (c) 2003-2010 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('EndsWith', function()
{
	var text = this.getLeafChild('<Text>');
	var suffix = this.getLeafChild('<Suffix>');
	
	if (text != null && suffix != null)
	{
		if (text.length >= suffix.length)
		{
			var index = text.length	- suffix.length;
			
			if (text.substring(index) == suffix)
				this.chargeLeafExit('<Yes>',text);
			else
				this.chargeLeafExit('<No>',text);
		}
	}
});// file: Equals.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function Equal() {};
Equal.prototype = new ActionNode();
Equal.prototype.constructor = Equal;
Equal.prototype.YES='<Yes>';
Equal.prototype.NO='<No>';
Equal.prototype.start = function ()
{
	var values = this.getTriggerValues();
	var lastValue = null;
	var allEqual = true;
	for (var j=0; j<values.length && allEqual; j++)
	{
		var value = values[j];
		if (lastValue == null)
			lastValue = value;
		else if (! compareValues(lastValue, value))
		{
			allEqual = false;
		}
	}
	if (allEqual)
	{
		if (this.getElement(this.YES))
			this.chargeExit(this.YES, lastValue);
	}
	else
	{
		if (this.getElement(this.NO) != null)
		{
			this.chargeEmptyExit(this.NO);
		}
	}	
};
// file: EqualsWithTolerance.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.EqualWithTolerance = function tersus_EqualWithTolerance() {};
tersus.EqualWithTolerance.prototype = new ActionNode();
tersus.EqualWithTolerance.prototype.constructor = tersus.EqualWithTolerance;
tersus.EqualWithTolerance.prototype.start = function ()
{
	var x = this.getLeafChild('<X>');
	var y = this.getLeafChild('<Y>');
	var tolerance = this.getLeafChild('<Tolerance>');
	if (x == null)
		modelExecutionError('Missng value for <X>', this);
	if (y == null)
		modelExecutionError('Missng value for <Y>', this);
	if (tolerance == null)
		modelExecutionError('Missng value for <Tolerance>', this);
	if (Math.abs(x-y) < tolerance)
		this.chargeLeafExit('<Yes>',x)
	else
		this.chargeLeafExit('<No>',x);
};
// file: ExcludeFromList.js
/*********************************************************************************************************
 * Copyright (c) 2003-2010 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
tersus.newAction('ExcludeFromList', function()
{
	var listTrigger = this.getElement('<List>');
	if (!listTrigger)
	{
		modelError("Missing trigger <List>");
		return;
	}
	
	var list = listTrigger.getChildren(this);
	var excludeTrigger = this.getElement('<Exclude>');
	
	if (list)
	{
		var excludeList;
		if (excludeTrigger.isRepetitive)
			excludeList = excludeTrigger.getChildren(this); 
		else
			excludeList = [excludeTrigger.getChild(this)];
		
		
		for (var i = 0; i < list.length; i++)
		{
			var toExclude = false;
			for (var j = 0; j< excludeList.length; j++)
			{	
				if (compareValues(excludeList[j], list[i]))
				{
					toExclude = true;
					break;
				}
			}
			
			if (!toExclude)
				this.accumulateExitValue('<Remaining>', list[i]);
		}
	}
});
// file: Exists.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 //Exits - checks if a trigger value was received

Exists = function Exists() {};
Exists.prototype = new ActionNode();
Exists.prototype.constructor = Exists;
Exists.prototype.start = function startExists()
{
	var value = null;
	if (this.triggers)
	{
		for (var i=0; i< this.triggers.length; i++)
		{
			var trigger = this.triggers[i];
			if (trigger.isRepetitive)
			{
				var values = trigger.getChildren(this);
				if (values && values.length>0)
				{
					value = values[0];
					break;
				}
			}
			else
			{
				value = trigger.getChild(this);
				if (value != null)
					break;
			}
		}
	}
	if (value != null)
	{
		this.chargeExit('<Yes>', value);
		
	}
	else
	{
		this.chargeEmptyExit('<No>');
	}
};// file: Expand.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function Expand() {};
Expand.prototype = new ActionNode();
Expand.prototype.constructor = Expand;
Expand.prototype.start = function ()
{
	var item = this.getChild('<Item>');
	var node = this;
	item.onExpand = function()
	{
		node.continueAfterExpand();
		item.onExpand = null;
	};
	node.pause();
	item.expand();
};
Expand.prototype.continueAfterExpand = function()
{
		this.continueExecution();
};// file: Extract.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('Extract', function()
{
	var text = this.getLeafChild('<Text>');
    if (text != null)
    {
        var beginIndex = this.getLeafChild('<Position>');
        if (beginIndex <0)
        {
	        modelExecutionError('Negative <Position>',this);
    	    return;
    	}
        var length = this.getLeafChild('<Length>');
        var endIndex = text.length;
        if (length != null)
        {
            if (length < 0)
		        modelExecutionError('Negative <Length>',this);
            endIndex = beginIndex + length;
            if (endIndex > text.length)
            {
               modelExecutionError("<Text> ('" + text
                            + "') too short to extract a segment of length "
                            + length + " at position " + beginIndex + ".");
               return;
            }
        }
        var segment = text.substring(beginIndex, endIndex);
        this.chargeLeafExit('<Segment>', segment);
	}
});

// file: FileInputField.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
FileInputField = function FileInputField() {
};
FileInputField.prototype = new DisplayNode();
FileInputField.prototype.constructor = FileInputField;

FileInputField.prototype.isFileInputField = true;

FileInputField.prototype.createViewNode = function createViewNode()
{
    this.fileNode = this.currentWindow.document.createElement('input');
    this.fileNode.type = 'file';
    if (this.sharedProperties.accept)
    	this.fileNode.accept=this.sharedProperties.accept;
    this.viewNode = this.currentWindow.document.createElement('form');
    this.viewNode.className='inline';
    this.viewNode.appendChild(this.fileNode);
    if (this.sharedProperties.size)
    	this.viewNode.size = this.sharedProperties.size;
};

FileInputField.prototype['get<Value>'] = function getFieldValue()
{
	if (this.children && this.children['<Value>'])
		return this.children['<Value>'];
	if (!this.fileNode || this.fileNode.value == "")
		return null;
	if (! this.inputFile)
	{
		var element = this.getElement('<Value>');
		if (element)
		{
			this.inputFile = element.createChild(this);
			this.inputFile.inputFieldDisplayNode = this;
		}
	}
	return this.inputFile;
};

FileInputField.prototype.registerEventHandler = function(event, domNode)
{
	if (!domNode)
		domNode = this.fileNode;
	DisplayNode.prototype.registerEventHandler.call(this,event,domNode);
};

// file: FilterNumbers.js
/*********************************************************************************************************
 * Copyright (c) 2003-2010 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
tersus.newAction('FilterNumbers', function()
{
	var numbersTrigger = this.getElement('<Numbers>');
	var numbers = numbersTrigger.getChildren(this);
	var filter = this.getChild('<Filter>');
	
	var filteringNumber = null;
	if (this.triggers)
	{
		for (var i=0; i< this.triggers.length; i++)
		{
			var trigger = this.triggers[i];
			var triggerRole = trigger.role;
			if (!('<Numbers>' == triggerRole) && !('<Filter>' == triggerRole))
			{
				filteringNumber = trigger.getChild(this);
				break;
			}
		};
	}
	
	if (numbers)
	{
		var filterValue = filter.leafValue;
		for (var i=0; i< numbers.length; i++)
		{
			var number = numbers[i].leafValue;
				
			if (filterValue.substring(0,2) == "<=" && number <= filteringNumber)
				this.accumulateExitValue('<Matching>', numbers[i]);
			else if (filterValue.substring(0,1) == "<" && number < filteringNumber)
				this.accumulateExitValue('<Matching>', numbers[i]);
			else if (filterValue.substring(0,1) == "=" && number == filteringNumber)
				this.accumulateExitValue('<Matching>', numbers[i]);
			else if (filterValue.substring(0,2) == ">=" && number >= filteringNumber)
				this.accumulateExitValue('<Matching>', numbers[i]);
			else if (filterValue.substring(0,1) == ">" && number > filteringNumber)
				this.accumulateExitValue('<Matching>', numbers[i]);
		};
	}
});
// file: Find.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Find =  tersus.DatabaseAction.__subclass('tersus.Find');
tersus.Find.prototype.getTablePrototype = function()
{
	return this.getExit('<Records>').getChildConstructor().prototype;
};
tersus.Find.prototype.run = function()
{
	var exit = this.getExit('<Records>');
	if (!exit)
	{
		modelError('Missing exit <Records>',this);
		return;
	}
	var  p = exit.getChildConstructor().prototype;
	var fl = this.getFieldList(p);
	var triggers  = this.triggers;
	if (triggers == null)
		triggers = [];
	var tableName = this.getTableName(p);
	var sqla = [];
	var explicitColumns = fl == null ? '*' : null;
	this.addSelectFieldsSQL(sqla, tableName, fl, explicitColumns);
	
	var first = true;
	var values = [];
	for (var i=0;i<triggers.length; i++)
	{
		var trigger = triggers[i];
		if (trigger.modelId == BuiltinModels.NOTHING_ID) // Control trigger
			continue;
		if (trigger.role.search(/<.*>$/)>=0) // Distinguished trigger (e.g. <Data Source>)
			continue;
		var field = fl.getField(trigger.role);
		if (first)
		{
			sqla.push('WHERE');
			first = false;
		}
		else
			sqla.push('AND');
		sqla.push(field.sqlName);
		var v = this.getSQLValue(trigger.role);
		if ( v == null)
			sqla.push ('IS NULL');
		else
		{
			sqla.push('= ?');
			values.push(v);
		}
	}
	var orderBy = this.getLeafChild('<Order By>');
	if (orderBy != null)
	{
		sqla.push('ORDER BY');
		sqla.push(orderBy);
	}
	var sql = sqla.join(' ');
	this.execSql(sql, values, function(transaction,results)
	{
		this.outputResults(results, exit);
		this.resumeContext();
	});
};
// file: FindElements.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function FindElements() {};
FindElements.prototype = new ActionNode();
FindElements.prototype.SCOPE_TRIGGER_ROLE='<Scope>';
FindElements.prototype.OPTIONS_TRIGGER_ROLE='<Options>';
FindElements.prototype.constructor = FindElements;

FindElements.prototype.start = function ()
{
	var filter = this.createFilter();
	var items = [];
	var scope = this.getChild(this.SCOPE_TRIGGER_ROLE);
	if (scope)	
		scope.scanChildren(filter, {}, items);
	else
	{
		// NO scope specified - scan everywhere
		window.rootDisplayNode.scanChildren(filter, {}, items);
		tersus.unregisterPopupNodes();
		for (var popupId in tersus.popupNodes)
		{
			node = tersus.popupNodes[popupId].scanChildren(filter, {}, items);
		}
	}
	if (items.length == 0)
	{
		var exit = this.getElement('<None>');
		if (exit)
			this.chargeEmptyExit(exit);
	}
	else
	{
		for (var i=0; i<items.length; i++)
		{
			var item = items[i];
			this.accumulateExitValue('<Elements>',item);
		}
	};
};

FindElements.prototype.createFilter = function()
{
	var filter = {};
	filter.components = [];
	var exit = this.getElement('<Elements>');
	var options = this.getChildren(this.OPTIONS_TRIGGER_ROLE);
	var ignoreCase=false;
	var useRegex=false;
	var allowPartial=false;
	for (var i=0;i<options.length;i++)
	{
		var v = options[i].leafValue;
		if (v.match(/ignore case/i))
			ignoreCase=true;
		if (v.match(/regular expressions/i))
			useRegex=true;
		if (v.match(/allow partial/i))
			allowPartial=true;
	}
	filter.modelId = exit.modelId;
	if (this.triggers)
		for (var i=0; i< this.triggers.length; i++)
		{
			var trigger = this.triggers[i];
			if (trigger.role != this.SCOPE_TRIGGER_ROLE && trigger.modelId != BuiltinModels.NOTHING_ID && trigger.role != this.OPTIONS_TRIGGER_ROLE)
			{
				var value = trigger.getChild(this);
				
				if (useRegex && value.isText)
				{
					var modifier = ignoreCase?'i':'';
					var pattern=value.leafValue;
					if (!allowPartial)
						pattern='^'+pattern+'$';
					var r = new RegExp(pattern, modifier);
					filter.components.push ({role:trigger.role,value:null,regex:r});
					
				}
				else
					filter.components.push ({role:trigger.role,value:value,regex:null});
			};
		};
	filter.match = function(item)
	{
		if (filter.modelId != tersus.ANYTHING && item.modelId != this.modelId)
			return false;
		for (var i =0; i<this.components.length;i++)
		{
			var component = this.components[i];
			if (component.role == '<Element Name>')
			{
				if (!item.element)
					return false;
				var role = item.element.role;
				if (component.regex)
				{
					if (!component.regex.test(role))
						return false;
				}
				else
				{	
					var testValue = component.value.leafValue;
					if (ignoreCase)
					{
						if (role.toUpperCase() != testValue.toUpperCase())
							return false;
					}
					else
					{
						if (role != component.value.leafValue)
							return false;
					}
				}
			}
			else
			{
				var element = item.getElement(component.role)
				if (element)
				{
					var value = element.getChild(item);
					if (value && value.isDataNode)
						value = value.value;  // Get the underlying value (not the wrapper)
					if (component.regex)
					{
						if (value == null || !component.regex.test(value.leafValue))
							return false;
					}
					else
					{
						if (!compareValues(value, component.value, ignoreCase))
							return false;
					}
						
				}
				else
				{ 
					return false;
				}
			}
		}
		return true;
	};
	return filter;
};
// file: FireEvent.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.FireEvent = function FireEvent() {};
tersus.FireEvent.prototype = new ActionNode();
tersus.FireEvent.prototype.constructor = tersus.FireEvent;
tersus.FireEvent.prototype.start = function ()
{
	var element = this.getChild('<Element>');
	var eventName = this.getLeafChild('<Event Name>');
	var eventSpec;
	for (var eventKey in Events)
	{
		if (Events[eventKey].elementName == eventName)
			eventSpec = Events[eventKey];
	}
	if (!eventSpec)
		eventSpec = new tersus.Event(eventName); // Ad-hoc event
		
	var parameters = {};
	for (var i = 0; i < this.triggers.length; i++)
	{
		var t = this.triggers[i];
		if (t.role.search('<') < 0)
		{
			var v;
			if ( t.isRepetitive)
				v =  t.getChildren(this);
			else
				v = t.getChild(this);
			if (v != null)
				parameters[t.role]=v;
		}
	}
	tersus.async.exec(function()
	{
		window.engine.handleEvent(element, eventSpec, parameters);
	}, true /* wait */);
}// file: First.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.First = function tesrsus_First(){};
tersus.First.prototype = new ActionNode();
tersus.First.prototype.constructor = tersus.First;
tersus.First.prototype.start = function ()
{
	var trigger = this.getElement('<List>');
	var list = trigger.getChildren(this);
	if (list&& list.length > 0)
	{
		var first = list[0];
		this.chargeExit('<First>', first);
		
	}
};
// file: FloatClose.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.FloatingClose = function tersus_DialogFooter() {};
tersus.FloatingClose.prototype = new Row();
tersus.FloatingClose.prototype.constructor = tersus.FloatingClose;
tersus.FloatingClose.prototype.attachViewNode = function DialogFooter_attachViewNode()
{
	
	//this.divNode = this.createHTMLElement('div');
	
	//this.divNode.appendChild(this.viewNode);
	if (this.parent&& this.parent.close)
	{
		this.parent.close.appendChild(this.viewNode);
	}

}
//PopupFooter=tersus.FloatingClose; // For backward compatability// file: Focus.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('Focus', function()
{
	var e = this.getChild('<Element>');
	if (e.focus)
		e.focus();
	else if (e.viewNode.focus)
		e.viewNode.focus();
});

// file: FormatDateAndTime.js
/*********************************************************************************************************
 * Copyright (c) 2003-2010 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at http://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
tersus.newAction('FormatDateAndTime', function()
{
	var dt0 = this.getChild('<Date and Time>');
	var dt = tersus.getDate(dt0);
	var t = dt.getTime();
	var tz = this.getLeafChild('<Time Zone>');
	if (tz != null)
	{
		modelExecutionError('<Time Zone> not supported in client side version of Format Date and Time (consider using <Time Zone Offset>)',this);
		return;
	}
	var tzOffset = this.getLeafChild('<Time Zone Offset>');
	if (tzOffset == null)
		tzOffset=-10000;
	else
		tzOffset=-tzOffset; // GWT timezone offset is reverse from intuitive.
	var format = this.getLeafChild('<Format>');
	var text  = gwtFormatDateAndTime(t, format, tzOffset);
	this.chargeLeafExit('<Text>',text);
});
// file: Forward.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Forward = function tersus_GetElmentName() {};
tersus.Forward.prototype = new ActionNode();
tersus.Forward.prototype.constructor = tersus.Forward;
tersus.Forward.prototype.start = function ()
{
	setTimeout(function () {window.history.forward();},0);
};
// file: GetCookie.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function GetCookie() {};
GetCookie.prototype = new ActionNode();
GetCookie.prototype.constructor = GetCookie;
GetCookie.prototype.start = function ()
{
	var name = this.getLeafChild('<Name>');
	if (name)
	{
		name=tersus.sqlize(name);
		var value;
		if (tersus.isNative && window.localStorage)
			value = localStorage.getItem('Cookie.'+name,value);
		else
			value = getCookie(name);
			
		if (value != null && value.length>0)
			this.chargeLeafExit('<Value>',value);
	}
};
// file: GetElement.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.GetElement = function tersus_GetElement() {};
tersus.GetElement.prototype = new ActionNode();
tersus.GetElement.prototype.constructor = tersus.GetElement;
tersus.GetElement.prototype.start = function ()
{
	var elementName = this.getLeafChild('<Element Name>');
	var exit = this.getExit('<Element>');
	var parent = this.getChild('<Parent>');
	if (parent && parent.leafValue && parent.leafValue.tagName) // Special handling of html node attributes
	{
		var attributeValue = parent.leafValue.getAttribute(elementName);
		this.chargeLeafExit('<Element>',attributeValue);
	}
	else if (parent != null && parent.getChild)
	{
		
		var element = parent.getElement(elementName);
		var values = null;
		if (element)
		{
			if (element.isRepetitive)
			{
				values = element.getChildren(parent);
				if (values)
				{
					if (!exit.isRepetitive)
						for (var i=0;i<values.length;i++)
							this.chargeExit(exit, values[i]);
					else
						for (var i=0;i<values.length;i++)
							this.accumulateExitValue(exit, values[i]);
											 
				}
			}
			else
			{
				var value = element.getChild(parent);
				if (value != null)
				{
					if (!exit.isRepetitive)
						this.chargeExit(exit, value);
					else
						this.accumulateExitValue(exit, value);
				}
			}
		}
	}
};
// file: GetElementName.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.GetElementName = function tersus_GetElementName() {};
tersus.GetElementName.prototype = new ActionNode();
tersus.GetElementName.prototype.constructor = tersus.GetElementName;
tersus.GetElementName.prototype.start = function ()
{
	var displayElement = this.getChild('<Element>');
	if (displayElement && displayElement.element)
	{
		var name = displayElement.element.role;
		this.chargeLeafExit('<Name>',name);
	}
};
// file: GetEventAttributes.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.GetEventAttributes = function tersus_GetEventAttributes() {};
tersus.GetEventAttributes.prototype = new ActionNode();
tersus.GetEventAttributes.prototype.constructor = tersus.GetEventAttributes;
tersus.GetEventAttributes.prototype.start = function()
{
	var atts = tersus.currentEventAttributes;
	if (atts)
	{
		this.chargeLeafExit('<Event Type>', atts.type);
		this.chargeLeafExit('<Key Code>', atts.keyCode);
		this.chargeLeafExit('<Mouse Button>',atts.button);
		this.chargeLeafExit('<Mouse X>',atts.clientX);
		this.chargeLeafExit('<Mouse Y>',atts.clientY);
		var e = this.getExit('<Target Element>');
		if (e)
			this.chargeExit(e, DisplayNode.findNodeForDocumentNode(atts.targetElement));
	}
	
};
// file: GetGeolocation.js
tersus.newAction('GetGeolocation',function()
{
	var position = null;
	var gl = navigator.geolocation;
	if (gl && gl.getCurrentPosition)
	{
		this.pause();
		this.errorMessage=null;
		var nodeId = this.getNodeId();
		gl.getCurrentPosition(function(p){tersus.GetGeolocation.handleResponse(p,null,nodeId);}, function(error){tersus.GetGeolocation.handleResponse(null, error,nodeId);});
	}
  	else
		this.chargeEmptyExit('<None>');	
});

tersus.GetGeolocation.prototype.resume = function()
{
	if (this.errorMessage)
		throw this.errorMessage;
};
tersus.GetGeolocation.handleResponse = function (p,error, nodeId)
{
	var node = tersus.findNode(nodeId);
	if (!node && node.status != FlowStatus.PAUSED)
		return;
	if (error)
	{
		node.errorMessage=error.message;
	}
	else 
	{
		if (p)
		{
			var exit = node.getExit('<Geolocation>');
   	     	if (exit)
   	     	{   	     		
  				var loc = exit.createChildInstance();
  				var cElement = loc.getElement('Coordinates');
  				var cStruct = cElement.createChild(loc);
  				var c = p.coords;
  				cStruct.setLeafChild('Latitude',c.latitude, Operation.REPLACE);
  				cStruct.setLeafChild('Longitude',c.longitude, Operation.REPLACE);
  				cStruct.setLeafChild('Accuracy',c.accuracy, Operation.REPLACE);
  				cStruct.setLeafChild('Altitude',c.altitude, Operation.REPLACE);
  				cStruct.setLeafChild('Altitude Accuracy',c.altitude_accuracy, Operation.REPLACE);
  				cStruct.setLeafChild('Heading',c.heading, Operation.REPLACE);
  				cStruct.setLeafChild('Speed',c.speed, Operation.REPLACE);
  				
  				
  				var ts = p.timestamp;
  				if (ts)
  				{
  					var dt = new Date();
  					dt.setTime(ts);
  					loc.setLeafChild('Timestamp',dt,  Operation.REPLACE);
  				}
  				node.chargeExit(exit,loc);
  			}
  		}
  		else
			node.chargeEmptyExit('<None>');
	}	
	node.continueExecution();
};// file: GetIndex.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('GetIndex', function()
{
	var e = this.getChild('<Element>');
	if (e.element && e.element.isRepetitive)
	{
		var siblings = e.element.getChildren(e.parent);
		for (var i=0;i<siblings.length;i++)
		{
			if (siblings[i] == e)
				this.chargeLeafExit('<Index>',i);
		}
	}
	
});

// file: GetMapEntry.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('GetMapEntry', function()
{
	var map = this.getChild("<Map>");
	if (map && map.leafValue)
		map = map.leafValue;
	
	var key = this.getChild("<Key>");
	if (key && key.leafValue != null)
		key = key.leafValue;
		
	var value = map[key];
	if (value)
		this.chargeExit('<Value>',value);
	else
		this.chargeEmptyExit('<None>',value);	
});
// file: GetModel.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.GetModel = function tersus_GetModel() {};
tersus.GetModel.prototype = new ActionNode();
tersus.GetModel.prototype.constructor = tersus.GetModel;
tersus.GetModel.prototype.start = function ()
{
	var modelId = null;
	if (this.getElement('<Model Id>') && this.getElement('<Instance>'))
		modelError("'<Model Id>' and '<Instance>' shouldn't both be specified");
	if (this.getElement('<Model Id>'))
	{
		modelId = this.getLeafChild('<Model Id>');
	}
	else
	{
		instance = this.getChild('<Instance>');
		if (instance)
			modelId = instance.modelId;
	}
	var model = null;
	if (modelId)
		model = tersus.repository.get(modelId).prototype;
	if (model)
	{
		if (! model.wrapper)
		{
			var exit =  this.getExit('<Model>');
			model.wrapper = exit.createChildInstance();
			model.wrapper.model = model;
		}
		this.chargeExit('<Model>',model.wrapper);
	}
};
// file: GetNumberedItem.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.GetNumberedItem = function tersus_GetNumberedItem() {};
tersus.GetNumberedItem.prototype = new ActionNode();
tersus.GetNumberedItem.prototype.constructor = tersus.GetNumberedItem;
tersus.GetNumberedItem.prototype.start = function ()
{
	var listTrigger = this.getElement('<List>');
	if (!listTrigger)
	{
		modelError("Missing trigger <List>");
		return;
	}
	var list = listTrigger.getChildren(this);
	if (list)
	{
		var index = this.getLeafChild('<Index>');
		if (!index)
		{
			modelExecutionError('No <Index> specified',this);
			return;
		}
		if (index>=1 && index <= list.length)
			this.chargeExit('<Item>', list[index-1]);
			
		else if (this.getElement('<Invalid Index>'))
			this.chargeLeafExit('<Invalid Index>',index);
		else
		{
			modelExecutionError('Invalid trigger '+index+' (list size is '+list.length + ')', this)
			return;
		}
	}
};
// file: GetPageParamters.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function GetPageParameters() {};
GetPageParameters.prototype = new FlowNode();
GetPageParameters.prototype.constructor = GetPageParameters();

GetPageParameters.prototype.start = function()
{
	for (var i=0; i< this.exits.length; i++)
	{
		var exit = this.exits[i];
		var paramValue = tersus.searchParams[exit.role];
		if (paramValue)
		{
			this.setLeafChild(exit.role, paramValue, Operation.REPLACE);
		}
	}
};// file: GetParent.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.GetParent = function GetParent(){};
tersus.GetParent.prototype = new ActionNode();
tersus.GetParent.prototype.constructor = tersus.GetParent;
tersus.GetParent.prototype.start = function()
{
	var trigger = this.getTrigger('<Element>');
	var parent = null;
	if (trigger != null) // Get the parent of the specified element
	{
		var element = trigger.getChild(this);
		if (element)
		{
			parent = element.parent;
		}
	}
	else // Get the DisplayNode that contains this action
	{
		parent = this;
		while (parent && !parent.isDisplayNode)
			parent = parent.parent;
	}
	if (parent)
		this.chargeExit('<Parent>',parent);	
};
// file: GetPosition.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('GetPosition', function()
{
	var text = this.getLeafChild('<Text>');
	var segment = this.getLeafChild('<Segment>');
	if (text != null && segment != null)
	{
		var position = text.indexOf(segment);
			if (position >= 0)
				this.chargeLeafExit('<Position>',position);
			else
			{
				if (this.getExit('<Not Found>') != null)
					this.chargeLeafExit('<Not Found>', segment);
				else
					modelExecutionError("Segment '" + segment + "' is not contained in '" + 
						text + "' but " + NOT_FOUND + " exit is missing", this);
			}	
	}
});// file: GetStyleAttribute.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('GetStyleAttribute', function()
{
	var e = this.getChild('<Element>');
	var n = this.getLeafChild('<Attribute>');
	var a = tersus.getJsAttributeName(n);
	if (e && e.viewNode && a)
	var v = e.viewNode.style[a];
	this.chargeLeafExit('<Value>',v);
});

// file: GetTableSelection.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function GetTableSelection() {};
GetTableSelection.prototype = new FlowNode();
GetTableSelection.prototype.constructor = GetTableSelection;
GetTableSelection.prototype.start = function getTableSelection()
{
	var table = this.getChild('<Table>');
	var selection = table.selectedRow;
	if (selection)
		this.setChild('<Selected Row>', selection, Operation.REPLACE);
	else
		this.setLeafChild('<None Selected>', 'none', Operation.REPLACE);
};
// file: GetWeek.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function GetWeek() {};
GetWeek.prototype = new FlowNode();
GetWeek.prototype.constructor = GetWeek;
GetWeek.prototype.start = function getWeek()
{
	var date = this.getChild('<Date>');
	if (!date)
		return;
	var year = date.getYear();
	var month = date.getMonth();
	var day = date.getDay();
	// Algorithm from http://www.tondering.dk/claus/calendar.html

	var a = Math.floor((14-month) / 12);
	var y = year + 4800 - a;
	var m = month + 12 * a - 3;
	var b = Math.floor(y/4) - Math.floor(y/100) + Math.floor(y/400);
	var J = day + Math.floor((153 * m + 2) / 5) + 365 * y + b - 32045;
	var d4 = (((J + 31741 - (J % 7)) % 146097) % 36524) % 1461;
	var L = Math.floor(d4 / 1460);
	var d1 = ((d4 - L) % 365) + L;
	var week = Math.floor(d1/7) + 1;
	var weekYear = year;
	if (week == 1 && month == 12)
		weekYear = year+1;
	if (week >= 52 && month == 1)
		weekYear = year-1;
		
	this.setLeafChild('<Year>', weekYear, Operation.REPLACE);
	this.setLeafChild('<Week Number>', week, Operation.REPLACE);		
};
// file: GotoURL.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
tersus.GotoURL = ServiceNode.__subclass('tersus.GotoURL');
tersus.GotoURL.prototype.excludedTriggers=new Set(['<URL>','<Method>', '<Replace>','<Reload>', '<Target Window>']);
tersus.GotoURL.prototype.isAction=true;
tersus.GotoURL.prototype.start = function ()
{
	var url = this.getLeafChild('<URL>');
	var replace = this.getLeafChild('<Replace>');
	var reload = this.getLeafChild('<Reload>');
	var targetWindow = this.getLeafChild('<Target Window>');
	if (reload != false)
		reload = true;
	var method = this.getLeafChild('<Method>');
	if (url == null)
		url = tersus.rootURL;
	
	if (method && method.search('^post$','i') >=0)
	{
		var form = this.prepareForm();
		if (targetWindow)
			form.target  = targetWindow;
		if (url.indexOf('?')>=0)
		{
			modelExecutionError('POST parameters are not part of the URL and should be provided using separate triggers');
			return;
		}
		form.action = url;
		form.method='POST';
		form.submit();
		for (var i=0; i< this.nodesToRemove.length; i++)
		{
			var node = this.nodesToRemove[i];
			node.parentNode.removeChild(node);
		}
	}
	else
	{
		var serializer;
		if (this.triggers)
		{
			var n = this.triggers.length;
			for (var i=0;i<n;i++)
			{
				var t = this.triggers[i];
				if (this.excludedTriggers.contains(t.role) || t.modelId == BuiltinModels.NOTHING_ID)
					continue;
				var value = t.getChild(this);
				if (value == null)
					continue;
				if (url.indexOf('#') < 0 && url.indexOf('?')<0)
					url+='?';
				else
					url += '&';
				url += encodeURIComponent(t.role)+'='+encodeURIComponent(value.leafValue);				
			}
		}
		if (targetWindow)
		{
			var w = window.open(url,targetWindow);
			if (w && w.focus)
				tersus.async.exec(function() {w.focus();}, false /* don't wait */); // Needed in Firefox
			else
				alert('Your browser blocks pop-up windows.\nTo use this application, please configure the browser to allow this site to open pop-up windows.');
		}				
		else
			tersus.async.exec(function () {tersus.gotoURL(url, replace, reload);}, false /* don't wait */);
	}
};
// file: Greater.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Greater = function tersus_Greater() {};
tersus.Greater.prototype = new ActionNode();
tersus.Greater.prototype.constructor = tersus.Greater;
tersus.Greater.prototype.start = function ()
{
	var x = this.getLeafChild('<X>');
	var y = this.getLeafChild('<Y>');
	if (x > y)
		this.chargeLeafExit('<Yes>',x);
	else
		this.chargeLeafExit('<No>',x);
};
// file: GreaterOrEqual.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.GreaterOrEqual = function tersus_GreaterOrEqual() {};
tersus.GreaterOrEqual.prototype = new ActionNode();
tersus.GreaterOrEqual.prototype.constructor = tersus.GreaterOrEqual;
tersus.GreaterOrEqual.prototype.start = function ()
{
	var x = this.getLeafChild('<X>');
	var y = this.getLeafChild('<Y>');
	if (x >= y)
		this.chargeLeafExit('<Yes>',x);
	else
		this.chargeLeafExit('<No>',x);
};
// file: Grid.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Grid = function tersus_Grid() {};
tersus.Grid.prototype = new Table();
tersus.Grid.prototype.constructor = tersus.Grid;
tersus.Grid.prototype.createCaption = tersus.Grid.prototype.createHeader = function() {};
// file: HTMLDisplay.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.HTMLDisplay = function tersus_HTMLDisplay(){};
tersus.HTMLDisplay.prototype = new tersus.DisplayField();
tersus.HTMLDisplay.prototype.constructor = tersus.HTMLDisplay;
tersus.HTMLDisplay.prototype.defaultTag = 'div';
tersus.HTMLDisplay.prototype.allowHTMLTags = true;
// file: HTMLEditor.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.HTMLEditor = function tersus_HTMLEditor(){};
tersus.HTMLEditor.prototype = new DisplayNode();
tersus.HTMLEditor.prototype.constructor = tersus.HTMLEditor;
tersus.HTMLEditor.prototype.defaultTag = 'textarea';
tersus.HTMLEditor.prototype.createViewNode = function()
{
	DisplayNode.prototype.createViewNode.call(this);
	this.open();
};
tersus.HTMLEditor.commandMap = { 'mceImage':'<Insert Image>'}
tersus.HTMLEditor.execCommandHandler = function tersus_html_editor_command_handler(editor_id, elm, command, user_interface, value)
{
	if (!elm)
		return false;
	if ( ! tersus.HTMLEditor.commandMap[command])
		return false;
	var w = getParentWindow(elm).parent;
	var tMCE = tersus.HTMLEditor.getTinyMCE(w);
	if (tMCE)
	{
		var editorInstance = tMCE.getInstanceById(editor_id);
		var nodeId = editor_id.match(/mce_(\d+)/)[1];
		var node = tersus.findNode(nodeId);
		
		var element = node.getElement(tersus.HTMLEditor.commandMap[command]);
		if (element)
		{
			var child = element.createChild(node);
			resumeRoot(); // child will be executerd
			if (child.isButton) // we need to simulate pushing the button
				child.onClick();

			
			return true;
		}

	}

	return false;
	
};
tersus.HTMLEditor.defaultSettings = 
{
	mode : "none"
	,theme : "advanced" 
	,content_css : "Styles?timestamp="+tersus.lastStyleModification
	,execcommand_callback : tersus.HTMLEditor.execCommandHandler
};
tersus.HTMLEditor.prototype.createChildren = function()
{ // supressing child creation - child elements represent components of the externally defined editor 
  // in the future we may support adding elements to the display
};
tersus.HTMLEditor.loaded = false;
tersus.HTMLEditor.loading = false;
window.tinyMCE_onload =  function tinyMCE_onload()
{
	var w = window;
	if (tersus.HTMLEditor.customizeTinyMCE)
		tersus.HTMLEditor.customizeTinyMCE(w);	
	w.tinymce.dom.Event.domLoaded=true;
	var s = {}; // settings for tinyMCE
	var d = tersus.HTMLEditor.defaultSettings;
	for (p in d)
		s[p]= d[p];
	var u = tersus.HTMLEditor.settings; // user settings
	if (u)
	{
		for (p in u)
			s[p]=u[p];
	}
	w.tinyMCE.init(s);
	w.tinyMCE_loaded = true;
	w.tinyMCE_loading = false;
	tersus.foreach(w.tinyMCE_openQueue, function(node)
	{
		node.showEditor();
	});
}
tersus.HTMLEditor.prototype.open = function()
{
	var node = this;
	var w = this.currentWindow;
	if (!w.tinyMCE_loaded && !w.tinyMCE_loading)
	{
		w.tinyMCE_loading = true;
		w.tinyMCE_openQueue = [];
		tersus.loadScript('tiny_mce/tiny_mce_src.js',w);
	}
 	if (w.tinyMCE_loaded)
		tersus.async.exec( function () {node.showEditor();}, false /* don't wait */);
 	else
	 	w.tinyMCE_openQueue.push(node);
	this.open = true;
};
tersus.HTMLEditor.prototype.showEditor = function()
{
	var tMCE = this.getTinyMCE();
	if (tMCE)
	{
		var id = 'mce_'+this.getNodeId();
		this.editorId = this.viewNode.id = this.viewNode.name = id;
        tMCE.execCommand('mceAddControl', false, id);
//		tinyMCE.execCommand('mceFocus',false,this.getEditorId());
//		editorInstance.select();
	}
		
	else
		modelExecutionError("HTML Editor not available",this);
};
tersus.HTMLEditor.prototype.getEditor = function()
{
	var tMCE = this.getTinyMCE();
	if (tMCE && this.open)
		return tMCE.get(this.editorId);
	else
		return null;
};
tersus.HTMLEditor.prototype.invokeAction  = function(action, caller)
{
	//Todo - some actions (e.g. focus) may need to be invoked as global command
	//     - some actions may be implemented in the model 
	var editor = this.getEditor();
	if (editor)
	{
		var value = caller.getLeafChild('Value');
		editor.execCommand(action, false, value);
		editor.execCommand('mceRepaint');
	}
};
tersus.HTMLEditor.prototype.destroy = function()
{
	if (this.open)
	{
		var tMCE = this.getTinyMCE();
		var node = this;
		if (tMCE)
		{
			// Because of timing issues it is possible that tinyMCE still doesn't know about our editor.
			// In such cases we don't remove it (and there's a danger of memory leak - probably mainly on older browsers)
			if (this.editorId && tMCE.get(this.editorId))
				tMCE.execCommand('mceRemoveControl', false, node.editorId);
		}
		this.open = false;
	}
	
	DisplayNode.prototype.destroy.call(this);
};
tersus.HTMLEditor.getTinyMCE = function (w)
{
	var tMCE = null;
	try
	{
		if (w && ! w.closed)
			tMCE = w.tinyMCE;
	}
	catch (e)
	{
	}
	return tMCE;
};
tersus.HTMLEditor.prototype.getTinyMCE = function()
{
	return tersus.HTMLEditor.getTinyMCE(this.currentWindow);
};
tersus.HTMLEditor.prototype['get<HTML Text>'] = function()
{
	var editor = this.getEditor();
	if (editor)
		return editor.getContent();
};
tersus.HTMLEditor.prototype['get<Selected HTML Text>'] = function()
{
	var editor = this.getEditor();
	if (editor)
	{
		var sel = new TinyMCE_Selection(editor);
		return sel.getSelectedHTML();
	}
};
tersus.HTMLEditor.prototype['get<Selected HTML Element>'] = function()
{
	var editor = this.getEditor();
	if (editor)
	{
		var sel = editor.selection;
		if (sel)
			return sel.getNode();
	}
};
tersus.HTMLEditor.setHTML = tersus.HTMLEditor.prototype['set<HTML Text>'] = function(text)
{
	if (!this.open)
	{
		alert('Editor Not Open');
		return;
	}
	var node = this;
	var f = function()
	{
		var editor = node.getEditor();
		if (editor)
			setTimeout(function(){editor.setContent(text);},100);
		else
			setTimeout(f, 100);
	}
	setTimeout(f, 100);
};

// file: Hyperlink.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function Hyperlink() {};
Hyperlink.prototype = new DisplayNode();
Hyperlink.prototype.constructor = Hyperlink;
Hyperlink.prototype.defaultTag = 'span';
// file: Image.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Image = function TersusImage(){};
tersus.Image.prototype = new DisplayNode();
tersus.Image.prototype.constructor = tersus.Image;
tersus.Image.prototype.getTag = function()
{
	return "img";
};
tersus.Image.prototype['set<Path>'] = function( path)
{
	this.srcPath = path;
	this.viewNode.src = tersus.translateResource(path);
};
tersus.Image.prototype['get<Path>'] = function()
{
	if (this.srcPath)
		return this.srcPath;
	else
		return null;
};
// file: Insert.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Insert = tersus.DatabaseAction.__subclass('tersus.Insert');
tersus.Insert.prototype.getTablePrototype = function()
{
	return this.getTrigger('<Record>').getChildConstructor().prototype;
};
tersus.Insert.prototype.run = function()
{
	var record = this.getChild('<Record>');
	var tableName = this.getTableName(record);
	var fieldList = this.getFieldList(record.constructor.prototype);
	var dupExit = this.getExit('<Duplicate>');
	if (dupExit)
		this.checkDup(record,tableName,fieldList);
	else
		this.insert(record,tableName,fieldList);
};
tersus.Insert.prototype.checkDup = function(record,tableName, fieldList)
{
	var sqla = [];
	var values = [];
	sqla.push("SELECT COUNT(1) as count FROM ");
	sqla.push(tableName);
	this.addPrimaryKeyWhereClause(record,fieldList,sqla, values);
	var sql = sqla.join(' ');
	this.execSql(sql, values, function(transaction,results)
	{
		var count = results.rows.item(0)['count'];
		if (count > 0)
		{
			this.chargeExit('<Duplicate>', record);
			this.resumeContext();			
		}
		else
			this.insert(record,tableName,fieldList);
	});
	
};
tersus.Insert.prototype.insert = function(record, tableName, fieldList)
{
	
	var sqla = []; // 'buffer' for creating the sql
	sqla.push('INSERT INTO');
	sqla.push(tableName);
	sqla.push('(');
	var values = []; // values
	var fields = fieldList.fields;
	var nfields = fields.length;
	var n = 0;
	for (var j=0;j<nfields;j++)
	{
		var field = fields[j];
		var v = field.getSQLValue(record);
		if (v != null)
		{
			if (n>0)
				sqla.push(',');
			sqla.push(field.sqlName);
			n++;
			values.push(v);
		}
	}
	sqla.push(')');
	sqla.push('VALUES (?');
	for (var i=1;i<n;i++)
		sqla.push(', ?');
	sqla.push(')');
	var sql = sqla.join(' ');
	this.execSql(sql, values, function(transaction,results)
	{
		this.chargeExit('<Inserted>',record);
		this.resumeContext();
	});
			
};// file: InsertNumberedItem.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.InsertNumberedItem = function InsertNumberedItem(){};
tersus.InsertNumberedItem.prototype = new ActionNode();
tersus.InsertNumberedItem.prototype.constructor = tersus.InsertNumberedItem;
tersus.InsertNumberedItem.prototype.start = function()
{
	var item = this.getChild('<Item>');
	var parent = this.getChild('<Parent>');
	if (!parent.elementList)
	{
		if (this.getElement('<Invalid Parent>'))
			this.chargeExit('<Invalid Parent>', parent);
		else
			modelExecutionError("Invalid parent "+parent.modelId,this);
		return;
	}
	var index = this.getLeafChild('<Index>');
	
	if (this.pluginVersion && this.pluginVersion ==1)
		index = index -1;
		
	var e = this.getTargetElement(parent,item);
	if (!e)
	{
			if (this.getElement('<Invalid Item>'))
				this.chargeExit('<Invalid Item>', item);
			else
				modelExecutionError("Invalid item "+item.modelId,this);
			return;
	}
	if (index < 0 || index > e.getChildren(parent).length)
	{
		if (this.getElement('<Invalid Index>'))
			this.chargeLeafExit('<Invalid Index>', index);
		else
			modelExecutionError("Invalid index "+index,this);
		return;
	}
	e.addAChild(parent, item);
	var children = e.getChildren(parent);
	e.moveChild(parent,children.length-1, index);
	this.chargeExit('<Inserted>',item);
	
};
tersus.InsertNumberedItem.prototype.getTargetElement = function(parent,item)
{
	var triggerId = this.getElement('<Item>').modelId;
	if (parent && parent.elementList)
	{
		var elements = parent.elementList;
		for (var i=0;i<elements.length;i++)
		{
			if (elements[i].isRepetitive && (elements[i].modelId == item.modelId || elements[i].modelId == triggerId))
			{
				return elements[i];
			}
		}
	}
	return null;
};

// file: InvokeElementAction.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.InvokeElementAction = function() {};
tersus.InvokeElementAction.prototype = new ActionNode();
tersus.InvokeElementAction.prototype.constructor = tersus.InvokeElementAction;
tersus.InvokeElementAction.prototype.start = function()
{
	var targetElement = this.getChild('<Element>');
	var action = this.getLeafChild('<Action>');
	if (targetElement.invokeAction)
	{
		var output = targetElement.invokeAction(action, this);
		// Here we would need to pause the current process, run the target action, and when that action is completed, resume the current action
	}
};

// file: IsEmpty.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('IsEmpty', function()
{
	var text = this.getLeafChild('<Text>');
	
	if (text == null)
	{
		this.chargeEmptyExit('<Missing>');
		return;
	}
		
	var length = text.length;
	if (length > 0)
		this.chargeLeafExit('<No>',text);
	else
		this.chargeLeafExit('<Yes>',text);
});// file: IsZero.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.IsZero = function tersus_IsZero() {};
tersus.IsZero.prototype = new ActionNode();
tersus.IsZero.prototype.constructor = tersus.IsZero;
tersus.IsZero.prototype.start = function ()
{
	var x = this.getLeafChild('<X>');
	if (x==0)
		this.chargeLeafExit('<Yes>',x)
	else
		this.chargeLeafExit('<No>',x);
};
// file: Label.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function Label() {};
Label.prototype = new tersus.DisplayField();
Label.prototype.constructor = Label;
Label.prototype.defaultStyleClass='label';
Label.prototype.textTranslation=true;

Label.prototype.initialize = function ()
{
	this.value = new tersus.Text();
	this.value.leafValue = this.getBaseCaption();
};
/*
//TODO(x) delete this
Label.prototype.initViewNode = function()
{
	this.value = new tersus.Text();
	this.value.leafValue = this.getCaption();
	tersus.GenericField.prototype.initViewNode.call(this);
}*/
Label.prototype.captionChanged = function()
{
	if (this.viewNode)
	{
		var caption = this.getBaseCaption();
		if (! caption.isNode)
		{
			var text = new tersus.Text();
			text.leafValue = caption;
			caption = text;
		}
		this.setFieldValue(caption);
	}
};
// file: Last.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Last = function tesrsus_Last(){};
tersus.Last.prototype = new ActionNode();
tersus.Last.prototype.constructor = tersus.Last;
tersus.Last.prototype.start = function ()
{
	var trigger = this.getElement('<List>');
	var list = trigger.getChildren(this);
	if (list&& list.length > 0)
	{
		var last = list[list.length-1];
		this.chargeExit('<Last>', last);
		
	}
};
// file: Later.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Later = function tersus_Later() {};
tersus.Later.prototype = new ActionNode();
tersus.Later.prototype.constructor = tersus.Later;
tersus.Later.prototype.start = function ()
{
	var time1 = this.getChild('<Time 1>');
	var time2 = this.getChild('<Time 2>');
	if (tersus.getDate(time1) > tersus.getDate(time2))
		this.chargeExit('<Yes>', time1);
	else
		this.chargeExit('<No>', time1);
};

// file: LaterOrEqual.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.LaterOrEqual = function tersus_LaterOrEqual() {};
tersus.LaterOrEqual.prototype = new ActionNode();
tersus.LaterOrEqual.prototype.constructor = tersus.LaterOrEqual;
tersus.LaterOrEqual.prototype.start = function ()
{
	var time1 = this.getChild('<Time 1>');
	var time2 = this.getChild('<Time 2>');
	if (tersus.getDate(time1) >= tersus.getDate(time2))
		this.chargeExit('<Yes>', time1);
	else
		this.chargeExit('<No>', time1);
};

// file: Latest.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Latest = function tersus_Latest() {};
tersus.Latest.prototype = new ActionNode();
tersus.Latest.prototype.constructor = tersus.Latest;
tersus.Latest.prototype.start = function ()
{
	var trigger = this.getElement('<Times>');
	var list = trigger.getChildren(this);
	if (list&& list.length > 0)
	{
		var latest = list[0];
		for (var i=1;i<list.length;i++)
		{
			var next = list[i];
			if (tersus.getDate(next) > tersus.getDate(latest))
				latest = next;
		}
		this.chargeExit('<Latest>', latest);
	}
};
// file: Layer.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Layer = function tersus_Layer() {};
tersus.Layer.prototype = new Pane();
tersus.Layer.prototype.constructor = tersus.Layer;
tersus.Layer.prototype.attachViewNode = function()
{
	this.currentWindow.currentRootDisplayNode.viewNode.appendChild(this.viewNode);
	this.viewNode.style.position='absolute';
	$(this.viewNode).css('z-index',  ++ tersus.topZindex);
};
tersus.Layer.prototype.onDelete = function onDelete()
{
	this.destroyHierarchy();
	if (this.viewNode && this.viewNode.parentNode)
		this.viewNode.parentNode.removeChild(this.viewNode);
};
// file: Length.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('Length', function()
{
	var text = this.getLeafChild('<Text>');
	if (text == null)
		this.chargeEmptyExit('<Missing>');
	else if (text.length != null)
		this.chargeLeafExit('<Length>',text.length);
});

// file: List Holder.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function List_Holder() {};
List_Holder.prototype = new tersus.HTMLTag();
List_Holder.prototype.constructor = List_Holder;
List_Holder.prototype.defaultTag = 'ul';// file: List Item.js

/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function List_Item() {};
List_Item.prototype = new DisplayNode();
List_Item.prototype.constructor = List_Item;
List_Item.prototype.defaultTag = 'li';// file: Logout.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Logout = function() {};
tersus.Logout.prototype = new ActionNode();
tersus.Logout.prototype.constructor = tersus.Logout;
tersus.Logout.prototype.start = function ()
{
	
	tersus.logout();
	tersus.async.exec(tersus.reloadApplication, false /* don't wait */);
};
// file: LowerCase.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('LowerCase', function()
{
	var text = this.getLeafChild('<Text>');
	if (text != null)
		this.chargeLeafExit('<Lower Case>',text.toLowerCase());
});

// file: Map.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Map = function tersus_Map() {};
tersus.Map.prototype = new DataValue();
tersus.Map.prototype.constructor = tersus.Map;
tersus.Map.prototype.isMap = true;
tersus.Map.prototype.init = function initMap()
{
	this.leafValue = {};
};
// file: Match.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('Match', function()
{
	var pattern = this.getLeafChild('<Pattern>');
	var text = this.getLeafChild('<Text>');
	var allowPartial = this.getLeafChild('<Allow Partial Match>');
	
	var matchesExit = this.getExit('<Matches>');
	
	if (text == null)
		this.chargeEmptyExit('<Missing>');
	else
	{	
		if (allowPartial != true)
		{
			if (pattern.charAt(0) != '^')
				pattern = '^' + pattern;
		
			if (pattern.charAt(pattern.length -1) != '$')
				pattern = pattern + '$';
		}
				
		var matches = text.match(new RegExp(pattern)); 
		
		if (matches != null)
		{
			this.chargeLeafExit('<Yes>',text);
			if (matchesExit)
			{
				var l = matches.length;
				for (var i=0;i<l;i++)
					this.chargeLeafExit(matchesExit, matches[i]);
			}
		}
    	else
    		this.chargeLeafExit('<No>',text);
    		
    }
});// file: Maximum.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Maximum = function tersus_Maximum() {};
tersus.Maximum.prototype = new ActionNode();
tersus.Maximum.prototype.constructor = tersus.Maximum;
tersus.Maximum.prototype.start = function ()
{
	var values = this.getTriggerValues();
	if (values.length == 0)
	{
		modelExecutionError("No values specified - can't compute maximum",this);
		return;
	}
	var maximum = values[0].leafValue;
	for (var j=1; j<values.length; j++)
	{
		var value = values[j];
		if (value && value.leafValue > maximum)
			maximum = value.leafValue;
	}
	this.chargeLeafExit('<Maximum>',maximum);
};
// file: MenuBar.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.MenuBar = Row.__subclass('tersus.MenuBar');
tersus.MenuBar.prototype.defaultStyleClass='menu_bar';
// file: MenuButton.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
tersus.MenuButton = function tersus_MenuButton() {};
tersus.MenuButton.prototype = new DisplayNode();
tersus.MenuButton.prototype.constructor = tersus.MenuButton;
tersus.MenuButton.prototype.defaultTag = 'button';
tersus.MenuButton.prototype.defaultStyleClass='menuButton';
tersus.MenuButton.prototype['set<Value>'] = function (value)
{
	this.value = value;
};
tersus.MenuButton.prototype['set<Value>'].requiresNodes = true;
tersus.MenuButton.prototype.createViewNode = function Button_createViewNode()
{
	DisplayNode.prototype.createViewNode.call(this);
	this.viewNode.innerHTML = tersus.escapeHTML(this.getCaption());
};
tersus.MenuButton.prototype.captionChanged = function()
{
	if (this.viewNode)
		this.viewNode.innerHTML = tersus.escapeHTML(this.getCaption());
};
tersus.MenuButton.prototype.setEventHandlers = function Button_setEventHandlers()
{
	this.viewNode.setAttribute('lid', this.getNodeId());
	this.viewNode.onclick = tersus.MenuButton.onclick;
	this.viewNode.onmousedown = tersus.MenuButton.mousedown;
	this.viewNode.onblur = tersus.MenuButton.blur;
	this.viewNode.onkeydown = tersus.MenuButton.keydown;
	this.viewNode.onkeyup = tersus.MenuButton.keyup;
};
tersus.MenuButton.prototype.getOptionText = tersus.GenericField.prototype.getOptionText;
tersus.MenuButton.onclick = function()
{
	var node = DisplayNode.findNodeForDocumentNode(this);
	if (node && ! node.isActive)
	{
		node.activate();
	}
};
tersus.MenuButton.keydown = function(evt)
{
	var node = DisplayNode.findNodeForDocumentNode(this);
	if (!node || ! node.isActive)
		return true;
	if (!evt) evt = window.event;
	if (!evt) return;
	var k=evt.keyCode;
	var option = node.highlightedOption;
	switch(k)
	{
		case 27: //Escape
		case 9: //Tab
			node.deactivate();break;
		case 37: case 38: // Left/Up
			node.highlight(option == null? -1:option-1);
			break; // prevent scrolling
		case 39: case 40: // Right/Down
			node.highlight(option == null?0:option+1);
			break;
		default:
	}
};
tersus.MenuButton.keyup = function(evt)
{
	var node = DisplayNode.findNodeForDocumentNode(this);
	if (!node || ! node.isActive)
	{
		return true;
	}
	if (!evt) evt = window.event;
	if (!evt) 
	{
		return;
	}
	var k=evt.keyCode;
	if (k == 13 && node.highlightedOption != null)
		node.select();
/*	if( k ==38 || k ==40) // Up|Down
	{
		var button = node.viewNode;
		node.dontDeactivate = true;
		button.blur();
		setTimeout(function(){button.focus();node.dontDeactivate=false;},10);
	}*/
};
tersus.MenuButton.highlight = function()
{
	var nodeId = this.getAttribute('nodeId');
	if (!nodeId)
		return;
	var node = tersus.findNode(nodeId);
	if (! node)
		return;
	node.mouseIn = true;
	node.highlight(this.getAttribute('optionIndex'));
};
tersus.MenuButton.unhighlight = function()
{
	this.className = '';
	var nodeId = this.getAttribute('nodeId');
	if (!nodeId)
		return;
	var node = tersus.findNode(nodeId);
	if (! node)
		return;
	node.mouseIn = false;
	node.clearHighlight();
};
tersus.MenuButton.select = function()
{
	var nodeId = this.getAttribute('nodeId');
	if (!nodeId)
		return;
	var node = tersus.findNode(nodeId);
	if (node)
		tersus.async.exec(function(){node.select();}, false /* don't wait - queue event is called by 'select' */);
};
tersus.MenuButton.prototype.highlight = function(index)
{
	if (index <0)
		index = this.options.length -1;
	if (index >=this.options.length)
		index = 0;
	this.clearHighlight();
	this.highlightedOption = index;
	this.optionRows[this.highlightedOption].className = 'highlight';
};
tersus.MenuButton.prototype.clearHighlight = function()
{
	if (this.highlightedOption != null)
	{
		this.optionRows[this.highlightedOption].className = '';
		this.highlightedOption = null;
	}
};
tersus.MenuButton.prototype.select = function ()
{
	if (checkBusy())
		return;
	this.deactivate();
	this.value = this.options[this.highlightedOption];
	this.queueEvent(Events.ON_SELECT);
};
tersus.MenuButton.mousedown = function()
{
	var node = DisplayNode.findNodeForDocumentNode(this);
	if (node && node.activate)
		node.activate();
};
tersus.MenuButton.blur = function()
{
	var node = DisplayNode.findNodeForDocumentNode(this);
	if (node && node.isActive && !node.mouseIn)
		setTimeout(function(){	node.deactivate();},50);
};
tersus.MenuButton.prototype.deactivate = function ()
{
	var t = this.optionsTable;
	if (t && t.parentNode)
		t.parentNode.removeChild(t);
	this.optionsTable = null;
	this.isActive = false;
	if (window.isWebKit)
	{
		tersus.MenuButton.currentNodeId = null;
		var w=this.currentWindow;
		w.document.removeEventListener('click',tersus.MenuButton.deactivator, false);
	}
};
tersus.MenuButton.deactivator = function()
{
	var node = tersus.findNode(tersus.MenuButton.currentNodeId);
	if (!node)
		return;
	if (node && ! node.mouseId)
		node.deactivate();
};
tersus.MenuButton.prototype.activate = function ()
{
	if (this.isActive)
		return;
	if (checkBusy())
		return;
	this.isActive = true;
	if (window.isWebKit)
	{
		var current = tersus.findNode(tersus.MenuButton.currentNodeId);
		if (current)
			current.deactivate();
		var w = this.currentWindow;
		tersus.MenuButton.currentNodeId = this.getNodeId();
		setTimeout(function()
		{
			w.document.addEventListener('mouseup',  tersus.MenuButton.deactivator,false);
		},50);
	}
	this.highlightedOption = null;
	this.mouseIn = false;
	var t = this.optionsTable = this.createHTMLElement('table');
	t.className='options';
	var tb = this.optionsTbody = this.createHTMLChildElement(t,'tbody');
	this.optionRows = [];
	for (var i=0; i<this.options.length ;i++)
	{
		var row = this.createHTMLChildElement(tb,'tr');
		row.onmouseOver = tersus.MenuButton.highlight;
		row.onmouseOut = tersus.MenuButton.unhighligh;
		row.setAttribute('optionIndex', i);
		row.onmouseover = tersus.MenuButton.highlight;
		row.onmouseout = tersus.MenuButton.unhighlight;
		row.onmouseup =  tersus.MenuButton.select;
		row.setAttribute('nodeId',this.getNodeId());
		var cell = this.createHTMLChildElement(row,'td');
		cell.innerHTML = this.translate(tersus.escapeHTML(this.getOptionText(this.options[i])));
		this.optionRows[i] = row;
		
	}
	var backgroundColor = getActualBackground(this.viewNode);
	if (backgroundColor == null) backgroundColor = 'white';
	t.style.backgroundColor=backgroundColor;
	
	t.style.position='absolute';
	$(t).css('z-index',++ tersus.topZindex)
	var d = this.viewNode.ownerDocument;
	d.body.appendChild(t);
	d.body.appendChild(t);
	var buttonPosition=getScreenPosition(this.viewNode);
	var x = buttonPosition.left;
	var y = (buttonPosition.top+this.viewNode.offsetHeight-1);
    t.style.top= (y - getScroll(t).top)+'px';
    t.style.left = (x - getScroll(t).left)+'px';
};

tersus.MenuButton.prototype.addOption = tersus.MenuButton.prototype['add<Options>'] = function tersus_MenuButton_addOption(value)
{
	if (!this.options) this.options=[];
	this.options.push(value);
};
tersus.MenuButton.prototype.addOption.requiresNodes = true;

tersus.MenuButton.prototype.getOptions = tersus.MenuButton.prototype['get<Options>'] = function tersus_MenuButton_getOptions()
{
	if (!this.options) this.options=[];
	return this.options;
};
tersus.MenuButton.prototype.removeOptions = tersus.MenuButton.prototype['remove<Options>'] = function tersus_MenuButton_removeOptions()
{
	this.options = [];
};
// file: MenuItem.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.MenuItem = DisplayNode.__subclass('tersus.MenuItem');
tersus.MenuItem.prototype.onload = function () {
 this.imagePath = this.sprop('imagePath');
 };
tersus.MenuItem.prototype.defaultTag = 'span';
tersus.MenuItem.prototype.defaultStyleClass='menu_item';
tersus.MenuItem['set<Image Path>'] = function(path)
{
	this.imagePath = path;
	if (this.image)
		this.image.src = path;
};
function menuOption()
{
};
tersus.MenuItem.prototype.createViewNode = function()
{
	DisplayNode.prototype.createViewNode.call(this);
	if (this.imagePath)
	{	
		this.image = this.createHTMLChildElement(this.viewNode,'img');
		this.image.src = this.imagePath;
	}
	this.link = this.createHTMLChildElement(this.viewNode,'a');
    this.link.href='javascript:menuOption("'+this.getCaption()+'")';
    this.link.onfocus = function(){this.className='focus';};
    this.link.onblur = function(){this.className='';};
		
	this.captionChanged();
};
tersus.MenuItem.prototype.captionChanged = function()
{
	if (this.link)
		this.link.innerHTML = this.escapeHTML(this.getCaption());
};
// file: Minimum.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Minimum = function tersus_Minimum() {};
tersus.Minimum.prototype = new ActionNode();
tersus.Minimum.prototype.constructor = tersus.Minimum;
tersus.Minimum.prototype.start = function ()
{
	var values = this.getTriggerValues();
	if (values.length == 0)
	{
		modelExecutionError("No values specified - can't compute minimum",this);
		return;
	}
	var minimum = values[0].leafValue;
	for (var j=1; j<values.length; j++)
	{
		var value = values[j];
		if (value && value.leafValue < minimum)
			minimum = value.leafValue;
	}
	this.chargeLeafExit('<Minimum>',minimum);
};
// file: Model.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Model = function tersus_Model() {};
tersus.Model.prototype = new DataValue();
tersus.Model.prototype.constructor = tersus.Model;
tersus.Model.prototype.pluginFunc = tersus.Model;
tersus.Model.prototype.toString = function()
{
	return 'Model[id='+this.model.modelId+']';
};
tersus.Model.prototype.addGetter('Plugin',function()
{
	if (! this.model)
	{
		modelError("Invalid instnace of "+this.modelId+"  Use 'Get Model' to obtain models and model elements");
		return null;
	}
	return this.model.plugin;
});
tersus.Model.prototype.addGetter('Model Id', function()
{
	if (! this.model)
	{
		modelError("Invalid instnace of "+this.modelId+"  Use 'Get Model' to obtain models and model elements");
		return null;
	}
	return this.model.modelId;
});
tersus.Model.prototype.addGetter('Model Name', function()
{
	if (! this.model)
	{
		modelError("Invalid instnace of "+this.modelId+"  Use 'Get Model' to obtain models and model elements");
		return null;
	}
	return this.model.modelName;
});
tersus.Model.prototype.addGetter('Elements',function()
{
	if (! this.model)
	{
		modelError("Invalid instnace of "+this.modelId+"  Use 'Get Model' to obtain models and model elements");
		return [];
	}
	if (! this.elementsWrappers)
	{
		var elementsElement = this.getElement('Elements');
		this.elementWrappers = [];
		if (this.model.elementList)
		{
			for (var i=0;i<this.model.elementList.length;i++)
			{
				var element = this.model.elementList[i];
				if (!element.wrapper)
				{
					element.wrapper = elementsElement.createChildInstance();
					element.wrapper.element = element;
				}
				this.elementWrappers.push(element.wrapper);
			}
		}
	}
	return this.elementWrappers;
});
// file: ModelElement.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.ModelElement = function tersus_ModelElement() {};
tersus.ModelElement.prototype = new DataValue();
tersus.ModelElement.prototype.constructor = tersus.ModelElement;
tersus.ModelElement.prototype.pluginFunc = tersus.ModelElement;
tersus.ModelElement.prototype.toString = function()
{
	return 'ModelElement[name='+this.element.role+',owner='+this.owner.modelId + ",modelId="+this.element.modelId+']';
};
tersus.ModelElement.prototype.addGetter('Model Id',function()
{
	if (! this.element)
	{
		modelError("Invalid instnace of "+this.modelId+"  Use 'Get Model' to obtain models and model elements");
		return null;
	}
	return this.element.modelId;
});
tersus.ModelElement.prototype.addGetter('Element Name',function()
{
	if (! this.element)
	{
		modelError("Invalid instnace of "+this.modelId+"  Use 'Get Model' to obtain models and model elements");
		return null;
	}
	return this.element.role;
});

tersus.ModelElement.prototype.addGetter('Repetitive',function()
{
	if (! this.element)
	{
		modelError("Invalid instnace of "+this.modelId+"  Use 'Get Model' to obtain models and model elements");
		return null;
	}
	return this.element.repetitive;
});
tersus.ModelElement.prototype.addGetter('Type',function()
{
	if (! this.element)
	{
		modelError("Invalid instnace of "+this.modelId+"  Use 'Get Model' to obtain models and model elements");
		return null;
	}
	if (this.element.isDisplayElement)
		return "Display";
	else if (this.element.isSubFlow)
		return "SubProcess";
});
// file: Multiply.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Multiply = function tersus_Multiply() {};
tersus.Multiply.prototype = new ActionNode();
tersus.Multiply.prototype.constructor = tersus.Multiply;
tersus.Multiply.prototype.start = function ()
{
	var values = this.getTriggerValues();
	var product = 1;
	for (var j=0; j<values.length; j++)
	{
		var value = values[j];
		if (value.leafValue != null)
			product *= parseFloat(value.leafValue);
	}
	this.chargeLeafExit('<Product>',product);
};
// file: Negative.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Negative = function tersus_Negative() {};
tersus.Negative.prototype = new ActionNode();
tersus.Negative.prototype.constructor = tersus.Negative;
tersus.Negative.prototype.start = function ()
{
	var x = this.getLeafChild('<X>');
	if (x<0)
		this.chargeLeafExit('<Yes>',x)
	else
		this.chargeLeafExit('<No>',x);
};
// file: NegativeOrZero.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.NegativeOrZero = function tersus_NegativeOrZero() {};
tersus.NegativeOrZero.prototype = new ActionNode();
tersus.NegativeOrZero.prototype.constructor = tersus.NegativeOrZero;
tersus.NegativeOrZero.prototype.start = function ()
{
	var x = this.getLeafChild('<X>');
	if (x<=0)
		this.chargeLeafExit('<Yes>',x)
	else
		this.chargeLeafExit('<No>',x);
};
// file: NewTree.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.NewTree = function NewTree(){};
tersus.NewTree.prototype = new DisplayNode();
tersus.NewTree.prototype.constructor = tersus.NewTree;
tersus.NewTree.prototype.defaultTag = 'div';
tersus.NewTree.prototype.defaultStyleClass = 'ntree';
tersus.NewTree.prototype.isNewTree = true;
tersus.NewTree.prototype['get<Selected Item>'] = function()
{
	return this.selectedItem;
};
tersus.NewTreeItem = function NewTreeItem(){};
tersus.NewTreeItem.prototype = new DisplayNode();
tersus.NewTreeItem.prototype.constructor = tersus.NewTreeItem;
tersus.NewTreeItem.prototype.defaultTag = 'div';
tersus.NewTreeItem.prototype.defaultStyleClass = 'ntree_item';
tersus.NewTreeItem.TREE_ITEM_ID="tree_item_id";
tersus.NewTreeItem.prototype.COLLAPSED = 'Collapsed';
tersus.NewTreeItem.prototype.EXPANDED = 'Expanded';
tersus.NewTreeItem.prototype.ATOMIC = 'Atomic';
tersus.NewTreeItem.prototype.isNewTreeItem = true;
tersus.NewTreeItem.prototype.getTree = function()
{
	var parent = this.parent;
	while (parent.isNewTreeItem)
	{
		parent = parent.parent;
	};
	if (parent && parent.isNewTree)
	{
		return parent;
	}
	else
		return null;
};
tersus.NewTreeItem.select = function()
{
	var itemId = this.getAttribute(tersus.NewTreeItem.TREE_ITEM_ID);
	var item = tersus.findNode(itemId);
	item.select();
};

tersus.NewTreeItem.prototype.select = function()
{
	var tree = this.getTree();
	if (tree)
	{
		if (tree.selectedItem)
			tree.selectedItem.deselect();
		tree.selectedItem = this;
		if (this.innerHeader)
			this.innerHeader.viewNode.className = "ntree_item_inner_header_selected";
	};
};
tersus.NewTreeItem.prototype.deselect = function()
{
		if (this.innerHeader)
			this.innerHeader.viewNode.className = "ntree_item_inner_header";
};
tersus.NewTreeItem.prototype.deleteMe = function()
{
	DisplayNode.prototype.deleteMe.call(this);
	var tree = this.getTree();
	if (tree && tree.selectedItem == this)
		tree.selectedItem = null;
};
tersus.NewTreeItem.prototype.createViewNode = function()
{
	DisplayNode.prototype.createViewNode.call(this);
	this.header = this.createHTMLChildElement(this.viewNode,'span','ntree_item_header');
	this.toggleImage = this.createHTMLChildElement(this.header, 'img', 'ntree_toggle_collapsed');
	this.body = this.createHTMLChildElement(this.viewNode, 'div', 'ntree_item_body');
};
tersus.NewTreeItem.prototype.initViewNode = function()
{
	if (this.defaultStyleClass)
		this.viewNode.className = this.defaultStyleClass;
	this.setProperties();
	this.toggleImage.src = 'images/spacer.gif';
	this.setEventHandlers();
	this.setState(this.COLLAPSED);
};
tersus.NewTreeItem.prototype.onClick = function(domNode, callback)
{
	if (domNode == this.toggleImage)
		this.toggle(callback);
	else
		DisplayNode.prototype.onClick.call(this, callback);
};
tersus.NewTreeItem.prototype.toggle = function(callback)
{
	if (this.state == this.COLLAPSED)
		this.expand(callback);
	else if (this.state == this.EXPANDED)
		this.collapse(callback);
};
tersus.NewTreeItem.prototype.collapse = function(callback)
{
	queueEvent(window.engine, window.engine.handleEvent, [this,Events.ON_COLLAPSE], new Date(), "Tree Item Collapsed", callback);
};
tersus.NewTreeItem.prototype.expand = function(callback)
{
	queueEvent(window.engine, window.engine.handleEvent, [this,Events.ON_EXPAND], new Date(), "Tree Item Expanded", callback);
};
tersus.NewTreeItem.prototype.setEventHandlers = function ()
{
	this.registerEventHandler(Events.ON_CLICK, this.toggleImage);
};
tersus.NewTreeItem.prototype.appendChildViewNode = function (child, childElement)
{
	if (child.element.role == '<Header>')
	{
		this.header.appendChild(child.viewNode);
		child.viewNode.className='ntree_item_inner_header';
		this.innerHeader = child;
		child.viewNode.setAttribute(tersus.NewTreeItem.TREE_ITEM_ID, this.getNodeId());
		child.registerEventHandler(Events.ON_CLICK);
		var item = this;
		var originalClickHandler = child.onClick;
		child.onClick = function(domNode)
		{
			item.select();
			originalClickHandler.call(child, domNode);
		};
		
	}
	else
		DisplayNode.appendChildNode(this.body,child.viewNode);
};

tersus.NewTreeItem.prototype['get<Expanded>'] = function()
{
	return (this.expanded);
};

tersus.NewTreeItem.prototype.setState = function(state)
{
	this.state = state;
	if (state == this.EXPANDED)
	{
		this.expanded = true;
		this.expandable = true;
		this.toggleImage.className = 'ntree_toggle_expanded';
		this.body.style.display='';
	}
	else if (state == this.COLLAPSED)
	{
		this.expanded = false;
		this.expandable = true;
		this.toggleImage.className = 'ntree_toggle_collapsed';
		this.body.style.display='none';
	}
	else if (state == this.ATOMIC)
	{
		this.expandable = false;
		this.toggleImage.className = 'ntree_toggle_atomic';
		this.body.style.display='none';
	}
};
tersus.NewTreeItem.prototype['set<Expanded>'] = function(expanded)
{
	if (expanded == this.expanded)
		return;
	if (this.expandable)
	{
	
		if (expanded)
			this.setState(this.EXPANDED);
		else 
			this.setState(this.COLLAPSED);
	}
	else 
	{
		this.expanded = expanded;
		this.setState(this.ATOMIC);
	}
	
};
tersus.NewTreeItem.prototype['get<Expandable>'] = function ()
{
	return (this.expandable);
};

tersus.NewTreeItem.prototype['set<Expandable>'] = function (expandable)
{
	if (expandable == this.expandable)
		return;
	if (expandable)
	{
		if (this.expanded)
			this.setState(this.EXPANDED);
		else
			this.setState(this.COLLAPSED);
	}
	else
	{
		this.setState(this.ATOMIC);
	};
};
// file: Now.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('Now', function()
{
	
	this.chargeLeafExit('<Now>',new Date());
});
// file: ParseJSON.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('ParseJSON', function()
{
	var text = this.getLeafChild('<JSON Text>');
	var exit = this.getExit('<Value>');
	
	
	var obj;
	try
	{
	  	obj = eval('var x='+text+';x;');
	}
	catch (e)
	{
		
		
		throw new tersus.Exception("Error parsing JSON Text ("+e+") text:\n"+text,e);
	}
	if (exit.isRepetitive)
	{
	    var values = obj;
    	for (var i=0; i< values.length; i++)
    	{
    		var dataValue  = exit.createChild(this);
    	    dataValue.copyObject(values[i]);
    	}
	}
	else
	{
		var dataValue = exit.createChild(this);
		dataValue.copyObject(obj);
	}
	
});// file: ParseXML.js
tersus.newAction('ParseXML', function()
{
	var xml = this.getLeafChild('<XML Document>');
	var exit = this.getExit('<Parsed Tree>');
	var allowUnknownElements = false; // When adding trigger, make sure to add equivalent in server side
	
	var doc;
	if (window.DOMParser)
	{
		var parser = new DOMParser();
		doc = parser.parseFromString (xml, "text/xml");
	} 
	else if (window.ActiveXObject)
	{
	    doc = new ActiveXObject ("Microsoft.XMLDOM");
	    doc.async = false;
	    doc.loadXML(xml);
    }
 	
 	var node = this.convert(doc.documentElement,exit, allowUnknownElements);
 	if (!node.isLeaf || node.leafValue != null)
 		this.chargeExit(exit,node);	
});
tersus.ParseXML.prototype.convert = function (domElement, element, allowUnknownElements)
{
	var text;
	var nodeConstructor = element.getChildConstructor();
	var prototype = nodeConstructor.prototype;
	var node = null;
	if (prototype.isLeaf)
		text = [];
	else
	{
		node = new nodeConstructor(); // Composite is always created
		for (var i=0;i<domElement.attributes.length;i++)
		{
			var attribute = domElement.attributes[i];
			if (attribute.value)
			{
				var localName = attribute.localName ? attribute.localName : attribute.baseName; // baseName is used in IE
				var elementHandler = this.getElementHandler(node,attribute.nodeName, attribute.localName, attribute.namespaceURI, true);
				if (elementHandler != null)
				{
			 		try
					{
						elementHandler.createChild(node).parseString(attribute.value, elementHandler);
					}
					catch (e)
					{
						throw {message:'Error parsing XML attribute: tagName='+domElement.tagName+' attribute:'+attribute.nodeName+ ' modelId='+elementHandler.modelId+'\nmessage='+e.message};
					}
				}
				else if (!allowUnknownElements && attribute.name.indexOf('xmlns')!=0)
			 		throw {message:'Error parsing XML: Unrecognized attribute ' + attribute.name + ' in '+prototype.modelId};
				
			}
		}		
	}
	for (var i=0;i<domElement.childNodes.length;i++)
	{
		var cn = domElement.childNodes[i];
		if (cn.nodeType == 1 || cn.nodeType == 2)
		{
			var localName = cn.localName ? cn.localName : cn.baseName; //naseName in IE
			var elementHandler = this.getElementHandler(node,cn.nodeName, localName, cn.namespaceURI, cn.nodeType==2);
			if (elementHandler)
			{
				if (cn.nodeType == 1)
				{
					var childNode = this.convert(cn, elementHandler, allowUnknownElements);
					if (childNode != null)
						elementHandler.addChild(node, childNode); 
				}
			}
			else if (!allowUnknownElements)//TODO support permissive parsing
			 	throw {message:'Error parsing XML: Unrecognized element ' + cn.nodeName + ' in '+prototype.modelId};
		}
		else if (text && cn.nodeType == 3) // Text Node
		{
			if (cn.nodeValue)
				text.push(cn.nodeValue);
		}
	}
	if (text && text.length>0) // Leaf node with actual text
	{
		node = new nodeConstructor();
		try
		{
			node.parseString(text.join(''), element);
		}
		catch (e)
		{
			throw {message:'Error parsing XML content: tagName='+domElement.tagName+' modelId='+node.modelId+'\nmessage='+e.message};
		}
	}
	return node;
};

tersus.ParseXML.prototype.getElementHandler = function(parent, name, localName, namespaceURI, isAttribute)
{
	if (!parent.elements)
		return;
	if (namespaceURI != null)
	{
		var elementName = isAttribute ?  '#'+localName : localName;
		for (var elementRole in parent.elements)
		{
			var e = parent.elements[elementRole];
			if (! e.xmlNamespace)
			{
				e.xmlNamespace = e.getChildConstructor().prototype.xmlNamespace;
			}
			if (e.xmlNamespace == namespaceURI)
			{
				if (! e.xmlName)
					e.xmlName = elementRole.substr(elementRole.lastIndexOf(':')+1);
				if (e.xmlName == elementName)
					return e;
			} 
		}
	};
	// Not found with explicit namespaceURI - search by raw name
	var elementName = isAttribute ?  '#'+name : name;
	return parent.elements[elementName];
};
// file: PasswordField.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.PasswordField = function tersus_PasswordField() {};
tersus.PasswordField.prototype = new tersus.GenericField();
tersus.PasswordField.prototype.constructor = tersus.PasswordField();
tersus.PasswordField.prototype.defaultTag = 'input';
tersus.PasswordField.prototype.updateSelfReadOnly = function tersus_GenericField_updateSelfOnly(readOnly)
{
	if (this.currentReadOnly == readOnly)
		return; // no change
	this.currentReadOnly = readOnly;
   	if (this.inputField)
       	this.value = this.getFieldValue();
	if (this.viewNode)
	{
		if (readOnly)
		{
			this.viewNode.disabled = true;
		}
		else
			this.viewNode.disabled = false;
	}
	this.setValueString();
};

tersus.PasswordField.prototype.createViewNode = function tersus_GenericField_createViewNode()
{
    DisplayNode.prototype.createViewNode.call(this);
    this.viewNode.type = 'password';
    this.inputField = this.viewNode;
};
tersus.PasswordField.prototype.setViewNode = function tersus_PasswordField_createViewNode(viewNode)
{
	this.viewNode = this.inputField = viewNode;
}

tersus.PasswordField.prototype.setEventHandlers =DisplayNode.prototype.setEventHandlers;
// file: PercentDisplay.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.PercentDisplay = function tersus_PercentField(){};
tersus.PercentDisplay.prototype = new tersus.DisplayField();
tersus.PercentDisplay.prototype.constructor = tersus.PercentDisplay;
tersus.PercentDisplay.prototype.setDefaultProperties = function()
{
	this.sharedProperties.showPercentage = true;
};
// file: PlaySound.webkit.js
tersus.newAction('PlaySound',function()
{
	var url=this.getLeafChild('<URL>');
	var audio = new Audio(url);
	audio.play();
});
// file: PopupCalendar.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 

function PopupCalendar(){};
PopupCalendar.prototype = new FlowNode();
PopupCalendar.prototype.constructor = PopupCalendar;

PopupCalendar.prototype.start = function()
{
	//Find the containing view node - to get the location
	var visualParent = this.parent;
	while (! visualParent.viewNode)
		visualParent = visualParent.parent;
	
	// Create a hidden field to hold the value and 'intercept' value change
	var hiddenField = visualParent.hiddenDateFieldForPopupCalendar;
	if ( ! hiddenField)
	{
		hiddenField = visualParent.createHTMLElement('input');
		hiddenField.type = 'hidden';
		this.currentWindow.document.body.appendChild(hiddenField);
		visualParent.hiddenDateFieldForPopupCalendar = hiddenField;
 	}
 	var date = this.getChild('<Base Date>');
 	if (date != null)
 		hiddenField.value = date.formatString();
	var node = this;
	var valueHandler = function handleDate(year, month, day)
	{
		var date = createDate(year, month, day);
		node.chargeLeafExit('<Date>', date);
		node.readyToResume();
		resumeRoot();
	};
	
//	this.popupStatus = PopupCalendar.WAITING_FOR_SELECTION;
	visualParent.currentWindow.popUpCalendar(visualParent.viewNode, hiddenField, valueHandler, DATE_FORMAT);
};// file: PopupCalendarWheels.iphone.non-simulator.js
PopupCalendar.prototype.start = function()
{
        var node = this;
        var callback = function datePickerCallback(year, month, day)
        {
        	if (year != null && month != null && day != null)
        	{
                var date = createDate(year, month, day);
                node.chargeLeafExit('<Date>', date);
             }
             node.readyToResume();
             resumeRoot();
        };
		
		var date = this.getLeafChild('<Base Date>');
		if (date == null)
		{
			var now = new Date();
			date = createDate(now.getFullYear(),now.getMonth()+1,now.getDate());
		}
        
		this.openDatePicker(date,callback);
};

PopupCalendar.prototype.openDatePicker = function(date, callback)
{
	var days = { };
	var years = { };
	var months = { 1: 'Jan', 2: 'Feb', 3: 'Mar', 4: 'Apr', 5: 'May', 6: 'Jun', 7: 'Jul', 8: 'Aug', 9: 'Sep', 10: 'Oct', 11: 'Nov', 12: 'Dec' };
	
	for( var i = 1; i < 32; i += 1 ) {
		days[i] = i;
	}

	for(var i = date.year-20; i < date.year+10; i += 1 ) {
		years[i] = i;
	}

	SpinningWheel.addSlot(years, 'right', date.year);
	SpinningWheel.addSlot(months, '', date.month);
	SpinningWheel.addSlot(days, 'right', date.day);
	SpinningWheel.setCancelAction(callback);
	SpinningWheel.setDoneAction(function()
	{
		try
		{
			var keys = SpinningWheel.getSelectedValues().keys;
			var year = parseInt(keys[0]);
			var month = parseInt(keys[1]);
			var day = parseInt(keys[2]);
			callback(year, month, day);
		}
		catch (e)
		{
			alert (e);
		}
	});
	SpinningWheel.open();
};// file: Positive.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Positive = function tersus_Positive() {};
tersus.Positive.prototype = new ActionNode();
tersus.Positive.prototype.constructor = tersus.Positive;
tersus.Positive.prototype.start = function ()
{
	var x = this.getLeafChild('<X>');
	if (x>0)
		this.chargeLeafExit('<Yes>',x)
	else
		this.chargeLeafExit('<No>',x);
};
// file: PositiveOrZero.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.PositiveOrZero = function tersus_PositiveOrZero() {};
tersus.PositiveOrZero.prototype = new ActionNode();
tersus.PositiveOrZero.prototype.constructor = tersus.PositiveOrZero;
tersus.PositiveOrZero.prototype.start = function ()
{
	var x = this.getLeafChild('<X>');
	if (x>=0)
		this.chargeLeafExit('<Yes>',x)
	else
		this.chargeLeafExit('<No>',x);
};
// file: Print.js
tersus.newAction('Print',function()
{
	var title = this.getLeafChild('<Title>');
	var heading = this.getLeafChild('<Heading>');
	var content = this.getChild('<Content>');
	var style = this.currentWindow.document.getElementById('main_stylesheet');
	var domNode = content.wrapperNode ? content.wrapperNode : content.viewNode;
	if (domNode == null)
		throw {message:'<Content> must be a visual display element (can\'t "data only" display elements)'};
	
	var values = {Title:title?tersus.escapeHTML(title):'', Heading: heading? '<h1>'+tersus.escapeHTML(heading)+'</h1>' : '', Content:tersus.outerHTML(domNode), Style: tersus.outerHTML(style) };
	var template='<html><head><title>${Title}</title>${Style}</head><body>${Heading}${Content}<script type="text/javascript">window.popupTest="OK";</script></body></html>';
	var html = template.replace(/[$]{([^{}]+)}/g,
        function (all, token) {
          return values[token];
        });
    var w = window.open();
    if (w == null || w.innerHeight == 0)
    {
    	throw {message:"Can't print because the browser blocks pop-up windows.\nPlease enable pop-up windows for this application."};
    }
    w.document.write(html);
    w.document.close();
    w.print();
    setTimeout(function(){w.close()},50);
});


tersus.outerHTML = function(node)
{
	var html;
	if (window.XMLSerializer)
	{
		return (new XMLSerializer()).serializeToString(node);
	}
	else if (node.outerHTML)
		return node.outerHTML;
	else
		throw {message:"Can't print: no outerHTML or XMLSerializer"};
};// file: Put.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Put = function tersus_Put() {};
tersus.Put.prototype = new ActionNode();
tersus.Put.prototype.constructor = tersus.Put;
tersus.Put.prototype.start = function ()
{
	var map = this.getChild("<Map>");
	if (map && map.leafValue)
		map = map.leafValue;
	
	var key = this.getChild("<Key>");
	if (key && key.leafValue != undefined)
		key = key.leafValue;
	var value = this.getChild("<Value>");
	if (value && value.leafValue != undefined)
		value = value.leafValue;
	if (map && (key != null && key != undefined))
	{
		if (value == null || value == undefined)
			delete map[key];
		else
			map[key]=value;
	}
};
// file: PutMapEntry.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('PutMapEntry', function()
{
	var map = this.getChild("<Map>");
	if (map && map.leafValue)
		map = map.leafValue;
	var key = this.getChild("<Key>");
	if (key && key.leafValue != undefined)
		key = key.leafValue;
	var value = this.getChild("<Value>");
	if (map && key != null)
	{
		if (value == null)
			delete map[key];
		else
			map[key]=value;
	}
});
// file: RadioButton.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.RadioButton = function tersus_RadioButton(){};
tersus.RadioButton.prototype = new tersus.PushButton();
tersus.RadioButton.prototype.constructor = tersus.RadioButton;
tersus.RadioButton.prototype.isRadioButton = true;
tersus.RadioButton.prototype.buttonType='radio';

tersus.RadioButton.prototype.getGroup = function()
{
	if (! this.group)
	{
		this.group = this.parent;
		while (this.group && ! this.group.isRadioButtonGroup)
		{
			this.group = this.group.parent;
		}
	}
	return this.group;
};
tersus.RadioButton.prototype.getButtonName = function()
{
	var group = this.getGroup();
	if (group)
		return group.getNodeId();
	else
		return this.getNodeId();
};
tersus.RadioButton.prototype.getButtonValue = tersus.RadioButton.prototype['get<Value>'] = function()
{
	if (this.value != undefined)
		return this.value;
	else
	{
		var c = tersus.repository.get(BuiltinModels.TEXT_ID);
		var node = new c();
        node.prototype = c.prototype;
        node.setLeaf(this.getCaption());
		return node;
	}
};
tersus.RadioButton.prototype['set<Value>'] = function (value)
{
	this.value = value;
};
tersus.RadioButton.prototype.labelClicked = function (domNode, callback)
{
	if (this.button && ! this.button.disabled)
	{
		if ( ! this.button.checked)
		{
			this.button.checked = true;
			this.onChange(domNode, callback);
		}
	}
};

tersus.RadioButton.prototype['set<Value>'].requiresNodes = true;
tersus.RadioButton.prototype.hasOnChange = true;
tersus.RadioButton.prototype.onChange = function(domNode, callback)
{
	if (this.button && this.button.checked)
	{
		var group = this.getGroup();
		if (group)
		{
			if (group && group.selectedButton == this)
				return; // was already checked
			group.selectedButton = this;
		}
		DisplayNode.prototype.onChange.call(this, domNode, callback);
		if (group)
			group.onChange(domNode, callback);
	}
	else
		DisplayNode.prototype.onChange.call(this);
	
};
tersus.RadioButton.prototype.setChecked =tersus.RadioButton.prototype['set<Checked>'] = function(checked)
{
	if (this.currentWindow)
	{
		var wasChecked = this.getChecked();
	}
	tersus.PushButton.prototype.setChecked.call(this,checked);
	if (checked && !wasChecked)
	{
		var group = this.getGroup();
		if (group)
			group.selectedButton = this;
	}
	else if (!checked && wasChecked)
	{
		var group = this.getGroup();
		if (group && group.selectedButton == this)
			group.selectedButton = null;
		
	}
};

// file: RadioButtonGroup.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.RadioButtonGroup = function tersus_RadioButtonGroup(){};
tersus.RadioButtonGroup.prototype = new tersus.DisplayGroup();
tersus.RadioButtonGroup.prototype.constructor = tersus.DisplayGroup;
tersus.RadioButtonGroup.prototype.isRadioButtonGroup = true;
tersus.RadioButtonGroup.prototype['get<Value>'] = function()
{
	if (! this.selectedButton)
		return null;
	else
		return this.selectedButton.getButtonValue();
};
tersus.RadioButtonGroup.prototype.setSelectedValue = tersus.RadioButtonGroup.prototype['set<Value>'] = function(value)
{
	var found = false;
	var op = function(node)
	{
		if (node.isRadioButton &&  compareValues(node.getButtonValue(), value)	)
		{	
			node.setChecked(true);
			found = true;
			return true; // stop traversal
		}
	};
	tersus.traverseHierarchy(this, DisplayNode.displayChildrenIterator, op);
	if ( ! found && tersus.settings.debug)
		modelExecutionError("Tried to set radio button group <Value> as '"+value+"', but the group contains no radio button with this value",this);
};
tersus.RadioButtonGroup.prototype['remove<Value>']= function()
{
	tersus.traverseHierarchy(this, DisplayNode.displayChildrenIterator, function (node){if (node.isRadioButton) node.setChecked(false);return true;});
};
tersus.RadioButtonGroup.prototype.setSelectedValue.requiresNodes = true;
// file: Range.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function Range() {};
Range.prototype = new ActionNode();
Range.prototype.constructor = Range;
Range.prototype.start = function ()
{
	var limit1 = parseInt(this.getLeafChild('<Limit 1>'));
	var limit2 = parseInt(this.getLeafChild('<Limit 2>'));
	
	var min = Math.min(limit1, limit2);
	var max = Math.max(limit1, limit2);
	for (var i = min; i<=max; i++)
	{
		this.setLeafChild('<Numbers>', i, Operation.ADD);
	}	
};
// file: ReadResource.js
tersus.newAction('ReadResource', function tersus_ReadResource_Start()
{
	var url = this.getLeafChild('<URL>');
	var exit = this.getExit('<Content>');
	if (exit.modelId != BuiltinModels.TEXT_ID)
		modelError('<Content> must be of type Text in client side',this);
	var notFoundExit = this.getExit('<Not Found>');
	var node = this;
	var errorCallback = null;
	var callback = function(text)
	{
		if (text != null)
			node.chargeLeafExit(exit,text);
		node.continueExecution();
	};
	var errorCallback = function (status, statusText, e)
	{
		if (notFoundExit)
			node.chargeEmptyExit(notFoundExit);
		else
			alert('Failed to load "'+url+'" - status='+status+' message='+statusText+' ex='+e);
			
		node.continueExecution();
			
	}
	this.pause();
	tersus.loadURL(url, callback, false, errorCallback, null); 
});// file: Refresh.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function Refresh() {};
Refresh.prototype = new ActionNode();
Refresh.prototype.constructor = Refresh;
Refresh.prototype.start = function ()
{
	var element = this.getChild('<Element>');
	if (element && element.refresh)
	{
		element.refresh();
		rootDisplayNode.onResize(); // Added for mobile widgets. Can probably be removed once we get rid of jQuery Mobile scripting
	}
		
};
// file: RelativeDate.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('RelativeDate', function()
{
	var date = tersus.getDate(this.getChild('<Base Date>')); // Using local timezone
	if (date == null)
		date = new Date();
        else
		date = new Date(date.getTime());
	var y = this.getLeafChild('<Years>');
	var m = this.getLeafChild('<Months>');
	var d = this.getLeafChild('<Days>');
	
	var oy = date.getFullYear();
	var om = date.getMonth();
	var od = date.getDate();
	if (y)
	{
		date.setFullYear(date.getFullYear()+y);
		if (om != date.getMonth()) // overflow because date was Feb 29th.  SetDate(0) brings us back to Feb 28th 
			date.setDate(0);
	}
	if (m)
	{
		date.setMonth(date.getMonth()+m);
		if (date.getDate() != od) // Month overflow - go back to last day of previous month
			date.setDate(0);
	}
	if (d)
		date.setDate(date.getDate()+d);
			
	this.chargeLeafExit('<Date>', createDate(date.getFullYear(), date.getMonth()+1, date.getDate())); //Using local timezone
	
});
// file: RelativeTime.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.RelativeTime = function tersus_Relative_Time() {};
tersus.RelativeTime.prototype = new ActionNode();
tersus.RelativeTime.prototype.constructor = tersus.RelativeTime;
tersus.RelativeTime.prototype.start = function ()
{
	var baseTime = this.getLeafChild('<Base Time>');
	if (baseTime == null)
		baseTime = new Date();
	else
		baseTime = new Date(baseTime.getTime());
	var seconds = this.getLeafChild('<Seconds>');
	var time = baseTime.getTime() +  seconds*1000;
	var newTime = new Date(time);
	this.chargeLeafExit('<Time>',newTime);
}
// file: ReloadApplication.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.ReloadApplication = function() {};
tersus.ReloadApplication.prototype = new ActionNode();
tersus.ReloadApplication.prototype.constructor = tersus.ReloadApplication;
tersus.ReloadApplication.prototype.start = function ()
{
	tersus.async.exec(tersus.reloadApplication, false /* don't wait */);
};
// file: RemoveNumberedItem.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
tersus.RemoveNumberedItem = function RemoveNumberedItem(){};
tersus.RemoveNumberedItem.prototype = new ActionNode();
tersus.RemoveNumberedItem.prototype.constructor = tersus.RemoveNumberedItem;
tersus.RemoveNumberedItem.prototype.start = function()
{
	var parent = this.getChild('<Parent>');
	if (!parent.elementList)
	{
		if (this.getElement('<Invalid Parent>'))
			this.chargeExit('<Invalid Parent>', parent);
		else
			modelExecutionError("Invalid parent "+parent.modelId,this);
		return;
	}
	
	var e = this.getTargetList(parent);
	if (!e)
	{
		if (this.getElement('<Invalid Parent>'))
			this.chargeExit('<Invalid Parent>', parent);
		else
			modelExecutionError("Invalid parent "+parent.modelId,this);
			
		return;
	}
	
	var index = this.getLeafChild('<Index>');
	if (index < 0 || index > e.getChildren(parent).length)
	{
		if (this.getElement('<Invalid Index>'))
			this.chargeLeafExit('<Invalid Index>', index);
		else
			modelExecutionError("Invalid index "+index,this);
		return;
	}
	
	if (this.getElement('<Removed Item>'))
	{
		var children = e.getChildren(parent);
		this.chargeLeafExit('<Removed Item>', children[index-1]);
	}
	
	e.removeNumberedChild(parent, index);
};

tersus.RemoveNumberedItem.prototype.getTargetList = function(parent)
{
	var elementName = this.getLeafChild('<Element Name>');
	if (parent && parent.elementList)
	{
		var elements = parent.elementList;
		for (var i=0;i<elements.length;i++)
		{
			if (elementName != null)
			{
				if (elements[i].isRepetitive && elementName == elements[i].role)
					return elements[i];
			}
			else if (elements[i].isRepetitive)
					return elements[i];
		}
	}
	return null;
};

// file: RemoveStyleClass.js
/*********************************************************************************************************
 * Copyright (c) 2003-2011 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/

tersus.newAction('RemoveStyleClass',function()
{	
	var element = this.getChild('<Element>');
	var value = this.getLeafChild('<Style Class>');
	if (!element || !value)
		return;
	var elementClass = element.getLeafChild('<Style Class>');
	if (!elementClass)
		return;

	elementClass = ' '+elementClass+' ';
	var classNames = value.split(/\s+/);
	for (var i=0;i<classNames.length;i++)
	{
		elementClass = elementClass.replace(' '+classNames[i]+' ',' ');
	}
	element.setLeafChild('<Style Class>',$.trim(elementClass), Operation.REPLACE);
});
// file: Replace.js
/*********************************************************************************************************
 * Copyright (c) 2003-2011 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at http://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
tersus.newAction('Replace', function()
{
	var text = this.getLeafChild('<Text>');
	var replace = this.getLeafChild('<Replace>');
	var by = this.getLeafChild('<By>');
	
	if (by == null)
		by = "";
	
	var i = 0;
	var b = []; //Buffer to collect result fragments
	while (i < text.length)
    {
    	var j = text.indexOf(replace,i);
    	if (j < 0)
    	{
    		b.push(text.substr(i));
    		break;
    	}
    	b.push(text.substr(i,j-i));
    	b.push(by);
    	i = j + replace.length;
    }
 
	this.chargeLeafExit('<Updated Text>',b.join(''));
});// file: Retrieve.js
/*********************************************************************************************************
 * Copyright (c) 2003-2011 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/

var m = tersus.baseURL.match('^(http|https)://[^/]+(:[0-9]+)?/([^/]*/)');
tersus.appScopePrefix =  (m && m.length>3)? m[3] : '';
tersus.newAction('Retrieve',function()
{	
	var key = this.getLeafChild('<Key>');
	var prefix = '';
	if (key)
	{
		var type = this.getLeafChild('<Storage Type>') || 'LOCAL';
		var s;
		if (type == 'LOCAL')
		{
			s = window.localStorage;
			if (s == null)
				throw 'No local storage available';
			key = tersus.appScopePrefix+key;
		}
		else if (type == 'SESSION')
		{
			s = window.sessionStorage;
			if (s == null)
				throw 'No session storage available';
		}
		var value = s.getItem(key,value);
		if (value != null && value.length>0)
			this.chargeLeafExit('<Value>',value);
		else
		{
			this.chargeEmptyExit('<None>');
		}
	}
});
// file: Round.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Round = function tersus_Round() {};
tersus.Round.prototype = new ActionNode();
tersus.Round.prototype.constructor = tersus.Round;
tersus.Round.prototype.start = function ()
{
	var number = this.getLeafChild('<Number>');
	if (number == null)
		modelExecutionError('Missng value for <Number>', this);
	var precision = this.getLeafChild('<Precision>');
	if (!precision)
		precision =0;
	if (precision!=Math.round(precision))
		modelExecutionError('<Precision> must be an integer', this);
	var factor = Math.pow(10, precision);
	var rounded = Math.round(number * factor)/factor;
	this.chargeLeafExit('<Rounded>',rounded);
};
// file: RunJavascript.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('RunJavascript', function()
{
	var scriptText = this.getLeafChild('<Script Text>');
	var triggerList = [];
	var valueList = [];
	var output;
	var exitList = [];
	var exitMap = {};
	triggerList.push('_node');
	valueList.push(this);
	for (var i=0; i<this.triggers.length; i++)
	{
		var trigger = this.triggers[i];
		if (trigger.role.charAt(0) != '<' ) // ignore 'special' triggers
		{
			triggerList.push(tersus.sqlize(trigger.role));
			//currently handling only leaf data types
			if (trigger.isRepetitive)
			{
				var array = [];
				var values = trigger.getChildren(this);
				var l = values.length;
				for (var j=0;j<l;j++)
				{
					var v = values[j];
					if (v != null)
						array.push(v.leafValue);
				}
				valueList.push(array);
			}
			else
			{
				var value = this.getLeafChild(trigger.role);
				valueList.push(value);
			}
		}
	}
	var functionText = 'var func = function('+triggerList.join(',')+'){';
	var packingText = '';
	functionText += '\nvar _out={};';
	if ( this.exits)
	{
		for (var i=0; i<this.exits.length; i++)
		{
			var exit = this.exits[i];
			if (exit.role.charAt(0) != '<') // ignore 'special' exits
			{
				var varName= tersus.sqlize(exit.role)
				functionText+='var '+varName+';'
				packingText+='_out["'+exit.role+'"]='+varName+';'
			}
		}
	}
	functionText += scriptText;
	functionText += ';'
	functionText += packingText;
	functionText+='return _out;};\nfunc;';
	var f  = eval(functionText);
	var out = f.apply(this,valueList);
	for (var exitName in out)
	{
		var v=out[exitName];
		if(v!=null)
		{
			var exit = this.getExit(exitName);
			if (exit.isRepetitive)
			{
				var l = v.length;
				for (var j=0;j<l;j++)
				{
					this.chargeLeafExit(exit, v[j]);
				}
			}
			else
			{
				this.chargeLeafExit(exitName, v);
			}
		}
	};
});		

// file: Scrollable.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function Scrollable() {};
Scrollable.prototype = new tersus.HTMLTag();
Scrollable.prototype.constructor = Scrollable;
Scrollable.prototype.defaultTag = 'div';
Scrollable.prototype.defaultStyleClass  = 'Scrollable';

//Scrollable.prototype.create = function()
Scrollable.prototype.copyValueOpt = function(value, nodeMap, htmlV, legacyNodes)
{
	
	tersus.HTMLTag.prototype.copyValueOpt.call(this,value, nodeMap, htmlV, legacyNodes)
	var scrollable = this;
	tersus.async.exec(function() {scrollable.calliScroll();} , false /* don't wait */);
};

Scrollable.prototype.calliScroll = function()
{
	if ( iScroll)
	{
		new iScroll(this.viewNode.firstChild);
	
	}
}
// file: SelectionField.js
var setter = tersus.setter;

tersus.Widget('SelectionField','HTMLTag',{
	defaultTag:'div'
	,jQuery:'fieldcontain'
	,templateData: function()
	{
		var n=this;
		var options=$(this.options).map(function(i,o){
													return {val:(o==null ? null:i), text:n.translate(n.getOptionText(o))};
													 });
		var btnIconSide = $(document.body).css('direction')=='rtl' ? 'left':'right';
		return {options:options, text:this.translate(this.getOptionText(this.value)), btnIconSide:btnIconSide};
	}
	,reset: function()
	{
		this.$select = null;
		this.supr('reset');
	}

	,template: '{{if caption}}<label for="${id}">${caption}</label>{{/if}}<div class="ui-select">'+
	'<div class="ui-btn ui-btn-icon-${btnIconSide} ui-btn-corner-all ui-shadow ui-btn-down-c ui-btn-up-c">'+
	'<span class="ui-btn-inner ui-btn-corner-all"><span class="ui-btn-text">${text}</span><span class="ui-icon ui-icon-arrow-d ui-icon-shadow"></span></span>'+
	'<select id="${id}">{{each(i,opt) options}}<option {{if (opt.val != null)}}value="${opt.val}"{{/if}}>${opt.text}</option>{{/each}}</select>'+
	'</div></div>'
	,attach:function()
	{
		 var self = this;
		 this.$select = $(this.viewNode).find('select');
		 this.$select.blur(function(){$.mobile.silentScroll(0);});
		 this.$buttonText = $(this.viewNode).find('.ui-btn-text');
		 this.$select.change(function(){
		 	self.jqRefresh();
		 	});
		 this.setSelectedIndex(this.findIndex(this.value));
		 self.jqRefresh();
	}
	,jqRefresh: function()
	{
		this.$buttonText.html(this.translate(this.getOptionText(this.getValue())));
/*		var s = this.$select.data('selectmenu');
		if (s && s.refresh)
			s.refresh(true);*/
			
	}
	,'set<Value>': setter(function(value)
	{
		this.value = value;
		if (this.$select)
		{
			this.setSelectedIndex(this.findIndex(this.value));
			this.jqRefresh();
		}
	},true)
	,'remove<Value>': function()
	{
		if (this.options && this.options.length>0)
			this.value = this.options[0];
		if (this.$select)
		{
			this.setSelectedIndex(0);
			this.jqRefresh();
		}
	}
	,getValue:function()
	{
		var selectedIndex;
//		if (this.tmpSelectedIndex != null) // In IE, the selected index is updated asynchronously, so we use tmpSelectedIndex if it is set
//			selectedIndex = this.tmpSelectedIndex;
//		else
		var selectedIndex = this.$select[0].selectedIndex;
		if (selectedIndex < 0)
			return null;
		return this.options[selectedIndex];
	}
	,'get<Value>':function()
	{
		return this.getValue();
	}	
	,findIndex: function(value)
	{
		if (this.options)
			for (var i=0; i<this.options.length; i++)
			{
				var option = this.options[i];
				if (compareValues(option, value))
					return i;
			}
		return -1;
	}
	,setSelectedIndex: function(index)
	{
		this.$select[0].selectedIndex = Math.max(index,0);
	}
	,'get<Placeholder Text>': function(){return this.emptyOptionText;}
	,'set<Placeholder Text>': function(text)
	{
		var isNew = this.emptyOptionText == null;
		this.emptyOptionText = text;
		if (isNew)
		{
			if (this.options)
				this.options.splice(0,0,null); // add null to beginning of list
			else
				this.options = [null];
		}
		if (this.$select)
		{
			if (isNew)
				this.$select.prepend($("<option>").text(this.translate(this.getOptionText(null))));
			else
				this.$select.find('option').first().text(this.translate(this.getOptionText(null)));
			if (this.value == null)
				this.setSelectedIndex(0);
			this.jqRefresh();
		}
	}
	,'remove<Placeholder Text>': function()
	{
		if (this.emptyOptionText != null)
		{
			this.emptyOptionText = null;
			this.options.splice(0,1); //remove first
			if (this.$select)
			{
				this.$select.first().remove();
				this.jqRefresh();
			}
		}
		
	}
	,'remove<Options>': function()
	{ 
		this.options = this.emptyOptionText == null ? [] : [null];
		if (this.$select)
		{
			var self = this;
			this.$select.children().filter(function(i){return i>0 || null == self.emptyOptionText;}).remove();
			this.jqRefresh();
		}
	}
	,'add<Options>':setter(function(option)
	 {
	 	if (!this.options)
	 		 this.options = [];
	 	this.options.push(option);
	 	if (this.$select)
	 	{
			this.$select.append($("<option>").text(this.translate(this.getOptionText(option))).val(this.options.length));
			if (this.value != null &&  compareValues(this.value, option) || this.options.length==1)
			{
				this.setSelectedIndex(this.options.length-1);
				this.jqRefresh();
			}
		}
	 },true)
    ,'get<Options>':function()
    {
    	if ( !this.options)
			return [];
			if (this.emptyOptionText != null)
			{
				return this.options.slice(1); // Ignoring the first item in the array, which is the empty option (null)
			}
		return this.options;
	}
	,getOptionText: function(value)
	{
        var outputValue = null;
        if (value && value.elementList)
        {
			var textElementRole = value.__textElementRole;
			if (textElementRole == null)
			{
				for (var i=0; i< value.elementList.length; i++)
				{
					var element = value.elementList[i];
					if (i==0)
					{
				        textElementRole = element.role; // If no text element is found, the first element is used
				        value.constructor.prototype.__textElement = element;
					}
					if (element.modelId == BuiltinModels.TEXT_ID)
					{
				        textElementRole = element.role;
				        value.constructor.prototype.__textElement = element;
				        break;
					}
				}
				value.constructor.prototype.__textElementRole = textElementRole;
			}
			if (textElementRole && value.children)
			        outputValue = value.children[textElementRole];
        }
        else
                outputValue = value;
        var text = '';
        if (outputValue != null)
                text =  outputValue.formatString(this.sharedProperties);
        else if (this.emptyOptionText != null)
                text = this.emptyOptionText;
        return text;
	}
});
// file: Separator.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function Separator(){};
Separator.prototype = new DisplayNode();
Separator.prototype.constructor = Separator;
Separator.prototype.defaultTag = 'hr';
// file: SequenceNumber.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.SequenceNumber = tersus.DatabaseAction.__subclass('tersus.SequenceNumber');
tersus.SequenceNumber.prototype.getTableName = function()
{
	var tableName = this.getLeafChild('<Table Name>');
	if (tableName == null)
		tableName = 'Sequence_Numbers';
	return tableName;
};
tersus.SequenceNumber.prototype.start = function()
{
	this.pause();
	if (this.tableOK)
	{
		this.run();
		return;
	}
	this.tableOK = true;
	this.getTableDef(this.getTableName(),function(tableDef)
	{
		if (tableDef != null)
			this.run();
		else
			this.execSql('CREATE TABLE '+this.getTableName()+' (Name TEXT PRIMARY Key, Value Integer)',[], this.run);
	},this);
};
tersus.SequenceNumber.prototype.run = function()
{
	var sql = 'SELECT Value from '+this.getTableName()+' WHERE Name=?';
	var key = this.getLeafChild('<Key>');
	if (key == null)
		key = this.modelName;
	
	var values = [key];
	this.execSql(sql, values, function(transaction,results)
	{
		var rows = results.rows;
		if (rows.length == 0)
		{
			this.chargeLeafExit('<Next>',1);
			this.execSql('INSERT INTO '+this.getTableName() +' (Name, Value) VALUES (?,?)', [key, 1], this.resumeContext);
		}
		else
		{
			var seq = rows.item(0).Value + 1;
			this.chargeLeafExit('<Next>',seq);
			this.execSql('UPDATE '+this.getTableName() + ' SET Value= ? Where Name = ?', [seq, key], this.resumeContext); 
		}
	});
};
// file: SetAlertMode.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.SetAlertMode = function tersus_SetAlertMode() {};
tersus.SetAlertMode.prototype = new ActionNode();
tersus.SetAlertMode.prototype.constructor = tersus.SetAlertMode;
tersus.SetAlertMode.prototype.start = function ()
{
	tersus.alertMode.interactive = this.getLeafChild('<Interactive>');
	if (tersus.alertMode.interactive)
		tersus.errors = null;
	else
		tersus.errors = [];
	tersus.alertMode.confirm = this.getLeafChild('<Confirm>');
}

// file: SetCookie.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function SetCookie() {};
SetCookie.prototype = new ActionNode();
SetCookie.prototype.constructor = SetCookie;
SetCookie.prototype.start = function ()
{
	var name = tersus.sqlize(this.getLeafChild('<Name>'));
	var value = this.getLeafChild('<Value>');
	var expires = this.getLeafChild('<Expires>');
	var domain = this.getLeafChild('<Domain>');
	var path = this.getLeafChild('<Path>');
	var secure = this.getLeafChild('<Secure>');
	
	if (tersus.isNative && window.localStorage)
	{
		localStorage.setItem('Cookie.'+name,value);
		return;
	}
	if (!expires)
		expires = new Date(2100,0,0);
	if (expires && expires.toGMTString)
		expires = expires.toGMTString();
	if (value == null)
		deleteCookie(name);
	else
		setCookie(name,value, expires, path, domain, secure);
};
// file: SetElement.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.SetElement = function tersus_SetElement() {};
tersus.SetElement.prototype = new ActionNode();
tersus.SetElement.prototype.constructor = tersus.SetElement;
tersus.SetElement.prototype.start = function ()
{
	var elementName = this.getLeafChild('<Element Name>');
	var parent = this.getChild('<Parent>');
	var value = this.getChild('<Value>');
	
	if (!parent)
	{
		modelExecutionError("Missing <Parent>",this);
		return;
	}
	if (!elementName)
	{
		modelExecutionError("Missing <Element Name>",this);
		return;
	}		
	var element = parent.getElement(elementName);
	if (element == null)
	{
		modelExecutionError("No element called "+elementName + " in "+ parent.modelId,this);
		return;
	}
	if (element.isRepetitive)
	{
		if (value != null)
			element.addAChild(parent,value);
		else
			element.removeChildren(parent);
	}
	else
	{
		if (value != null)
			element.replaceChild(parent,value, true /* reset display elements before copying new */);
		else
			element.removeChild(parent);
	}	
};
// file: SetStyleAttribute.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('SetStyleAttribute', function()
{
	var e = this.getChild('<Element>');
	var n = this.getLeafChild('<Attribute>');
	var a = tersus.getJsAttributeName(n);
	var v = this.getLeafChild('<Value>');
	if (e && e.viewNode && a)
	{
		 e.viewNode.style[a] = v;
	}
});

// file: Show.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function Show() {};
Show.prototype = new ActionNode();
Show.prototype.constructor = Show;
Show.prototype.start = function ()
{
	var item = this.getChild('<Item>');
	if (item.select)
		item.select();
	var scrollableNode = item.viewNode;
	while (scrollableNode)
	{
		if (scrollableNode == this.mainWindow.rootDisplayNode.viewNode)
		{
			scrollableNode = this.mainWindow.document.body; // workaround for a bug in detecting the scrollable node in IE
			break;
		}
		if (isScrollable(scrollableNode))
			break;
		else
			scrollableNode = scrollableNode.parentNode;
	}
	if (scrollableNode)
	{
		var s = scrollableNode;
		var sSize = getVisibleSize(s);
		var v = item.viewNode;
		/*
			offsetTop = position of top relative to containing 
		*/
		
		var visible = (getOffsets(v).top >= getOffsets(s).top + s.scrollTop)
					&& (getOffsets(v).top+v.offsetHeight <= getOffsets(s).top + s.scrollTop +sSize.height);
		if (! visible)
		{
			var scrollTarget;
			var scrollToTop = v.offsetTop - s.offsetTop;
			var scrollToMiddle = scrollToTop - sSize.height/2;
			var scrollToBottom = scrollToTop  + (sSize.height-v.offsetHeight);
			if (v.offsetHeight >= sSize.height)
				scrollTarget = scrollToTop;
			else if (v.offsetHeight >= sSize.height/2)
				scrollTarget =scrollToBottom;
			else
				scrollTarget = scrollToMiddle;
			var scrollTo = Math.max(0, scrollTarget);
			var scrollTo = Math.min(scrollTo, s.scrollHeight - sSize.height);
			s.scrollTop = scrollTo;
		}
		var u = function ()
		{
			var message = "";
			message += "viewNode.scrollTop="+v.scrollTop+"\n";
			message += "viewNode.scrollHeight="+v.scrollHeight+"\n";
			message += "viewNode.offsetTop="+v.offsetTop+"\n";
			message += "viewNode.offsetHeight="+v.offsetHeight+"\n";
			message += "scrollableNode.scrollTop="+scrollableNode.scrollTop+"\n";
			message += "scrollableNode.scrollHeight="+scrollableNode.scrollHeight+"\n";
			message += "scrollableNode.offsetTop="+scrollableNode.offsetTop+"\n";
			message += "scrollableNode.offsetHeight="+scrollableNode.offsetHeight+"\n";
			alert(message);
		};
		this.mainWindow.showInfo = u;
		this.mainWindow.scrollableNode = scrollableNode;
	}
};// file: SimpleTable.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 //Simple Table - A Table whos rows are based on a repetitive Data Element
function SimpleTable() {};
SimpleTable.prototype = new Table();
SimpleTable.prototype.constructor = SimpleTable;

SimpleTable.prototype.createHeader = function createHeader()
{
	if (! this.rowElement) return;
	var rowPrototype = tersus.repository.get(this.rowElement.modelId).prototype;
	if (! rowPrototype.elementList) return;
	var headerRow = this.createHTMLElement('tr');
	for (var i=0; i< rowPrototype.elementList.length; i++)
	{	
		var columnElement = rowPrototype.elementList[i];
		var label = this.translate(columnElement.role);
		var headerCell = this.createHTMLElement('th');
		headerCell.innerHTML = tersus.escapeHTML(label);
		if (i==0)
			headerCell.className='first';
		headerRow.appendChild(headerCell);
	}
	this.header = this.createHTMLElement('thead');
	var caption = this.createHTMLElement('caption');
	caption.innerHTML = tersus.escapeHTML(this.getCaption());
	this.header.appendChild(headerRow);
	this.viewNode.appendChild(caption);
	this.viewNode.appendChild(this.header);

		
};

// Overload addIntermediateDataElement - and create a SimpleTableRowElement instead
SimpleTable.prototype.addIntermediateDataElement = function addIntermediateDataElement(role, modelId)
{
	if (role.charAt(0) == '<' && this.getGetter(role))
	{
		element = new MethodElement(this, role, modelId);
		return this.addElement(element);
	}
	else
	{
		if (this.rowElement)
			modelError('A SimpleTable can only have one intemediate data element. '+this.modelId + ' has both "'+this.rowElement.role+'" and "'+role+'"');
		this.rowElement = new SimpleTableRowElement(role, modelId);
		return this.addElement(this.rowElement);
	}
};
SimpleTable.ROW_NODE_ID='rowNodeId';
SimpleTable.TABLE_NODE_ID='tableNodeId';

SimpleTable.prototype.isSimpleTable = true;
SimpleTable.handleRowClick = function()
{
	var htmlRow = this;
	var tableNode = tersus.findNode(htmlRow.getAttribute(SimpleTable.TABLE_NODE_ID));
	makeAssertion(tableNode != null,"Simple Table node not found");
	var rowId=htmlRow.getAttribute(SimpleTable.ROW_NODE_ID);
	var rowRecord = null;
	if (tableNode.rowElement)
	{
		var rowNodes=tableNode.rowElement.getChildren(tableNode);
		for (var i=0;i<rowNodes.length;i++)
		{
			if (rowNodes[i].getNodeId() == rowId)
			{
				rowRecord = rowNodes[i];
				break;
			}
		}
	}
	makeAssertion(rowRecord != null,"Simple Table row node not found");
	if (tableNode.selectedRow && tableNode.selectedHTMLRow)
		tableNode.selectedHTMLRow.className = null;
	tableNode.selectedHTMLRow = htmlRow;
	htmlRow.className = 'selected';
	tableNode.selectedRow = rowRecord;
};
//SimpleTableRowElement - An Element of SimpleTable that represents the row content.
//This is a special element for which the 'Add' operation creates rows to the table
function SimpleTableRowElement(role, modelId)
{
	this.role = role;
	this.modelId = modelId;
};
SimpleTableRowElement.prototype = new Element();
SimpleTableRowElement.prototype.constructor = SimpleTableRowElement;
SimpleTableRowElement.prototype.addAChild = function addAChild(parent, value)
{
	if (parent.currentWindow) // if parent is a real display node, add a row to the table
	{
		makeAssertion(parent.isSimpleTable, 'SimpleTableRowElement must be a child of SimpleTable');
		makeAssertion(this.isRepetitive, 'Row element of SimpleTable must be repetitive');
		//Create a body row 
		var row = parent.createHTMLElement('tr');
		var rowPrototype = tersus.repository.get(this.modelId).prototype;
		if (rowPrototype.elementList)
		{
			for (var i=0; i< rowPrototype.elementList.length;i++)
			{
				var cell = parent.createHTMLElement('td');
				var cellElement = rowPrototype.elementList[i];
				var cellValue = cellElement.getChild(value);
				if (cellValue && (cellValue.leafValue || cellValue.leafValue == 0 || cellValue == ''))
					cell.innerHTML = tersus.escapeHTML(''+cellValue.leafValue); // convert to String
				row.appendChild(cell);				
			}
		}
		parent.bodyNode.appendChild(row);
		var selectable = parent.sharedProperties.selectable == true; 
		if (selectable)
		{
			row.setAttribute(SimpleTable.ROW_NODE_ID,value.getNodeId());
			row.setAttribute(SimpleTable.TABLE_NODE_ID,parent.getNodeId());
			row.onclick = SimpleTable.handleRowClick;
		}
	}
	Element.prototype.addAChild.call(this, parent, value);

};
SimpleTableRowElement.prototype.removeChildren = function removeChildren(parent)
{
	Element.prototype.removeChildren.call(this, parent);
	if (parent.currentWindow && parent.viewNode) // if parent is a real display node, add a row to the table
	{
		var table=parent.viewNode;
		parent.selectedRow = null;
		if (table && table.tBodies.length)
		{
			var body = table.tBodies[0];
			while (body.rows.length > 0)
			{
				body.removeChild(body.rows[0]);
			}
		}
	}
};
// file: Smaller.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Smaller = function tersus_Smaller() {};
tersus.Smaller.prototype = new ActionNode();
tersus.Smaller.prototype.constructor = tersus.Smaller;
tersus.Smaller.prototype.start = function ()
{
	var x = this.getLeafChild('<X>');
	var y = this.getLeafChild('<Y>');
	if (x < y)
		this.chargeLeafExit('<Yes>',x);
	else
		this.chargeLeafExit('<No>',x);
};
// file: SmallerOrEqual.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.SmallerOrEqual = function tersus_SmallerOrEqual() {};
tersus.SmallerOrEqual.prototype = new ActionNode();
tersus.SmallerOrEqual.prototype.constructor = tersus.SmallerOrEqual;
tersus.SmallerOrEqual.prototype.start = function ()
{
	var x = this.getLeafChild('<X>');
	var y = this.getLeafChild('<Y>');
	if (x <= y)
		this.chargeLeafExit('<Yes>',x);
	else
		this.chargeLeafExit('<No>',x);

};
// file: Sort.js
tersus.newAction('Sort', function()
{
	var ORDER_BY='<Order By>';
	var items = this.getTriggerValues(ORDER_BY);
	var orderBy = this.getLeafChild(ORDER_BY);
	
	
	var comparator;
	if (orderBy == null || orderBy.match(/^asc(ending)?$/i))
	{
		comparator = function(o1,o2)
		{
			return tersus.compare(o1, o2);
		};
	}
	else if (orderBy.match(/^desc(ending)?$/i))
	{
		comparator = function(o1, o2)
		{
			return tersus.compare(o2, o1);
		};
	}
	else
	{
		var clauses = orderBy.split(/\s*,\s*/);
		var sortComponents = [];	
		for (var i=0; i< clauses.length; i++)
		{
			var clause = clauses[i];
			var path = clause;
			var desc  = false;
			var parts = clause.split(" ");
			if (parts.length>1)
			{
				var suffix = parts[parts.length-1];
				if (suffix.match(/^asc(ending)?$/i))
				{
					path = $.trim(clause.substring(0, clause.length-suffix.length));
				}
				else if (suffix.match(/^desc(ending)?$/i))
				{
					path = $.trim(clause.substring(0, clause.length-suffix.length));
					desc = true;
				}
			}			
			sortComponents.push(new tersus.Sort.Component(path, desc));
		}
		comparator = function(o1, o2)
		{
			for (var i=0;i<sortComponents.length;i++)
			{
				var cc = sortComponents[i].compare(o1,o2);
				if (cc !=0)
					return cc;
			}
			return 0;
		};

	}
	
	tersus.quicksort(items, comparator);
	
	var e = this.getExit('<Sorted Items>');
	for (var i=0;i<items.length;i++)
	{
		this.accumulateExitValue(e, items[i]);
	};
});
tersus.Sort.Component = function(pathStr, desc)
{
	this.path= new Path(pathStr);
	this.desc=desc;
};
tersus.Sort.Component.prototype.compare = function(o1, o2)
{
	if (this.desc)
	{
		var t = o1;
		o1 = o2;
		o2 = t;
	}
	if (o1 == null && o2 == null)
		return 0;
	if (o1 == null)
		return -1;
	if (o2 == null)
		return 1;
		
	var l1 = o1.getValues(this.path);
	var l2 = o2.getValues(this.path);
	if (l1.length > 1 || l2.length>1)
		throw "Can't compare repetitive elements";
	var obj1 = l1[0] || null;
	var obj2 = l2[0] || null;
	return tersus.compare(obj1, obj2);
};



tersus.swap=function(a, i, j) {	var t=a[i];a[i]=a[j];a[j]=t;};
tersus.partition = function(a, left, right, piv, comp)
{
	var pv=a[piv];
	tersus.swap(a, piv, right-1); // Put pivot value at the end
	var pos=left;
	for(var i=left; i<right-1; i++) {
		if(comp(a[i],pv)<=0) {
			tersus.swap(a, pos, i);
			++pos;
		}
	}
	tersus.swap(a, right-1, pos);

	return pos;
};
tersus._quicksort = function(a, left, right, comp)
{
	if(right-1>left) {
		var piv=left+Math.floor(Math.random()*(right-left));

		piv=tersus.partition(a, left, right, piv, comp); // position is updated in 'partition'

		tersus._quicksort(a, left, piv, comp);
		tersus._quicksort(a, piv+1, right, comp);
	}
};
tersus.quicksort = function(array, comparator)
{
	tersus._quicksort(array, 0, array.length, comparator);
};
// file: Split.js
/*********************************************************************************************************
 * Copyright (c) 2003-2010 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
tersus.newAction('Split', function()
{
	var sep = this.getLeafChild('<Separators>');
	var text = this.getLeafChild('<Text>');
	var allowEmpty = this.getLeafChild('<Return Empty Strings>');


	var start = 0;
	var pos = 0;
	while (pos < text.length)
	{
		var c = text.charAt(pos);
		if (sep.indexOf(c) >= 0) // Found a separator
		{
			if (allowEmpty || pos > start)
				this.chargeLeafExit('<Segments>',text.substring(start, pos));
			++pos;
			start = pos;
		}
		else
			++pos;
	}
	if (allowEmpty || pos > start)
		this.chargeLeafExit('<Segments>',text.substring(start, pos));
});
// file: SplitDate.js
// See SplitDateAndTime// file: SplitDateAndTime.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('SplitDateAndTime', function()
{
	var d = this.getLeafChild('<Date and Time>');
	if (d==null)
		d = this.getLeafChild('<Date>');
	if (d==null)
		return;	
	if (d.isTersusDate)
	{
		var date=d;
		this.chargeLeafExit('<Year>',date.year);	
		this.chargeLeafExit('<Month>',date.month);
		this.chargeLeafExit('<Day>',date.day);
		if (this.elements['<Day of Week>'])
		{
			var d = new Date(date.year, date.month-1, date.day);
			this.chargeLeafExit('<Day of Week>',d.getDay()+1);
		} 
		this.chargeLeafExit('<Hour>',0);
		this.chargeLeafExit('<Minute>',0);
		this.chargeLeafExit('<Second>',0);
		this.chargeLeafExit('<Millisecond>',0);
	}
	else
	{
		this.chargeLeafExit('<Year>',d.getFullYear());	
		this.chargeLeafExit('<Month>',d.getMonth()+1);
		this.chargeLeafExit('<Day>',d.getDate());
		this.chargeLeafExit('<Day of Week>',d.getDay()+1);
		this.chargeLeafExit('<Hour>',d.getHours());
		this.chargeLeafExit('<Minute>',d.getMinutes());
		this.chargeLeafExit('<Second>',d.getSeconds());
		this.chargeLeafExit('<Millisecond>',d.getMilliseconds());
	}
});
tersus.SplitDate = tersus.SplitDateAndTime;

// file: SquareRoot.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.SquareRoot = function tersus_SquareRoot() {};
tersus.SquareRoot.prototype = new ActionNode();
tersus.SquareRoot.prototype.constructor = tersus.SquareRoot;
tersus.SquareRoot.prototype.start = function ()
{
	var x = this.getLeafChild('<X>');
	if (x != 0)
		this.chargeLeafExit('<Square Root>',Math.sqrt(x))
};// file: StartsWith.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('StartsWith', function()
{
	var text = this.getLeafChild('<Text>');
	var prefix = this.getLeafChild('<Prefix>');
	if (text != null && prefix != null)
	{
		if (text.length >= prefix.length && text.substring(0,prefix.length) == prefix)
			this.chargeLeafExit('<Yes>',text);
		else
			this.chargeLeafExit('<No>',text);
	}
});

// file: Store.js
/*********************************************************************************************************
 * Copyright (c) 2003-2011 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/

tersus.newAction('Store',function()
{	
	var key = this.getLeafChild('<Key>');
	var value = this.getLeafChild('<Value>');
	var prefix = '';
	if (key)
	{
		var type = this.getLeafChild('<Storage Type>') || 'LOCAL';
		var s;
		if (type == 'LOCAL')
		{
			s = window.localStorage;
			if (s == null)
				throw 'No local storage available';
			key = tersus.appScopePrefix+key;
		}
		else if (type == 'SESSION')
		{
			s = window.sessionStorage;
			if (s == null)
				throw 'No session storage available';
		}
		if (value == null)
			s.removeItem(key)
		else
			s.setItem(key,value);
	}
});
// file: Subscribe.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Subscribe = ActionNode.__subclass('Subscribe');
tersus.Subscribe.loadURL = tersus.loadURL; // overridden when notification server is on another host
tersus.Subscribe.prototype.ON_MESSAGE =  '<On Message>';
tersus.Subscribe.prototype.MESSAGE =  '<Message>';
tersus.Subscribe.prototype.CHANNEL_T =  '<Channel>';
tersus.Subscribe.prototype.CHANNEL_D =  'Channel';
tersus.Subscribe.prototype.SOURCE =  'Source';
tersus.Subscribe.prototype.TIMESTAMP =  'Timestamp';
tersus.Subscribe.prototype.TYPE =  'Type';
tersus.Subscribe.prototype.CONTENT =  'Content';
tersus.Subscribe.pendingSubscriptions = null;
tersus.Subscribe.subscriptions = {};
tersus.Subscribe.clientId = null;

tersus.Subscribe.updateSubscriptions = function()
{
	if (!tersus.Subscribe.pendingSubscriptions)
		return;
	var a = [];
	var p = tersus.Subscribe.pendingSubscriptions;
	for (var channel in p)
	{
		if (p[channel]='ADD')
			a.push('+'+channel);
		else
			a.push('-'+channel);
	}
	tersus.Subscribe.pendingSubscriptions = null;
	if (a.length > 0)
	{
		var channels= encodeURIComponent(a.join(','));
		var url = tersus.notificationServer+'?action=subscribe&clientId='+tersus.Subscribe.clientId+'&channels='+channels+'&timestamp='+(new Date()).getTime();
		if (tersus.notificationServerAppId)
			url+='&appId='+tersus.notificationServerAppId;
		
		tersus.Subscribe.loadURL(url, false); //TODO better error handling?
	}
};
tersus.Subscribe.reconnect = function(timeout)
{
		tersus.Subscribe.clientId = null;
		tersus.Subscribe.pendingSubscriptions = {};
		for (var channel in tersus.Subscribe.subscriptions)
			tersus.Subscribe.pendingSubscriptions[channel]='ADD';
		setTimeout(tersus.Subscribe.connect,timeout);
};

tersus.Subscribe.handleResponse = function(responseText)
{
	var p = null;var l = null;
	if(responseText)
	{
		// we expect the second line to contain the response: first line is for "keep-alive" dots
		var ll = responseText.split(/[\n]/);
		if (ll.length >=2)
			l = ll[1];
		else if (ll.length>=1)
			l = ll[0];
		if (l!= null)
			p = l.split(/[\t]/);
	}
	if (!p || p.length == 0 || ! p[0])
	{
		tersus.Subscribe.reconnect(10000);
	}
	else if (p[0] == 'CONNECTED')
	{
		tersus.Subscribe.clientId = p[1];
		tersus.Subscribe.poll();
		tersus.Subscribe.updateSubscriptions();
	}
	else if (p[0] == 'RECONNECT')
	{
		tersus.Subscribe.reconnect(500);
	}
	else if (p[0] == 'MESSAGE')
	{
		//Not using p because split doesn't work consistently across browsers
		pp = l.match(/MESSAGE\t([^\t]*)\t([^\t]*)\t([^\t]*)\t([^\t]*)\t([^\t]*)/);
		var timestamp = new Date(); // TODO parse pp[1] for timestamp
		var channel = tersus.Subscribe.unescapeJava(pp[2]);
		var source = tersus.Subscribe.unescapeJava(pp[3]);
		var type = tersus.Subscribe.unescapeJava(pp[4]);
		var message = tersus.Subscribe.unescapeJava(pp[5]);
		var nodeIds = tersus.Subscribe.subscriptions[channel];
		if (nodeIds)
		{
			for (var i=0; i<nodeIds.length; i++)
			{
				var node = tersus.findNode(nodeIds[i]);
				if (node)
					node.onMessage(timestamp, channel, source, type, message);
			}
		}
		tersus.Subscribe.poll();
	}
	else if (p[0] == 'OK')
	{
		// alert('Subscribed');
	}
	else
	{
		tersus.Subscribe.reconnect(10000);
	}
};
tersus.Subscribe.repoll = function(status, statusText, e)
{
	setTimeout(tersus.Subscribe.poll,1000);
};
tersus.Subscribe.handleError = function(status, statusText, e)
{
	setTimeout(tersus.Subscribe.poll,10000);
};
tersus.Subscribe.poll = function()
{
	var url = tersus.notificationServer+'?action=connect&clientId='+tersus.Subscribe.clientId+'&timestamp='+(new Date()).getTime();
	tersus.Subscribe.xhr = tersus.Subscribe.loadURL(url, true);	
}
tersus.Subscribe.disconnected = true; // indicates that client is neither connected nor connecting to notification server
tersus.Subscribe.unescapeJava = function(str)
{
	if (str.search(/[^\\]["]/) >= 0)
		throw new tersus.Exception('Invalid string literal ['+str+']');
	var s = '"'+str+'"';
	return eval (s);
	
};
tersus.Subscribe.prototype.start = function()
{
	if (!tersus.notificationServer)
		tersus.notificationServer = tersus.rootURL+'Notifier';
	var channel = this.channel = this.getLeafChild(this.CHANNEL_T);
	var list = tersus.Subscribe.subscriptions[channel];
	if (!list)
	{
		list = [];
		tersus.Subscribe.subscriptions[channel] = list;
		if (!tersus.Subscribe.pendingSubscriptions)
		{
			tersus.Subscribe.pendingSubscriptions = {};
		}				
		tersus.Subscribe.pendingSubscriptions[channel]='ADD';
	}
	list.push(this.getNodeId());
	if (tersus.Subscribe.disconnected)
	{
		tersus.Subscribe.connect();
	}
	else
	{
		setTimeout(tersus.Subscribe.updateSubscriptions,1000);
	}
	this.chargeLeafExit('<Subscription Id>',this.getNodeId());
};
tersus.Subscribe.connect = function()
{
	if (!tersus.Subscribe.connectRetries)
		tersus.Subscribe.connectRetries = 0;
	if (tersus.Subscribe.loadURL == null)
	{
		if (tersus.Subscribe.connectRetries < 20)
		{
			setTimeout(tersus.Subscribe.connect,2000);
			return;
		}
		else
		{
			alert('Failed to connect to notification server: COMMFrame not loaded');
			return;
		}
	}
	tersus.Subscribe.disconnected = false;
	var url = tersus.notificationServer+'?action=connect&timestamp='+(new Date()).getTime();
	tersus.Subscribe.loadURL(url, false);
};

tersus.Subscribe.prototype.onMessage = function(timestamp,channel,source, type,content)
{
	var eventElement = this.getElement(this.ON_MESSAGE);
	if (!eventElement)
	{
		modelError('Missing sub-process "'+this.ON_MESSAGE+'"',this);
		return;
	}
	var messageTrigger = eventElement.getChildConstructor().prototype.getTrigger(this.MESSAGE);
	var message = messageTrigger.createChildInstance();
	timestamp = new Date(timestamp.getTime());
	message.setLeafChild(this.TIMESTAMP,timestamp,Operation.REPLACE);
	message.setLeafChild(this.CHANNEL_D,channel,Operation.REPLACE);
	message.setLeafChild(this.SOURCE,source,Operation.REPLACE);
	message.setLeafChild(this.TYPE,type,Operation.REPLACE);
	message.setLeafChild(this.CONTENT,content,Operation.REPLACE);
	var p = {};
	p[this.MESSAGE]=message;
	this.queueEvent(new tersus.Event(this.ON_MESSAGE), p);
};
tersus.NOTIFICATIONS_FRAME = 'notifications';
tersus.Subscribe.loadURL = function(url, useErrorHandler)
{
	return tersus.loadURL(url, tersus.Subscribe.handleResponse, false, useErrorHandler? tersus.Subscribe.handleError : tersus.Subscribe.commError);
}
tersus.Subscribe.loadURLSafari =  function(url, useErrorHandler)
{
	document.domain=tersus.Subscribe.parentDomain;
	var f = document.getElementById(tersus.NOTIFICATIONS_FRAME);
	var fw = getIFrameWindow(f);
	fw.loadURLSafari(url,useErrorHandler);
	document.domain=tersus.Subscribe.mainHost;
};
tersus.Subscribe.prototype.onload = function()
{
	if (tersus.notificationServer)
	{
   		tersus.Subscribe.mainHost = document.domain;
   		var nsHost = tersus.notificationServer.match(/\/\/([^:/]+)[:/]/)[1];
 		tersus.Subscribe.parentDomain = commonSuffix(nsHost,tersus.Subscribe.mainHost).match(/[^.]*.(.*)/)[1];
   		tersus.Subscribe.loadURL = null;
        var iframe = window.createAnIFrame(window, tersus.NOTIFICATIONS_FRAME,'notification.html');
	}
};
tersus.Subscribe.commError = function()
{
   			parent.alert('An error occurred when communicating with notification server');
};

function commonSuffix(s1,s2)
{
	var l1=s1.length;
	var l2=s2.length;
	for (var l = Math.min(l1,l2); l>=0; l--)
	{
		var suf1 = s1.substr(l1-l,l);
		var suf2 = s2.substr(l2-l,l);
		if (suf1 == suf2)
			return suf1;
	}
	return null;
}
// file: Substract.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Substract = function tersus_Substract() {};
tersus.Substract.prototype = new ActionNode();
tersus.Substract.prototype.constructor = tersus.Substract;
tersus.Substract.prototype.start = function ()
{
	var x = parseFloat(this.getLeafChild('<X>'));
	var y = parseFloat(this.getLeafChild('<Y>'));
	this.chargeLeafExit('<Difference>',x-y);
};
// file: Sum.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Sum = function tersus_Add() {};
tersus.Sum.prototype = new ActionNode();
tersus.Sum.prototype.constructor = tersus.Sum;
tersus.Sum.prototype.start = function ()
{
	var values = this.getTriggerValues();
	var sum = 0;
	for (var j=0; j<values.length; j++)
	{
		var value = values[j];
		if (value.leafValue != null)
			sum += parseFloat(value.leafValue);
	}
	this.chargeLeafExit('<Sum>',sum);
};
// file: SwitchView.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function SwitchView() {};
SwitchView.prototype = new FlowNode();
SwitchView.prototype.constructor = SwitchView();

SwitchView.prototype.start = function()
{
	var view = this.getLeafChild('<View>');
	var perspective = this.getLeafChild('<Perspective>');
	var parameters = ''
	for (var i=0; i<this.triggers.length; i++)
	{
		var trigger = this.triggers[i];
		if (trigger.role.charAt(0) != '<') // ignore 'special' triggers
		{
			var value = this.getLeafChild(trigger.role);
			if (value)
			{
				if (parameters)
					parameters += '&';
				parameters+=encodeURIComponent(trigger.role)+'='+encodeURIComponent(value);
			}
		}
	}
	
	tersus.switchView(perspective,view,parameters);
};
// file: TabbedPane.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function TabbedPane() {};
TabbedPane.prototype = new Pane();
TabbedPane.prototype.constructor = TabbedPane;
TabbedPane.prototype.reset = function()
{
	this.selectedTab = this.selectedTabRole = null;
	DisplayNode.prototype.reset.call(this);
};
TabbedPane.prototype.createSpacerDiv = function TabbedPane_createSpacerDiv()
{
	var div = this.createHTMLElement('div');
	div.style.clear='both';
	var img = this.createHTMLElement('img');
	img.src="images/spacer.gif";
	if (document.compatMode != 'BackCompat')
		img.className='spacer_image';
	div.className='spacer_div';
	div.appendChild(img);
	return div;
};
TabbedPane.prototype.createViewNode = function TabbedPane_crateViewNode()
{
	var direction = getActualStyle(this.currentWindow.document.body, 'direction');
	var rtlSuffix = (direction == 'rtl') ? '_rtl':''; // Support for right-to-left languages
	DisplayNode.prototype.createViewNode.call(this);
	var header = this.createHTMLChildElement(this.viewNode,'div', 'tabbedPaneHeader'+rtlSuffix);
	header.appendChild(this.createSpacerDiv());
	this.tabList = this.createHTMLChildElement(header,'ul');
	this.defaultElement = this.displayElements[0];
	this.tabs = {};
	this.links = {};
	var footer = this.createSpacerDiv();
	footer.className="footer";
	header.appendChild(footer);
	var contentTR = this.createHTMLChildElement(this.viewNode, 'div', 'tabbedPaneContentTR'+rtlSuffix);
	var contentBR = this.createHTMLChildElement(contentTR, 'div', 'tabbedPaneContentBR'+rtlSuffix);
	var contentTL = this.createHTMLChildElement(contentBR, 'div', 'tabbedPaneContentTL'+rtlSuffix);
	var contentBL = this.createHTMLChildElement(contentTL, 'div', 'tabbedPaneContentBL'+rtlSuffix);
	contentBL.appendChild(this.createSpacerDiv());
	this.contentNode = this.createHTMLChildElement(contentBL,'div','tabbedPaneContent'+rtlSuffix);
	var width = this.sharedProperties.width
	if ( width != undefined && width != ' ')
	{
		if (window.browser == 'IE')
			this.contentNode.style.width='100%';
		if (width != '100%' || window.browser != 'Mozilla')
			this.viewNode.style.width = this.sharedProperties.width;
	}
	var cHeight = this.sharedProperties.contentHeight;
	if ( cHeight!= undefined && cHeight != ' ')
	{
		this.cHeight = cHeight;
		this.contentNode.style.height=cHeight;
	}
	contentBL.appendChild(this.createSpacerDiv());
};

TabbedPane.prototype.addTab = function (tabCaption, childNode, previousTab)
{
	var childId = childNode.getNodeId();
	var item = this.createHTMLElement('li');
	var nextTab = null;
	if (previousTab)
		nextTab = previousTab.nextSibling;
	else
		nextTab = this.tabList.firstChild;
	item.childId=childId;
	this.tabList.insertBefore(item, nextTab);
	var div = this.createHTMLChildElement(item,'div');
	var link = this.createHTMLChildElement(div,'a');
	this.tabs[childId]=item;
	this.links[childId]=link;
	link.href="javascript:tersus.findNode("+this.getNodeId()+").select("+childId+")";
	link.innerHTML=tersus.escapeHTML(tabCaption);
	if ((! this.selectedTab) && (childNode.element == this.defaultElement) )
	{
		item.className='selected';
		this.selectedTab = item;
		this.selectedChild = childNode;
	}
	var pane = this;
	if (this.sharedProperties.adjustBottom == true)
		tersus.async.exec(function() {pane.onResize();}, false /* don't wait */);
	
};

TabbedPane.prototype.fireOnSelect = function (childNode)
{
	window.engine.handleEvent(childNode, Events.ON_SELECT);
};
TabbedPane.prototype.select = function (nodeId)
{
	var childNode = tersus.findNode(nodeId);
	if (!childNode)
		return;
	this.links[nodeId].blur();
	this.setSelectedTab(childNode);
	if (childNode.hasEventHandler(Events.ON_SELECT))
	{
		this.fireOnSelect(childNode);
	}
};

TabbedPane.prototype['get<Selected Pane>'] = function()
{
	return this.selectedChild;
};
TabbedPane.prototype.setSelectedTab  = TabbedPane.prototype['set<Selected Pane>'] = function (childNode)
{
	var nodeId = childNode.getNodeId();
	var tab = this.tabs[nodeId];
	if (tab == null || tab == undefined)
		modelError('Error in TabbedPane.setSelectedTab: no tab called '+tab+' in '+ this.modelId);
	if (this.selectedTab == tab)
	{
		return;
	}
	this.selecting = true;
	if (this.selectedTab)
	{
		this.selectedTab.className = 'normal';
		this.selectedChild.viewNode.style.display='none';
	}
	tab.className='selected';
	if (childNode)
	{
		childNode.viewNode.style.display = '';
		//reattach the child node to reset the parent height
		var pane = childNode.viewNode;
		var parent = pane.parentNode;
	}
	
	this.selectedChild = childNode;
	this.selectedTab = tab;
	
	if (this.selectedChild)
		this.selectedChild.onResize();

};

TabbedPane.prototype.setSelectedTab.requiresNodes = true;

TabbedPane.prototype.removeChildViewNode = function (child)
{
	DisplayNode.prototype.removeChildViewNode.call(this,child);
	var tabItem = this.tabs[child.getNodeId()];
	if (tabItem && this.tabList)
		this.tabList.removeChild(tabItem);
	if (this.selectedTab == tabItem)
	{
		this.selectedTab == null;
		this.selectedChild = null;
	}
};
TabbedPane.prototype.appendChildViewNode = function (child)
{
	this.contentNode.appendChild(child.viewNode);
	var previous = this.findPreviousSibling(child);
	var previousTab = null;
	if (previous)
		previousTab = this.tabs[previous.getNodeId()];
	this.addTab(child.getCaption(), child, previousTab);
	if (child == this.selectedChild)
	{
		child.viewNode.style.display='';
		this.selectedChild = child;
	}
	else
		child.viewNode.style.display='none';

	var tabbedPane = this;
	child.captionChanged = function()
	{
		tabbedPane.links[child.getNodeId()].innerHTML = tersus.escapeHTML(this.getCaption());
	};
		
};
TabbedPane.prototype.onResize = function()
{
	if (this.sharedProperties.adjustBottom == true)
	{
		this.adjustBottom();
	};
	if (this.selectedChild)
		this.selectedChild.onResize();
};

TabbedPane.prototype.adjustHeight = function (delta)
{
	tersus.adjustHeight(this.contentNode, delta);
};
// file: TakePicture.js
tersus.SAMPLE_IMAGE='data:image/png;base64,\niVBORw0KGgoAAAANSUhEUgAAAGQAAABkCAIAAAD/gAIDAAAAAXNSR0IArs4c6QAAANNJREFUeNrt\n3NEJgDAMQEErruGgOoAO6iB1hUoJsfTevx8eERpBS611UVsrAliwYMGCBQsBLFiwYMGChQAWLFiw\nYMFC0N7Wc/G5HyPe8/XcJstjCAsWLMGCBQvWjCf4oINyZ3F7hcmCBQsWLFiCBQsWLOtO8FKStUiZ\nLFiwYMGCJViwYMGCBUuwYMGCBQuWYH0o7R181nt0kwULFixYggUL1lSH0kG/czVZsGDBggVLsGDB\n+nPF/+BNFixYsGDBQgALFixYsGAhgAULFixYsBDACukFZWESy0+VwUsAAAAASUVORK5CYII=\n';
tersus.newAction('TakePicture', function()
{
	var dataURLExit = this.getExit('<Data URL>');
	var fileURIExit = this.getExit('<File URI>');
	var imgQuality=70;
	var q=this.getLeafChild('<Quality>');
	if (q)
	{
		q = Math.round(q);
		
		if (q>0 && q<=100)
			q = null;
	}
	
	var w = this.getLeafChild('<Width>');
	var h = this.getLeafChild('<Height>');
	
	var dest = dataURLExit ?  Camera.DestinationType.DATA_URL : Camera.DestinationType.FILE_URI;
	
	/*if (tersus.simulateCamera != false)
	{
		if(window.confirm('No camera available.  Would you like to use a sample image?'))
			this.chargeLeafExit('<Data URL>',tersus.SAMPLE_IMAGE);
	}
	else */if (navigator.camera && navigator.camera.getPicture)
	{
		var node = this;
		this.pause();
		var success = function(data)
		{
			console.log('Got Image '+data);			
			if (dataURLExit)
				node.chargeLeafExit(dataURLExit,"data:image/jpeg;base64," + data.replace(/\n/g,''));
			else
			{
				node.chargeLeafExit(fileURIExit, data);
			}
			node.continueExecution();
			console.log('Got Image (done)');			
			
		}
		var error = function(message)
		{
			if (message == 'no image selected' || message == 'Camera cancelled.')
			{
				node.chargeEmptyExit('<None>');
			}
			else
			{
				console.log('Get Picture: Error: '+message);			
				modelExecutionError("Failed to take picture: "+message,node);
			}
			node.continueExecution();
		};
		
		var p = {quality:0.7, destinationType:dest};
		
		if (w)
			p.targetWidth=w+0;
		if (h)
			p.targetHeight=h+0;
			
		if (q)
			p.quality=q;
			
			

		console.log('Calling getPicture w='+p.targetWidth+' h='+p.targetHeight);			
		navigator.camera.getPicture(success, error, p); 
		
	}
	else
		modelExecutionError('No camera available.');		
});
// file: TestCase.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.TestCase = ActionNode;// file: TestSuite.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.TestSuite = ActionNode;
// file: TextArea.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.TextArea = function tersus_TextArea() {};
tersus.TextArea.prototype = new tersus.GenericField();
tersus.TextArea.prototype.constructor = tersus.TextArea;
tersus.TextArea.prototype.inputFieldTag = 'textarea';
TextArea=tersus.TextArea; // compatibility with very old models

// file: TimeDifference.js
/*********************************************************************************************************
 * Copyright (c) 2003-2010 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
tersus.newAction('TimeDifference', function()
{
	var from = this.getChild('<From>');
	var fromDate = tersus.getDate(from);
	var to = this.getChild('<To>');
	var toDate = tersus.getDate(to);
	
	var differenceMilliseconds = toDate - fromDate;
	var differenceSeconds = differenceMilliseconds/1000.0;
    var differneceMinutes = differenceSeconds/60.0;
    var differenceHours = differneceMinutes/60.0;
    var differenceDays = differenceHours/24.0;
    
    this.chargeLeafExit('<Days>', differenceDays);	
	this.chargeLeafExit('<Hours>', differenceHours);
	this.chargeLeafExit('<Minutes>', differneceMinutes);
	this.chargeLeafExit('<Seconds>', differenceSeconds);
	this.chargeLeafExit('<Milliseconds>', differenceMilliseconds);
});// file: Timer.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Timer = function tersus_Timer() {};
tersus.Timer.prototype = new ActionNode();
tersus.Timer.prototype.constructor = tersus.Timer;
tersus.Timer.prototype.start = function()
{
	if (!this.subFlowList || this.subFlowList.length == 0)
	{
		modelError("Missing activity in Timer "+this.modelId);
		return;
	}
	if (this.subFlowList.length > 1)
	{
		modelError("Ambigous activity in Timer "+this.modelId + " (more than one sub-process)");
		return;
	}
	var activityElement = this.subFlowList[0];
	var interval = this.getLeafChild('<Interval>');
	if (!interval)
		modelError("Missing <Interval> for Timer "+this.modelId);
	var timer = this;
	var invoker = function()
	{
		if (timer.busy)
			return;
		timer.busy = true;
		var process = new BackgroundProcess(timer, activityElement);
		if (timer.links)
		{
			for (var i = 0; i < timer.links.length; i++)
			{
				var link = timer.links[i];
				if (link.source.segments.length != 1 || link.target.segments.length != 2)
				{
					modelError("Invalid link "+link.role + " in Timer " + this.modelId);
					return;
				}	
				var outerTriggerName = link.source.segments[0];
				var innerTriggerName = link.target.segments[1];
				var value = timer.getChild(outerTriggerName);
				if (value)
					process.setTriggerValue(innerTriggerName, value);
			}
		}
		process.whenDone(function()
		{
			timer.busy = false;
		});
		process.start();
		
	};
	var timerId = window.setInterval(invoker, interval);
	if (tlog) tlog("Set timer "+timerId + " [" + this.modelId+"]");
	this.chargeLeafExit('<Timer Id>', timerId);
};
// file: Today.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function Today() {};
Today.prototype = new ActionNode();
Today.prototype.constructor = Today();

Today.prototype.start = function()
{
	var now = new Date();
	var d  = now.getDate();
	var m = now.getMonth()+1;
	var y  = now.getFullYear();
	var today = createDate(y,m,d);
	this.chargeLeafExit('<Today>',today);
};
// file: Tree.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function Tree() { this.selectedItem = null};
Tree.prototype = new DisplayNode();
Tree.prototype.constructor = Tree;
Tree.prototype.defaultTag = 'div';
Tree.prototype.start = function startTree()
{
	this.root = new TreeItem();
	this.root.attach(this,null);
	this.setStatus(FlowStatus.WAITING_FOR_INPUT);
};
Tree.prototype.refresh = function()
{
	if (this.root.childItems)
	{
		for (var i=0; i< this.root.childItems.length; i++)
			this.root.childItems[i].onDelete();
	}
	this.childItems = null;
	this.root.loadChildren();
};
Tree.prototype.findById = function (id)
{
	if (this.nodeId == id)
		return this;
	else if (this.root)
		return this.root.findById(id);
	else
		return null;
};
Tree.prototype['get<Selected Item>'] = function()
{
 	if (this.selectedItem)
 		return this.selectedItem;
 	else
 		return null;
};
Tree.prototype.scanChildren = function(filter, alreadyScanned, outputList)
{
	if (filter.match(this))
		outputList.push(this);
	if (this.root)
		this.root.scanChildren(filter, alreadyScanned,outputList);		
};// file: TreeItem.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function TreeItem() {};
TreeItem.prototype = new DataValue();
TreeItem.prototype.constructor = TreeItem;
TreeItem.prototype.getText = function getText() 
{
	var textNode = this.getChild('<Text>');
	if (textNode)
		return textNode.leafValue;
	else
		return '';
};
TreeItem.prototype.getNodeId = DisplayNode.prototype.getNodeId;
TreeItem.prototype.getImage = function getImage()
{
	var imageNode = this.getChild('<Image Path>');
	if (imageNode)
		return imageNode.leafValue;
	else
		return null;
};

TreeItem.prototype.isExpandable = function()
{
	if (this.childItems)
		return this.childItems.length > 0;
	var element = this.getElement('<Expandable>');
	if (! element)
		return true; // backward compatability
	var expandable = element.getChild(this); 
	return (expandable && expandable.leafValue);
}


TreeItem.COLLAPSED = 'Collapsed';
TreeItem.EXPANDED = 'Expanded';
TreeItem.ATOMIC = 'Atomic';
TreeItem.select = function()
{
	var item = DisplayNode.findNodeForDocumentNode(this);
	if (item)
		item.select();
};
TreeItem.toggle = function()
{
	var item = DisplayNode.findNodeForDocumentNode(this);
	if (item)
		item.toggle();
};
TreeItem.openItem = function()
{
	var treeItem = DisplayNode.findNodeForDocumentNode(this);
	if (treeItem)
		treeItem.openItem();
};
TreeItem.prototype.attach = function attach(tree,parentItem)
{
	this.tree = tree;
	var doc = tree.currentWindow.document;
	this.isRoot = ! parentItem;	
	if (parentItem)
	{
		this.parentItem = parentItem;
		this.viewNode = this.tree.createHTMLElement('div','tree_item');
		this.headerNode = this.tree.createHTMLChildElement(this.viewNode,'span','tree_item_header');
		var thisItem = this;
		this.image = this.tree.createHTMLChildElement(this.headerNode, 'img', 'tree_item_collapsed');
		this.image.src = 'images/spacer.gif';
		this.icon = this.tree.createHTMLChildElement(this.headerNode, 'img', 'tree_item_icon');
		this.icon.style.backgroundImage = 'url('+encodeURI(this.getImage())+')';
		this.icon.src = 'images/spacer.gif';
		this.headerText = this.tree.createHTMLChildElement(this.headerNode, 'span', 'tree_item_header_text');
		this.headerText.innerHTML = tersus.escapeHTML(this.getText());
		var id = this.getNodeId();
		this.icon.id = id;
		this.headerText.id = id;
		this.icon.onclick = TreeItem.select;
		this.headerText.onclick= TreeItem.select;
		this.icon.ondblclick = TreeItem.openItem;
		this.headerText.ondblclick = TreeItem.openItem;
		this.resetExpandable();
		parentItem.getOrCreateContentNode().appendChild(this.viewNode);
	}
	else
	{
		this.viewNode = this.contentNode = doc.createElement('div');
		this.expand();
		
		tree.appendChildViewNode(this);
	}
};
TreeItem.prototype.getOrCreateContentNode = function()
{
	if (!this.contentNode)
		this.contentNode = this.tree.createHTMLChildElement(this.viewNode,'div', 'tree_item_content');
	return this.contentNode;
		
};
TreeItem.prototype.resetExpandable = function()
{
	this.image.onclick = null;
	if (this.isExpandable())
	{
		this.image.id = this.getNodeId();
		this.collapse();
		this.image.onclick = TreeItem.toggle;
	}
	else
		this.atomic();
};
TreeItem.prototype.findById = function (id)
{
	if (this.nodeId == id)
		return this;
	if (this.childItems)
	{
		for (var i=0; i<this.childItems.length; i++)
		{
			var child = this.childItems[i];
			if (child)
			{
				var node = child.findById(id);
				if (node)
					return node;
			}
		}
	}
	return null;
};
TreeItem.prototype.loadChildren = function ()
{
	var item = this;
	var loaderElement = this.tree.getElement('<Get Children>');
	var process = new BackgroundProcess(this.tree, loaderElement);
	if (! this.isRoot) // The root item is not a 'real' item (as defined in the model)
		process.setTriggerValue('<Parent>', this);
	process.whenDone(function()
	{
		item.childItems = process.getExitValues('<Children>');
		if (item.childItems.length >0)
		{
			for (var i=0; i<item.childItems.length; i++)
			{
				var child = item.childItems[i];
				child.attach(item.tree,item);
			}
			item.state = TreeItem.EXPANDED;
		
			if (item.headerNode)
			{
				item.getOrCreateContentNode().style.display = 'block';
			}
			if (item.image)
				item.image.className='tree_item_expanded';
		}
		else
		{
			item.atomic();
		}
		
		if (item.onExpand)
			item.onExpand();
	});
	process.start();

};
TreeItem.prototype.expand  = function expand()
{
		if (! this.childItems)
		{
			this.loadChildren();
		}
		else
		{
			this.state = TreeItem.EXPANDED;
			if (this.headerNode)
			{
				this.getOrCreateContentNode().style.display = 'block';
			}
			if (this.image)
				this.image.className='tree_item_expanded';
			if (this.onExpand)
				this.onExpand();
		}
};
TreeItem.prototype.atomic = function()
{
		this.state = TreeItem.ATOMIC;
		if (this.image)
			this.image.className='tree_item_atomic';

};

TreeItem.prototype.collapse = function collapse()
{
		this.state = TreeItem.COLLAPSED;
		if (this.image)
			this.image.className='tree_item_collapsed';
		if (this.contentNode)
			this.contentNode.style.display='none';
		if (this.childItems)
		{
			for (var i=0; i< this.childItems.length; i++)
				this.childItems[i].onDelete();
		}
		this.childItems = null;

};
TreeItem.prototype.toggle = function toggle()
{
	if (this.state == TreeItem.COLLAPSED)
		this.expand();
	else
		this.collapse();
};

TreeItem.prototype.select = function select()
{
	if (!this.selected)
	{
		if (this.tree.selectedItem)
			this.tree.selectedItem.deselect();
		this.tree.selectedItem = this;
		this.headerText.className = 'tree_item_header_text_selected';
		this.selected = true;
	};
};
TreeItem.prototype.deselect = function deselect()
{
	this.headerText.className = 'tree_item_header_text';
	this.selected = false;
};
TreeItem.prototype.openItem = function openItem()
{
	var openSubFlow = this.tree.getEventHandler(Events.OPEN);
	if (openSubFlow)
	{
		var child = openSubFlow.createChild(this.tree);
		child.setChild('<Selected Item>', this, Operation.REPLACE);
		child.readyToStart();
		window.rootDisplayNode.doResume();
	};
		
};
TreeItem.prototype.scanChildren = function(filter, alreadyScanned,outputList)
{
	var id = this.getNodeId();
	if (alreadyScanned[id])
		return;
	alreadyScanned[id]=true;
	if (filter.match(this))
		outputList.push(this);
	if (this.childItems)
	{
		for (var i=0; i< this.childItems.length; i++)
			this.childItems[i].scanChildren(filter, alreadyScanned,outputList);		
	}
};

TreeItem.prototype.refresh = function()
{
	this.collapse();
	this.loadChildren();
	this.resetExpandable();
	
};
/* Returns true if this item and its ancestors are all expanded */
TreeItem.prototype.isExpandedAndVisible = function()
{
	if (this.state != TreeItem.EXPANDED)
		return false;
	else if (! this.parentItem)
		return true;
	else
		return this.parentItem.isExpandedAndVisible();
}
TreeItem.prototype.onDelete = function ()
{
	if (this.parentItem)
	{
		this.parentItem.contentNode.removeChild(this.viewNode);
	}
};
// file: Trim.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('Trim', function()
{
	var text = this.getLeafChild('<Text>');
    if (text != null)
    {
    	var m = text.replace(/^\s*([\S\s]*?)\s*$/, '$1');
    	if (m && m.length >= 1)
    		this.chargeLeafExit('<Trimmed Text>', m);
	}
});

// file: UniqueItems.js
tersus.newAction('UniqueItems', function()
{
	var items = this.getTriggerValues();
	var output = [];
	
	for (var i=0;i<items.length;i++)
	{
		var item = items[i];
		var alreadyListed = false;
		for (var j=0;j<output.length;j++)
		{
			if (compareValues(item, output[j]))
			{
				alreadyListed = true;
				break;
			}
		}
		if (!alreadyListed)
			output.push(item);
	}
	var e = this.getExit('<Unique Items>');
	for (var i=0;i<output.length;i++)
	{
		this.accumulateExitValue(e, output[i]);
	};
});

// file: Unsubscribe.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Unsubscribe = tersus.CallJavaApplet.__subclass('Unsubscribe');

tersus.Unsubscribe.prototype.start = function()
{
	var id = this.getLeafChild('<Subscription Id>');
	var node = tersus.findNode(id);
	if (node)
	{
		var channel = node.channel;
		if (channel)
		{
			var list = tersus.Subscribe.subscriptions[channel];
			if (list)
			{
				for (var i=0;i<list; i++)
				{
					if (list[i] == id)
					{
						list.splice(id,1);
						if (list.length == 0)
						{
							delete tersus.Subscribe.subscriptions[id];
							tersus.Subscribe.pendingSubscriptions[channel] = 'REMOVE';
							setTimeout(tersus.updateSubscriptions,1000); // we're delaying updating of subscriptions in order to use one server request for numerous subscriptions
						}
						break;
					}
				}
			}
		}
		
	}

};
// file: Update.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Update = tersus.DatabaseAction.__subclass('tersus.Update');
tersus.Update.prototype.getTablePrototype = function()
{
	return this.getTrigger('<Record>').getChildConstructor().prototype;
};
tersus.Update.prototype.run = function()
{
	var record = this.getChild('<Record>');
	var tableName = this.getTableName(record);
	var fieldList = this.getFieldList(record.constructor.prototype);
	var sqla = []; // 'buffer' for creating the sql
	var values = []; // values
	sqla.push('UPDATE '+tableName + " SET ");
	var fields = fieldList.fields;
	var nfields = fields.length;
	var n = 0;
	var first = false;
	for (var j=0;j<nfields;j++)
	{
		var field = fields[j];
		if (!field.isPrimaryKey)
		{
			if (n++>0)
				sqla.push(',');
			sqla.push(field.sqlName);
			var v = field.getSQLValue(record);
			if (v != null)
			{
				sqla.push('= ?');
				values.push(v);
			}
			else
				sqla.push (' = NULL');
		}
	}
	this.addPrimaryKeyWhereClause(record,fieldList,sqla, values);
	var sql = sqla.join(' ');
	this.execSql(sql, values, function(transaction,results)
	{
		this.handleResults(results,record,tableName);
		this.resumeContext();
	});
			
};
tersus.Update.prototype.handleResults = function(results, record, tableName)
{
	var count = results.rowsAffected;
	if (count == 1)
		this.chargeExit('<Updated>',record);
	else if (count == 0)
		this.chargeExit('<Not Found>',record);
	else
		modelExecutionError("Update resulted in updating multiple records in table "
                                    + tableName
                                    + ". Check primary key settings", this);
                                    
};// file: UpperCase.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.newAction('UpperCase', function()
{
	var text = this.getLeafChild('<Text>');
	if (text != null)
		this.chargeLeafExit('<Upper Case>',text.toUpperCase());
});

// file: VSplitPane.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.VSplitPane = function tersus_VSplitPane(){};
tersus.VSplitPane.prototype = new SplitPane();
tersus.VSplitPane.prototype.constructor = tersus.VSplitPane;
tersus.VSplitPane.prototype.bottomPaneHeight = 150;
tersus.VSplitPane.prototype.minHeight=20;
tersus.VSplitPane.prototype.contentPaneStyle = "vSplitPaneContent";
tersus.VSplitPane.prototype.dividerStyle = "vSplitPaneDivider";
tersus.VSplitPane.prototype.onResize = function ()
{
	if (this.sharedProperties.height)
		this.setDimensions();
	else
	{
		var d = this.viewNode.ownerDocument;
		var body = this.viewNode.ownerDocument.body;
		var height = getVisibleSize(body).height;
	
		var currentBottom = getOffsets(this.viewNode).top + this.viewNode.offsetHeight;
		if (window.browser != 'IE')
			currentBottom += 0;
		
		var delta = height-currentBottom;
		var currentHeight = this.bottomPaneHeight + this.topPaneHeight;
		if (delta+currentHeight >= 2*this.minHeight)
		{
			this.topPaneHeight = Math.max(this.minHeight, this.topPaneHeight+delta);
			this.bottomPaneHeight = currentHeight+delta-this.topPaneHeight;
			this.setPaneHeight(this.pane1,this.topPaneHeight);
			this.setPaneHeight(this.pane2,this.bottomPaneHeight);
		}
		var pane = this;
		var resizeChildren = function(){pane.onResizeChildren()};
		this.currentWindow.setTimeout(resizeChildren,10);
	}
};
tersus.VSplitPane.prototype.setDimensions = function()
{
	if (this.sharedProperties.height)
		this.height =parseInt(this.sharedProperties.height);
	
	this.divider.style.height = this.dividerThickness+ 'px';
	if (this.sharedProperties.bottomPaneHeight)
	{
		this.bottomPaneHeight=parseInt(this.sharedProperties.bottomPaneHeight);
	}
	this.topPaneHeight = this.height - this.bottomPaneHeight;
	this.setPaneHeight(this.pane1, this.topPaneHeight);
	this.setPaneHeight(this.pane2,this.bottomPaneHeight);
};
tersus.VSplitPane.prototype.resize = function(delta)
{
	if (this.topPaneHeight+delta.y >= this.minHeight && this.bottomPaneHeight-delta.y >= this.minHeight)
	{
		this.topPaneHeight += delta.y;
		this.bottomPaneHeight -= delta.y;
		this.setPaneHeight(this.pane1, this.topPaneHeight);
		this.setPaneHeight(this.pane2,this.bottomPaneHeight);
		this.onResizeChildren();
	}
};
// file: VariablePane.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function VariablePane(){};
VariablePane.prototype = new DisplayNode();
VariablePane.prototype.constructor = VariablePane;
VariablePane.prototype.defaultTag='div';
VariablePane.prototype.preloadOnStart=false;

VariablePane.prototype.createChildren = function()
{
};
VariablePane.prototype.createSubFlows = function()
{
	//initializes sub flows
	if (this.subFlowList)
	{
		for (var i=0;i<this.subFlowList.length; i++)
		{
			var subFlow = this.subFlowList[i];
			if (!subFlow.isDisplayElement)
				subFlow.init(this);
		}
	}
};

VariablePane.prototype.addDisplayElement = function addElement(role, modelId, properties)
{
	if (role.charAt(0) == '<' && this.getGetter(role))
	{
		var element = new MethodElement(this, role, modelId);
	}
	else
		var element = new VariableNodeDisplayElement(role, modelId);
	if (! this.displayElements)
		this.displayElements = [element];
	else
		this.displayElements.push(element);
	this.registerSubFlow(element);
	element.properties = properties;
	return this.addElement(element);
};
VariablePane.prototype['get<Current Content>'] = function()
{
	if (this.children)
	{
		for (var i=0; i<this.displayElements.length; i++)
		{
			var e = this.displayElements[i];
			var c = e.getChild(this);
			if (c != null)
				return c;
		}
	}
	return null;
}

function VariableNodeDisplayElement(role, modelId)
{
	this.role = role;
	this.modelId = modelId;
}

VariableNodeDisplayElement.prototype = new DisplayElement();
VariableNodeDisplayElement.prototype.constructor = VariableNodeDisplayElement;
VariableNodeDisplayElement.prototype.createChild = function createChild(parent, modelId, value)
{
	if (parent.currentWindow) // parent is a real display node
	{
		//TODO fix this so that 'onDelete()' is called for all children
		// (currently, some cleanup may be skipped)
		parent.children = [];
		parent.viewNode.innerHTML = '';
	}
	var child = DisplayElement.prototype.createChild.call(this,parent, modelId, value);
	return child;
};
VariableNodeDisplayElement.prototype.replaceChild = function replaceChild(parent, value)
{
	//TODO(x) complete the refactoring - replace deepCopy with copyValue. Compare with base implementation to see if it's needed at all
	if (parent.currentWindow) // parent is a real display node
	{
		var child = this.getChild(parent);
		if (child)
			child.deepCopy(value);
		else
			child = this.createChild(parent, this.modelId, value);		
	}
	else
	{
		Element.prototype.replaceChild.call(this, parent, value);
	}
};// file: Wait.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Wait = function tersus_Wait() {};
tersus.Wait.prototype = new ActionNode();
tersus.Wait.prototype.constructor = tersus.Wait;
tersus.Wait.prototype.done = function ()
{
	this.continueExecution();
};
tersus.Wait.prototype.resume = function()
{
};
tersus.Wait.DELAY=50;
tersus.Wait.prototype.start = function ()
{
	var time = this.getLeafChild('<Milliseconds>');
	var action = this;
	/* Not sure timing here is always correct.  We should probably always wait until the execution context is not busy
	   But there is no mechanism to do it and the required behavior is not fully defined */
	var done = function()
	{
		tersus.async.exec(function(){action.done();}, false /* don't wait */); // already waiting in checkResume  
	};
	if (time != null)
		setTimeout(done,time);  
	else
	{
		var checkResume = function()
		{
			if (tlog) tlog('browser location='+window.location.href+' tersus location='+tersus.history.location);
			tersus.history.checkLocation();
			if (isBusy() || tersus.async.isBusy())
				setTimeout(checkResume,tersus.Wait.DELAY);
			else
				tersus.async.exec(done, false /* don't wait */ ); // Looks like a redundant call to 'async' - not sure.  
		};
		tersus.async.exec(checkResume, false /* don't wait */);
	}
	if (window.trace)
		window.trace.flush();
	this.pause();
};
// file: WatchAccelerometer.js
tersus.newAction('WatchAccelerometer', function()
{
	tersus.WatchAccelerometer.watcherId = this.getNodeId();
	var threshold = this.getLeafChild('<Threshold>');
	if (threshold == null)
		threshold = 0.5;
	document.location='tersus://WatchAccelerometer/'+threshold;
});

tersus.WatchAccelerometer.busy = false;
var count = 0;
tersus.WatchAccelerometer.handleResponse = function(x, y, z, timestamp)
{
	++count;
	if (tersus.WatchAccelerometer.busy)
		return;
	tersus.WatchAccelerometer.busy = true;	
	var node = window.rootDisplayNode;
	var actionElement = node.getElement('<On Acceleration>');
	if (!actionElement)
		modelError('Missing element <On Acceleration>');
	var process = new BackgroundProcess(node, actionElement);
	process.setTriggerLeafValue('<X>',x);
	process.setTriggerLeafValue('<Y>',y);
	process.setTriggerLeafValue('<Z>',z);
	process.setTriggerLeafValue('<Timestamp>',timestamp);
			
	process.whenDone(function()
	{
		tersus.WatchAccelerometer.busy = false;
	});
	process.start();
};
// file: iPhoneVariablePane.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function iPhoneVariablePane(){};
iPhoneVariablePane.prototype = new DisplayNode();
iPhoneVariablePane.prototype.constructor = iPhoneVariablePane;
iPhoneVariablePane.prototype.defaultTag='div';
iPhoneVariablePane.prototype.preloadOnStart=false;

iPhoneVariablePane.prototype.createChildren = function()
{
};
iPhoneVariablePane.prototype.createSubFlows = function()
{
	//initializes sub flows
	if (this.subFlowList)
	{
		for (var i=0;i<this.subFlowList.length; i++)
		{
			var subFlow = this.subFlowList[i];
			if (!subFlow.isDisplayElement)
				subFlow.init(this);
		}
	}
};

iPhoneVariablePane.prototype.addDisplayElement = function addElement(role, modelId, properties)
{
	if (role.charAt(0) == '<' && this.getGetter(role))
	{
		var element = new MethodElement(this, role, modelId);
	}
	else
		var element = new VariableNodeDisplayElement(role, modelId);
	if (! this.displayElements)
		this.displayElements = [element];
	else
		this.displayElements.push(element);
	this.registerSubFlow(element);
	element.properties = properties;
	return this.addElement(element);
};
iPhoneVariablePane.prototype['get<Current Content>'] = function()
{
	if (this.children)
	{
		for (var i=0; i<this.displayElements.length; i++)
		{
			var e = this.displayElements[i];
			var c = e.getChild(this);
			if (c != null)
				return c;
		}
	}
	return null;
}

function VariableNodeDisplayElement(role, modelId)
{
	this.role = role;
	this.modelId = modelId;
}

iPhoneVariablePane.prototype.setProperties = function()
{
	DisplayNode.prototype.setProperties.call(this);
	var vari = this;
	tersus.async.exec(function() {vari.onResize();}, false /* don't wait */);
};
iPhoneVariablePane.prototype.onResize = function()
{
	this.adjustHeightMiddle();
	
	this.onResizeChildren();

};
iPhoneVariablePane.prototype.setHeight = function(newHeight)
{
	this.viewNode.style.height = newHeight;
};
iPhoneVariablePane.prototype.adjustHeightMiddle = function()
{
	var par = this.viewNode.parentNode;
	var pheight = getVisibleSize(par).height;
	
	var kidsHeight = 0;
	var kids = par.childNodes;   // Get the list of children
    
    for(var i = 0; i < kids.length; i++)  // calc sibilings height
     {
 		if (kids[i].className != 'Variable')
 		{
 			kidsHeight += getVisibleSize(kids[i]).height;
 		}
 	} 	
	var delta = pheight - kidsHeight;
	this.setHeight(delta);
}
VariableNodeDisplayElement.prototype = new DisplayElement();
VariableNodeDisplayElement.prototype.constructor = VariableNodeDisplayElement;
VariableNodeDisplayElement.prototype.createChild = function createChild(parent, modelId, value)
{
	if (parent.currentWindow) // parent is a real display node
	{
		//TODO fix this so that 'onDelete()' is called for all children
		// (currently, some cleanup may be skipped)
		parent.children = [];
		parent.viewNode.innerHTML = '';
	}
	var child = DisplayElement.prototype.createChild.call(this,parent, modelId, value);
	return child;
};
VariableNodeDisplayElement.prototype.replaceChild = function replaceChild(parent, value)
{
	//TODO(x) complete the refactoring - replace deepCopy with copyValue. Compare with base implementation to see if it's needed at all
	if (parent.currentWindow) // parent is a real display node
	{
		var child = this.getChild(parent);
		if (child)
			child.deepCopy(value);
		else
			child = this.createChild(parent, this.modelId, value);		
	}
	else
	{
		Element.prototype.replaceChild.call(this, parent, value);
	}
};
// file: iscroll.non-msie.js
/*!
 * iScroll v4.1.9 ~ Copyright (c) 2011 Matteo Spinelli, http://cubiq.org
 * Released under MIT license, http://cubiq.org/license
 */
(function(){
var m = Math,
	mround = function (r) { return r >> 0; },
	vendor = (/webkit/i).test(navigator.appVersion) ? 'webkit' :
		(/firefox/i).test(navigator.userAgent) ? 'Moz' :
		'opera' in window ? 'O' : '',

	// Browser capabilities
	has3d = 'WebKitCSSMatrix' in window && 'm11' in new WebKitCSSMatrix(),
	hasTouch = 'ontouchstart' in window,
	hasTransform = vendor + 'Transform' in document.documentElement.style,
	isAndroid = (/android/gi).test(navigator.appVersion),
	isIDevice = (/iphone|ipad/gi).test(navigator.appVersion),
	isPlaybook = (/playbook/gi).test(navigator.appVersion),
	hasTransitionEnd = isIDevice || isPlaybook,
	nextFrame = (function() {
	    return window.requestAnimationFrame
			|| window.webkitRequestAnimationFrame
			|| window.mozRequestAnimationFrame
			|| window.oRequestAnimationFrame
			|| window.msRequestAnimationFrame
			|| function(callback) { return setTimeout(callback, 1); }
	})(),
	cancelFrame = (function () {
	    return window.cancelRequestAnimationFrame
			|| window.webkitCancelRequestAnimationFrame
			|| window.mozCancelRequestAnimationFrame
			|| window.oCancelRequestAnimationFrame
			|| window.msCancelRequestAnimationFrame
			|| clearTimeout
	})(),

	// Events
	RESIZE_EV = 'onorientationchange' in window ? 'orientationchange' : 'resize',
	START_EV = hasTouch ? 'touchstart' : 'mousedown',
	MOVE_EV = hasTouch ? 'touchmove' : 'mousemove',
	END_EV = hasTouch ? 'touchend' : 'mouseup',
	CANCEL_EV = hasTouch ? 'touchcancel' : 'mouseup',
	WHEEL_EV = vendor == 'Moz' ? 'DOMMouseScroll' : 'mousewheel',

	// Helpers
	trnOpen = 'translate' + (has3d ? '3d(' : '('),
	trnClose = has3d ? ',0)' : ')',

	// Constructor
	iScroll = function (el, options) {
		var that = this,
			doc = document,
			i;

		that.wrapper = typeof el == 'object' ? el : doc.getElementById(el);
		that.wrapper.style.overflow = 'hidden';
		that.scroller = that.wrapper.children[0];

		// Default options
		that.options = {
			hScroll: true,
			vScroll: true,
			x: 0,
			y: 0,
			bounce: true,
			bounceLock: false,
			momentum: true,
			lockDirection: true,
			useTransform: true,
			useTransition: false,
			topOffset: 0,
			checkDOMChanges: false,		// Experimental

			// Scrollbar
			hScrollbar: true,
			vScrollbar: true,
			fixedScrollbar: isAndroid,
			hideScrollbar: isIDevice,
			fadeScrollbar: isIDevice && has3d,
			scrollbarClass: '',

			// Zoom
			zoom: false,
			zoomMin: 1,
			zoomMax: 4,
			doubleTapZoom: 2,
			wheelAction: 'scroll',

			// Snap
			snap: false,
			snapThreshold: 1,

			// Events
			onRefresh: null,
			onBeforeScrollStart: function (e) { e.preventDefault(); },
			onScrollStart: null,
			onBeforeScrollMove: null,
			onScrollMove: null,
			onBeforeScrollEnd: null,
			onScrollEnd: null,
			onTouchEnd: null,
			onDestroy: null,
			onZoomStart: null,
			onZoom: null,
			onZoomEnd: null
		};
		// Helpers FIX ANDROID BUG!
		// translate3d and scale doesn't work together! 
		// Ignoring 3d ONLY WHEN YOU SET that.zoom
		if ( that.zoom && isAndroid ){
			trnOpen = 'translate(';
			trnClose = ')';
		}
		// User defined options
		for (i in options) that.options[i] = options[i];
		
		// Set starting position
		that.x = that.options.x;
		that.y = that.options.y;

		// Normalize options
		that.options.useTransform = hasTransform ? that.options.useTransform : false;
		that.options.hScrollbar = that.options.hScroll && that.options.hScrollbar;
		that.options.vScrollbar = that.options.vScroll && that.options.vScrollbar;
		that.options.zoom = that.options.useTransform && that.options.zoom;
		that.options.useTransition = hasTransitionEnd && that.options.useTransition;
		
		// Set some default styles
		that.scroller.style[vendor + 'TransitionProperty'] = that.options.useTransform ? '-' + vendor.toLowerCase() + '-transform' : 'top left';
		that.scroller.style[vendor + 'TransitionDuration'] = '0';
		that.scroller.style[vendor + 'TransformOrigin'] = '0 0';
		if (that.options.useTransition) that.scroller.style[vendor + 'TransitionTimingFunction'] = 'cubic-bezier(0.33,0.66,0.66,1)';
		
		if (that.options.useTransform) that.scroller.style[vendor + 'Transform'] = trnOpen + that.x + 'px,' + that.y + 'px' + trnClose;
		else that.scroller.style.cssText += ';position:absolute;top:' + that.y + 'px;left:' + that.x + 'px';

		if (that.options.useTransition) that.options.fixedScrollbar = true;

		that.refresh();

		that._bind(RESIZE_EV, window);
		that._bind(START_EV);
		if (!hasTouch) {
			that._bind('mouseout', that.wrapper);
			that._bind(WHEEL_EV);
		}

		if (that.options.checkDOMChanges) that.checkDOMTime = setInterval(function () {
			that._checkDOMChanges();
		}, 500);
	};

// Prototype
iScroll.prototype = {
	enabled: true,
	x: 0,
	y: 0,
	steps: [],
	scale: 1,
	currPageX: 0, currPageY: 0,
	pagesX: [], pagesY: [],
	aniTime: null,
	wheelZoomCount: 0,
	
	handleEvent: function (e) {
		var that = this;
		switch(e.type) {
			case START_EV:
				if (!hasTouch && e.button !== 0) return;
				that._start(e);
				break;
			case MOVE_EV: that._move(e); break;
			case END_EV:
			case CANCEL_EV: that._end(e); break;
			case RESIZE_EV: that._resize(); break;
			case WHEEL_EV: that._wheel(e); break;
			case 'mouseout': that._mouseout(e); break;
			case 'webkitTransitionEnd': that._transitionEnd(e); break;
		}
	},
	
	_checkDOMChanges: function () {
		if (this.moved || this.zoomed || this.animating ||
			(this.scrollerW == this.scroller.offsetWidth * this.scale && this.scrollerH == this.scroller.offsetHeight * this.scale)) return;

		this.refresh();
	},
	
	_scrollbar: function (dir) {
		var that = this,
			doc = document,
			bar;

		if (!that[dir + 'Scrollbar']) {
			if (that[dir + 'ScrollbarWrapper']) {
				if (hasTransform) that[dir + 'ScrollbarIndicator'].style[vendor + 'Transform'] = '';
				that[dir + 'ScrollbarWrapper'].parentNode.removeChild(that[dir + 'ScrollbarWrapper']);
				that[dir + 'ScrollbarWrapper'] = null;
				that[dir + 'ScrollbarIndicator'] = null;
			}

			return;
		}

		if (!that[dir + 'ScrollbarWrapper']) {
			// Create the scrollbar wrapper
			bar = doc.createElement('div');

			if (that.options.scrollbarClass) bar.className = that.options.scrollbarClass + dir.toUpperCase();
			else bar.style.cssText = 'position:absolute;z-index:100;' + (dir == 'h' ? 'height:7px;bottom:1px;left:2px;right:' + (that.vScrollbar ? '7' : '2') + 'px' : 'width:7px;bottom:' + (that.hScrollbar ? '7' : '2') + 'px;top:2px;right:1px');

			bar.style.cssText += ';pointer-events:none;-' + vendor + '-transition-property:opacity;-' + vendor + '-transition-duration:' + (that.options.fadeScrollbar ? '350ms' : '0') + ';overflow:hidden;opacity:' + (that.options.hideScrollbar ? '0' : '1');

			that.wrapper.appendChild(bar);
			that[dir + 'ScrollbarWrapper'] = bar;

			// Create the scrollbar indicator
			bar = doc.createElement('div');
			if (!that.options.scrollbarClass) {
				bar.style.cssText = 'position:absolute;z-index:100;background:rgba(0,0,0,0.5);border:1px solid rgba(255,255,255,0.9);-' + vendor + '-background-clip:padding-box;-' + vendor + '-box-sizing:border-box;' + (dir == 'h' ? 'height:100%' : 'width:100%') + ';-' + vendor + '-border-radius:3px;border-radius:3px';
			}
			bar.style.cssText += ';pointer-events:none;-' + vendor + '-transition-property:-' + vendor + '-transform;-' + vendor + '-transition-timing-function:cubic-bezier(0.33,0.66,0.66,1);-' + vendor + '-transition-duration:0;-' + vendor + '-transform:' + trnOpen + '0,0' + trnClose;
			if (that.options.useTransition) bar.style.cssText += ';-' + vendor + '-transition-timing-function:cubic-bezier(0.33,0.66,0.66,1)';

			that[dir + 'ScrollbarWrapper'].appendChild(bar);
			that[dir + 'ScrollbarIndicator'] = bar;
		}

		if (dir == 'h') {
			that.hScrollbarSize = that.hScrollbarWrapper.clientWidth;
			that.hScrollbarIndicatorSize = m.max(mround(that.hScrollbarSize * that.hScrollbarSize / that.scrollerW), 8);
			that.hScrollbarIndicator.style.width = that.hScrollbarIndicatorSize + 'px';
			that.hScrollbarMaxScroll = that.hScrollbarSize - that.hScrollbarIndicatorSize;
			that.hScrollbarProp = that.hScrollbarMaxScroll / that.maxScrollX;
		} else {
			that.vScrollbarSize = that.vScrollbarWrapper.clientHeight;
			that.vScrollbarIndicatorSize = m.max(mround(that.vScrollbarSize * that.vScrollbarSize / that.scrollerH), 8);
			that.vScrollbarIndicator.style.height = that.vScrollbarIndicatorSize + 'px';
			that.vScrollbarMaxScroll = that.vScrollbarSize - that.vScrollbarIndicatorSize;
			that.vScrollbarProp = that.vScrollbarMaxScroll / that.maxScrollY;
		}

		// Reset position
		that._scrollbarPos(dir, true);
	},
	
	_resize: function () {
		var that = this;
		setTimeout(function () { that.refresh(); }, isAndroid ? 200 : 0);
	},
	
	_pos: function (x, y) {
		x = this.hScroll ? x : 0;
		y = this.vScroll ? y : 0;

		if (this.options.useTransform) {
			this.scroller.style[vendor + 'Transform'] = trnOpen + x + 'px,' + y + 'px' + trnClose + ' scale(' + this.scale + ')';
		} else {
			x = mround(x);
			y = mround(y);
			this.scroller.style.left = x + 'px';
			this.scroller.style.top = y + 'px';
		}

		this.x = x;
		this.y = y;

		this._scrollbarPos('h');
		this._scrollbarPos('v');
	},

	_scrollbarPos: function (dir, hidden) {
		var that = this,
			pos = dir == 'h' ? that.x : that.y,
			size;

		if (!that[dir + 'Scrollbar']) return;

		pos = that[dir + 'ScrollbarProp'] * pos;

		if (pos < 0) {
			if (!that.options.fixedScrollbar) {
				size = that[dir + 'ScrollbarIndicatorSize'] + mround(pos * 3);
				if (size < 8) size = 8;
				that[dir + 'ScrollbarIndicator'].style[dir == 'h' ? 'width' : 'height'] = size + 'px';
			}
			pos = 0;
		} else if (pos > that[dir + 'ScrollbarMaxScroll']) {
			if (!that.options.fixedScrollbar) {
				size = that[dir + 'ScrollbarIndicatorSize'] - mround((pos - that[dir + 'ScrollbarMaxScroll']) * 3);
				if (size < 8) size = 8;
				that[dir + 'ScrollbarIndicator'].style[dir == 'h' ? 'width' : 'height'] = size + 'px';
				pos = that[dir + 'ScrollbarMaxScroll'] + (that[dir + 'ScrollbarIndicatorSize'] - size);
			} else {
				pos = that[dir + 'ScrollbarMaxScroll'];
			}
		}

		that[dir + 'ScrollbarWrapper'].style[vendor + 'TransitionDelay'] = '0';
		that[dir + 'ScrollbarWrapper'].style.opacity = hidden && that.options.hideScrollbar ? '0' : '1';
		that[dir + 'ScrollbarIndicator'].style[vendor + 'Transform'] = trnOpen + (dir == 'h' ? pos + 'px,0' : '0,' + pos + 'px') + trnClose;
	},
	
	_start: function (e) {
		var that = this,
			point = hasTouch ? e.touches[0] : e,
			matrix, x, y,
			c1, c2;

		if (!that.enabled) return;

		if (that.options.onBeforeScrollStart) that.options.onBeforeScrollStart.call(that, e);

		if (that.options.useTransition || that.options.zoom) that._transitionTime(0);

		that.moved = false;
		that.animating = false;
		that.zoomed = false;
		that.distX = 0;
		that.distY = 0;
		that.absDistX = 0;
		that.absDistY = 0;
		that.dirX = 0;
		that.dirY = 0;

		// Gesture start
		if (that.options.zoom && hasTouch && e.touches.length > 1) {
			c1 = m.abs(e.touches[0].pageX-e.touches[1].pageX);
			c2 = m.abs(e.touches[0].pageY-e.touches[1].pageY);
			that.touchesDistStart = m.sqrt(c1 * c1 + c2 * c2);

			that.originX = m.abs(e.touches[0].pageX + e.touches[1].pageX - that.wrapperOffsetLeft * 2) / 2 - that.x;
			that.originY = m.abs(e.touches[0].pageY + e.touches[1].pageY - that.wrapperOffsetTop * 2) / 2 - that.y;

			if (that.options.onZoomStart) that.options.onZoomStart.call(that, e);
		}

		if (that.options.momentum) {
			if (that.options.useTransform) {
				// Very lame general purpose alternative to CSSMatrix
				matrix = getComputedStyle(that.scroller, null)[vendor + 'Transform'].replace(/[^0-9-.,]/g, '').split(',');
				x = matrix[4] * 1;
				y = matrix[5] * 1;
			} else {
				x = getComputedStyle(that.scroller, null).left.replace(/[^0-9-]/g, '') * 1;
				y = getComputedStyle(that.scroller, null).top.replace(/[^0-9-]/g, '') * 1;
			}
			
			if (x != that.x || y != that.y) {
				if (that.options.useTransition) that._unbind('webkitTransitionEnd');
				else cancelFrame(that.aniTime);
				that.steps = [];
				that._pos(x, y);
			}
		}

		that.absStartX = that.x;	// Needed by snap threshold
		that.absStartY = that.y;

		that.startX = that.x;
		that.startY = that.y;
		that.pointX = point.pageX;
		that.pointY = point.pageY;

		that.startTime = e.timeStamp || Date.now();

		if (that.options.onScrollStart) that.options.onScrollStart.call(that, e);

		that._bind(MOVE_EV);
		that._bind(END_EV);
		that._bind(CANCEL_EV);
	},
	
	_move: function (e) {
		var that = this,
			point = hasTouch ? e.touches[0] : e,
			deltaX = point.pageX - that.pointX,
			deltaY = point.pageY - that.pointY,
			newX = that.x + deltaX,
			newY = that.y + deltaY,
			c1, c2, scale,
			timestamp = e.timeStamp || Date.now();

		if (that.options.onBeforeScrollMove) that.options.onBeforeScrollMove.call(that, e);

		// Zoom
		if (that.options.zoom && hasTouch && e.touches.length > 1) {
			c1 = m.abs(e.touches[0].pageX - e.touches[1].pageX);
			c2 = m.abs(e.touches[0].pageY - e.touches[1].pageY);
			that.touchesDist = m.sqrt(c1*c1+c2*c2);

			that.zoomed = true;

			scale = 1 / that.touchesDistStart * that.touchesDist * this.scale;

			if (scale < that.options.zoomMin) scale = 0.5 * that.options.zoomMin * Math.pow(2.0, scale / that.options.zoomMin);
			else if (scale > that.options.zoomMax) scale = 2.0 * that.options.zoomMax * Math.pow(0.5, that.options.zoomMax / scale);

			that.lastScale = scale / this.scale;

			newX = this.originX - this.originX * that.lastScale + this.x,
			newY = this.originY - this.originY * that.lastScale + this.y;

			this.scroller.style[vendor + 'Transform'] = trnOpen + newX + 'px,' + newY + 'px' + trnClose + ' scale(' + scale + ')';

			if (that.options.onZoom) that.options.onZoom.call(that, e);
			return;
		}

		that.pointX = point.pageX;
		that.pointY = point.pageY;

		// Slow down if outside of the boundaries
		if (newX > 0 || newX < that.maxScrollX) {
			newX = that.options.bounce ? that.x + (deltaX / 2) : newX >= 0 || that.maxScrollX >= 0 ? 0 : that.maxScrollX;
		}
		if (newY > that.minScrollY || newY < that.maxScrollY) { 
			newY = that.options.bounce ? that.y + (deltaY / 2) : newY >= that.minScrollY || that.maxScrollY >= 0 ? that.minScrollY : that.maxScrollY;
		}

		if (that.absDistX < 6 && that.absDistY < 6) {
			that.distX += deltaX;
			that.distY += deltaY;
			that.absDistX = m.abs(that.distX);
			that.absDistY = m.abs(that.distY);

			return;
		}

		// Lock direction
		if (that.options.lockDirection) {
			if (that.absDistX > that.absDistY + 5) {
				newY = that.y;
				deltaY = 0;
			} else if (that.absDistY > that.absDistX + 5) {
				newX = that.x;
				deltaX = 0;
			}
		}

		that.moved = true;
		that._pos(newX, newY);
		that.dirX = deltaX > 0 ? -1 : deltaX < 0 ? 1 : 0;
		that.dirY = deltaY > 0 ? -1 : deltaY < 0 ? 1 : 0;

		if (timestamp - that.startTime > 300) {
			that.startTime = timestamp;
			that.startX = that.x;
			that.startY = that.y;
		}
		
		if (that.options.onScrollMove) that.options.onScrollMove.call(that, e);
	},
	
	_end: function (e) {
		if (hasTouch && e.touches.length != 0) return;

		var that = this,
			point = hasTouch ? e.changedTouches[0] : e,
			target, ev,
			momentumX = { dist:0, time:0 },
			momentumY = { dist:0, time:0 },
			duration = (e.timeStamp || Date.now()) - that.startTime,
			newPosX = that.x,
			newPosY = that.y,
			distX, distY,
			newDuration,
			snap,
			scale;

		that._unbind(MOVE_EV);
		that._unbind(END_EV);
		that._unbind(CANCEL_EV);

		if (that.options.onBeforeScrollEnd) that.options.onBeforeScrollEnd.call(that, e);

		if (that.zoomed) {
			scale = that.scale * that.lastScale;
			scale = Math.max(that.options.zoomMin, scale);
			scale = Math.min(that.options.zoomMax, scale);
			that.lastScale = scale / that.scale;
			that.scale = scale;

			that.x = that.originX - that.originX * that.lastScale + that.x;
			that.y = that.originY - that.originY * that.lastScale + that.y;
			
			that.scroller.style[vendor + 'TransitionDuration'] = '200ms';
			that.scroller.style[vendor + 'Transform'] = trnOpen + that.x + 'px,' + that.y + 'px' + trnClose + ' scale(' + that.scale + ')';
			
			that.zoomed = false;
			that.refresh();

			if (that.options.onZoomEnd) that.options.onZoomEnd.call(that, e);
			return;
		}

		if (!that.moved) {
			if (hasTouch) {
				if (that.doubleTapTimer && that.options.zoom) {
					// Double tapped
					clearTimeout(that.doubleTapTimer);
					that.doubleTapTimer = null;
					if (that.options.onZoomStart) that.options.onZoomStart.call(that, e);
					that.zoom(that.pointX, that.pointY, that.scale == 1 ? that.options.doubleTapZoom : 1);
					if (that.options.onZoomEnd) {
						setTimeout(function() {
							that.options.onZoomEnd.call(that, e);
						}, 200); // 200 is default zoom duration
					}
				} else {
					that.doubleTapTimer = setTimeout(function () {
						that.doubleTapTimer = null;

						// Find the last touched element
						target = point.target;
						while (target.nodeType != 1) target = target.parentNode;

						if (target.tagName != 'SELECT' && target.tagName != 'INPUT' && target.tagName != 'TEXTAREA') {
							ev = document.createEvent('MouseEvents');
							ev.initMouseEvent('click', true, true, e.view, 1,
								point.screenX, point.screenY, point.clientX, point.clientY,
								e.ctrlKey, e.altKey, e.shiftKey, e.metaKey,
								0, null);
							ev._fake = true;
							target.dispatchEvent(ev);
						}
					}, that.options.zoom ? 250 : 0);
				}
			}

			that._resetPos(200);

			if (that.options.onTouchEnd) that.options.onTouchEnd.call(that, e);
			return;
		}

		if (duration < 300 && that.options.momentum) {
			momentumX = newPosX ? that._momentum(newPosX - that.startX, duration, -that.x, that.scrollerW - that.wrapperW + that.x, that.options.bounce ? that.wrapperW : 0) : momentumX;
			momentumY = newPosY ? that._momentum(newPosY - that.startY, duration, -that.y, (that.maxScrollY < 0 ? that.scrollerH - that.wrapperH + that.y - that.minScrollY : 0), that.options.bounce ? that.wrapperH : 0) : momentumY;

			newPosX = that.x + momentumX.dist;
			newPosY = that.y + momentumY.dist;

 			if ((that.x > 0 && newPosX > 0) || (that.x < that.maxScrollX && newPosX < that.maxScrollX)) momentumX = { dist:0, time:0 };
 			if ((that.y > that.minScrollY && newPosY > that.minScrollY) || (that.y < that.maxScrollY && newPosY < that.maxScrollY)) momentumY = { dist:0, time:0 };
		}

		if (momentumX.dist || momentumY.dist) {
			newDuration = m.max(m.max(momentumX.time, momentumY.time), 10);

			// Do we need to snap?
			if (that.options.snap) {
				distX = newPosX - that.absStartX;
				distY = newPosY - that.absStartY;
				if (m.abs(distX) < that.options.snapThreshold && m.abs(distY) < that.options.snapThreshold) { that.scrollTo(that.absStartX, that.absStartY, 200); }
				else {
					snap = that._snap(newPosX, newPosY);
					newPosX = snap.x;
					newPosY = snap.y;
					newDuration = m.max(snap.time, newDuration);
				}
			}

			that.scrollTo(mround(newPosX), mround(newPosY), newDuration);

			if (that.options.onTouchEnd) that.options.onTouchEnd.call(that, e);
			return;
		}

		// Do we need to snap?
		if (that.options.snap) {
			distX = newPosX - that.absStartX;
			distY = newPosY - that.absStartY;
			if (m.abs(distX) < that.options.snapThreshold && m.abs(distY) < that.options.snapThreshold) that.scrollTo(that.absStartX, that.absStartY, 200);
			else {
				snap = that._snap(that.x, that.y);
				if (snap.x != that.x || snap.y != that.y) that.scrollTo(snap.x, snap.y, snap.time);
			}

			if (that.options.onTouchEnd) that.options.onTouchEnd.call(that, e);
			return;
		}

		that._resetPos(200);
		if (that.options.onTouchEnd) that.options.onTouchEnd.call(that, e);
	},
	
	_resetPos: function (time) {
		var that = this,
			resetX = that.x >= 0 ? 0 : that.x < that.maxScrollX ? that.maxScrollX : that.x,
			resetY = that.y >= that.minScrollY || that.maxScrollY > 0 ? that.minScrollY : that.y < that.maxScrollY ? that.maxScrollY : that.y;

		if (resetX == that.x && resetY == that.y) {
			if (that.moved) {
				that.moved = false;
				if (that.options.onScrollEnd) that.options.onScrollEnd.call(that);		// Execute custom code on scroll end
			}

			if (that.hScrollbar && that.options.hideScrollbar) {
				if (vendor == 'webkit') that.hScrollbarWrapper.style[vendor + 'TransitionDelay'] = '300ms';
				that.hScrollbarWrapper.style.opacity = '0';
			}
			if (that.vScrollbar && that.options.hideScrollbar) {
				if (vendor == 'webkit') that.vScrollbarWrapper.style[vendor + 'TransitionDelay'] = '300ms';
				that.vScrollbarWrapper.style.opacity = '0';
			}

			return;
		}

		that.scrollTo(resetX, resetY, time || 0);
	},

	_wheel: function (e) {
		var that = this,
			wheelDeltaX, wheelDeltaY,
			deltaX, deltaY,
			deltaScale;

		if ('wheelDeltaX' in e) {
			wheelDeltaX = e.wheelDeltaX / 12;
			wheelDeltaY = e.wheelDeltaY / 12;
		} else if ('detail' in e) {
			wheelDeltaX = wheelDeltaY = -e.detail * 3;
		} else {
			wheelDeltaX = wheelDeltaY = -e.wheelDelta;
		}
		
		if (that.options.wheelAction == 'zoom') {
			deltaScale = that.scale * Math.pow(2, 1/3 * (wheelDeltaY ? wheelDeltaY / Math.abs(wheelDeltaY) : 0));
			if (deltaScale < that.options.zoomMin) deltaScale = that.options.zoomMin;
			if (deltaScale > that.options.zoomMax) deltaScale = that.options.zoomMax;
			
			if (deltaScale != that.scale) {
				if (!that.wheelZoomCount && that.options.onZoomStart) that.options.onZoomStart.call(that, e);
				that.wheelZoomCount++;
				
				that.zoom(e.pageX, e.pageY, deltaScale, 400);
				
				setTimeout(function() {
					that.wheelZoomCount--;
					if (!that.wheelZoomCount && that.options.onZoomEnd) that.options.onZoomEnd.call(that, e);
				}, 400);
			}
			
			return;
		}
		
		deltaX = that.x + wheelDeltaX;
		deltaY = that.y + wheelDeltaY;

		if (deltaX > 0) deltaX = 0;
		else if (deltaX < that.maxScrollX) deltaX = that.maxScrollX;

		if (deltaY > that.minScrollY) deltaY = that.minScrollY;
		else if (deltaY < that.maxScrollY) deltaY = that.maxScrollY;

		that.scrollTo(deltaX, deltaY, 0);
	},
	
	_mouseout: function (e) {
		var t = e.relatedTarget;

		if (!t) {
			this._end(e);
			return;
		}

		while (t = t.parentNode) if (t == this.wrapper) return;
		
		this._end(e);
	},

	_transitionEnd: function (e) {
		var that = this;

		if (e.target != that.scroller) return;

		that._unbind('webkitTransitionEnd');
		
		that._startAni();
	},


	/**
	 *
	 * Utilities
	 *
	 */
	_startAni: function () {
		var that = this,
			startX = that.x, startY = that.y,
			startTime = Date.now(),
			step, easeOut,
			animate;

		if (that.animating) return;
		
		if (!that.steps.length) {
			that._resetPos(400);
			return;
		}
		
		step = that.steps.shift();
		
		if (step.x == startX && step.y == startY) step.time = 0;

		that.animating = true;
		that.moved = true;
		
		if (that.options.useTransition) {
			that._transitionTime(step.time);
			that._pos(step.x, step.y);
			that.animating = false;
			if (step.time) that._bind('webkitTransitionEnd');
			else that._resetPos(0);
			return;
		}

		animate = function () {
			var now = Date.now(),
				newX, newY;

			if (now >= startTime + step.time) {
				that._pos(step.x, step.y);
				that.animating = false;
				if (that.options.onAnimationEnd) that.options.onAnimationEnd.call(that);			// Execute custom code on animation end
				that._startAni();
				return;
			}

			now = (now - startTime) / step.time - 1;
			easeOut = m.sqrt(1 - now * now);
			newX = (step.x - startX) * easeOut + startX;
			newY = (step.y - startY) * easeOut + startY;
			that._pos(newX, newY);
			if (that.animating) that.aniTime = nextFrame(animate);
		};

		animate();
	},

	_transitionTime: function (time) {
		time += 'ms';
		this.scroller.style[vendor + 'TransitionDuration'] = time;
		if (this.hScrollbar) this.hScrollbarIndicator.style[vendor + 'TransitionDuration'] = time;
		if (this.vScrollbar) this.vScrollbarIndicator.style[vendor + 'TransitionDuration'] = time;
	},

	_momentum: function (dist, time, maxDistUpper, maxDistLower, size) {
		var deceleration = 0.0006,
			speed = m.abs(dist) / time,
			newDist = (speed * speed) / (2 * deceleration),
			newTime = 0, outsideDist = 0;

		// Proportinally reduce speed if we are outside of the boundaries 
		if (dist > 0 && newDist > maxDistUpper) {
			outsideDist = size / (6 / (newDist / speed * deceleration));
			maxDistUpper = maxDistUpper + outsideDist;
			speed = speed * maxDistUpper / newDist;
			newDist = maxDistUpper;
		} else if (dist < 0 && newDist > maxDistLower) {
			outsideDist = size / (6 / (newDist / speed * deceleration));
			maxDistLower = maxDistLower + outsideDist;
			speed = speed * maxDistLower / newDist;
			newDist = maxDistLower;
		}

		newDist = newDist * (dist < 0 ? -1 : 1);
		newTime = speed / deceleration;

		return { dist: newDist, time: mround(newTime) };
	},

	_offset: function (el) {
		var left = -el.offsetLeft,
			top = -el.offsetTop;
			
		while (el = el.offsetParent) {
			left -= el.offsetLeft;
			top -= el.offsetTop;
		}
		
		if (el != this.wrapper) {
			left *= this.scale;
			top *= this.scale;
		}

		return { left: left, top: top };
	},

	_snap: function (x, y) {
		var that = this,
			i, l,
			page, time,
			sizeX, sizeY;

		// Check page X
		page = that.pagesX.length - 1;
		for (i=0, l=that.pagesX.length; i<l; i++) {
			if (x >= that.pagesX[i]) {
				page = i;
				break;
			}
		}
		if (page == that.currPageX && page > 0 && that.dirX < 0) page--;
		x = that.pagesX[page];
		sizeX = m.abs(x - that.pagesX[that.currPageX]);
		sizeX = sizeX ? m.abs(that.x - x) / sizeX * 500 : 0;
		that.currPageX = page;

		// Check page Y
		page = that.pagesY.length-1;
		for (i=0; i<page; i++) {
			if (y >= that.pagesY[i]) {
				page = i;
				break;
			}
		}
		if (page == that.currPageY && page > 0 && that.dirY < 0) page--;
		y = that.pagesY[page];
		sizeY = m.abs(y - that.pagesY[that.currPageY]);
		sizeY = sizeY ? m.abs(that.y - y) / sizeY * 500 : 0;
		that.currPageY = page;

		// Snap with constant speed (proportional duration)
		time = mround(m.max(sizeX, sizeY)) || 200;

		return { x: x, y: y, time: time };
	},

	_bind: function (type, el, bubble) {
		(el || this.scroller).addEventListener(type, this, !!bubble);
	},

	_unbind: function (type, el, bubble) {
		(el || this.scroller).removeEventListener(type, this, !!bubble);
	},


	/**
	 *
	 * Public methods
	 *
	 */
	destroy: function () {
		var that = this;

		that.scroller.style[vendor + 'Transform'] = '';

		// Remove the scrollbars
		that.hScrollbar = false;
		that.vScrollbar = false;
		that._scrollbar('h');
		that._scrollbar('v');

		// Remove the event listeners
		that._unbind(RESIZE_EV, window);
		that._unbind(START_EV);
		that._unbind(MOVE_EV);
		that._unbind(END_EV);
		that._unbind(CANCEL_EV);
		
		if (that.options.hasTouch) {
			that._unbind('mouseout', that.wrapper);
			that._unbind(WHEEL_EV);
		}
		
		if (that.options.useTransition) that._unbind('webkitTransitionEnd');
		
		if (that.options.checkDOMChanges) clearInterval(that.checkDOMTime);
		
		if (that.options.onDestroy) that.options.onDestroy.call(that);
	},

	refresh: function () {
		var that = this,
			offset,
			i, l,
			els,
			pos = 0,
			page = 0;

		if (that.scale < that.options.zoomMin) that.scale = that.options.zoomMin;
		that.wrapperW = that.wrapper.clientWidth || 1;
		that.wrapperH = that.wrapper.clientHeight || 1;

		that.minScrollY = -that.options.topOffset || 0;
		that.scrollerW = mround(that.scroller.offsetWidth * that.scale);
		that.scrollerH = mround((that.scroller.offsetHeight + that.minScrollY) * that.scale);
		that.maxScrollX = that.wrapperW - that.scrollerW;
		that.maxScrollY = that.wrapperH - that.scrollerH + that.minScrollY;
		that.dirX = 0;
		that.dirY = 0;

		if (that.options.onRefresh) that.options.onRefresh.call(that);

		that.hScroll = that.options.hScroll && that.maxScrollX < 0;
		that.vScroll = that.options.vScroll && (!that.options.bounceLock && !that.hScroll || that.scrollerH > that.wrapperH);

		that.hScrollbar = that.hScroll && that.options.hScrollbar;
		that.vScrollbar = that.vScroll && that.options.vScrollbar && that.scrollerH > that.wrapperH;

		offset = that._offset(that.wrapper);
		that.wrapperOffsetLeft = -offset.left;
		that.wrapperOffsetTop = -offset.top;

		// Prepare snap
		if (typeof that.options.snap == 'string') {
			that.pagesX = [];
			that.pagesY = [];
			els = that.scroller.querySelectorAll(that.options.snap);
			for (i=0, l=els.length; i<l; i++) {
				pos = that._offset(els[i]);
				pos.left += that.wrapperOffsetLeft;
				pos.top += that.wrapperOffsetTop;
				that.pagesX[i] = pos.left < that.maxScrollX ? that.maxScrollX : pos.left * that.scale;
				that.pagesY[i] = pos.top < that.maxScrollY ? that.maxScrollY : pos.top * that.scale;
			}
		} else if (that.options.snap) {
			that.pagesX = [];
			while (pos >= that.maxScrollX) {
				that.pagesX[page] = pos;
				pos = pos - that.wrapperW;
				page++;
			}
			if (that.maxScrollX%that.wrapperW) that.pagesX[that.pagesX.length] = that.maxScrollX - that.pagesX[that.pagesX.length-1] + that.pagesX[that.pagesX.length-1];

			pos = 0;
			page = 0;
			that.pagesY = [];
			while (pos >= that.maxScrollY) {
				that.pagesY[page] = pos;
				pos = pos - that.wrapperH;
				page++;
			}
			if (that.maxScrollY%that.wrapperH) that.pagesY[that.pagesY.length] = that.maxScrollY - that.pagesY[that.pagesY.length-1] + that.pagesY[that.pagesY.length-1];
		}

		// Prepare the scrollbars
		that._scrollbar('h');
		that._scrollbar('v');

		if (!that.zoomed) {
			that.scroller.style[vendor + 'TransitionDuration'] = '0';
			that._resetPos(200);
		}
	},

	scrollTo: function (x, y, time, relative) {
		var that = this,
			step = x,
			i, l;

		that.stop();

		if (!step.length) step = [{ x: x, y: y, time: time, relative: relative }];
		
		for (i=0, l=step.length; i<l; i++) {
			if (step[i].relative) { step[i].x = that.x - step[i].x; step[i].y = that.y - step[i].y; }
			that.steps.push({ x: step[i].x, y: step[i].y, time: step[i].time || 0 });
		}

		that._startAni();
	},

	scrollToElement: function (el, time) {
		var that = this, pos;
		el = el.nodeType ? el : that.scroller.querySelector(el);
		if (!el) return;

		pos = that._offset(el);
		pos.left += that.wrapperOffsetLeft;
		pos.top += that.wrapperOffsetTop;

		pos.left = pos.left > 0 ? 0 : pos.left < that.maxScrollX ? that.maxScrollX : pos.left;
		pos.top = pos.top > that.minScrollY ? that.minScrollY : pos.top < that.maxScrollY ? that.maxScrollY : pos.top;
		time = time === undefined ? m.max(m.abs(pos.left)*2, m.abs(pos.top)*2) : time;

		that.scrollTo(pos.left, pos.top, time);
	},

	scrollToPage: function (pageX, pageY, time) {
		var that = this, x, y;

		if (that.options.onScrollStart) that.options.onScrollStart.call(that);

		if (that.options.snap) {
			pageX = pageX == 'next' ? that.currPageX+1 : pageX == 'prev' ? that.currPageX-1 : pageX;
			pageY = pageY == 'next' ? that.currPageY+1 : pageY == 'prev' ? that.currPageY-1 : pageY;

			pageX = pageX < 0 ? 0 : pageX > that.pagesX.length-1 ? that.pagesX.length-1 : pageX;
			pageY = pageY < 0 ? 0 : pageY > that.pagesY.length-1 ? that.pagesY.length-1 : pageY;

			that.currPageX = pageX;
			that.currPageY = pageY;
			x = that.pagesX[pageX];
			y = that.pagesY[pageY];
		} else {
			x = -that.wrapperW * pageX;
			y = -that.wrapperH * pageY;
			if (x < that.maxScrollX) x = that.maxScrollX;
			if (y < that.maxScrollY) y = that.maxScrollY;
		}

		that.scrollTo(x, y, time || 400);
	},

	disable: function () {
		this.stop();
		this._resetPos(0);
		this.enabled = false;

		// If disabled after touchstart we make sure that there are no left over events
		this._unbind(MOVE_EV);
		this._unbind(END_EV);
		this._unbind(CANCEL_EV);
	},
	
	enable: function () {
		this.enabled = true;
	},
	
	stop: function () {
		if (this.options.useTransition) this._unbind('webkitTransitionEnd');
		else cancelFrame(this.aniTime);
		this.steps = [];
		this.moved = false;
		this.animating = false;
	},
	
	zoom: function (x, y, scale, time) {
		var that = this,
			relScale = scale / that.scale;

		if (!that.options.useTransform) return;

		that.zoomed = true;
		time = time === undefined ? 200 : time;
		x = x - that.wrapperOffsetLeft - that.x;
		y = y - that.wrapperOffsetTop - that.y;
		that.x = x - x * relScale + that.x;
		that.y = y - y * relScale + that.y;

		that.scale = scale;
		that.refresh();

		that.x = that.x > 0 ? 0 : that.x < that.maxScrollX ? that.maxScrollX : that.x;
		that.y = that.y > that.minScrollY ? that.minScrollY : that.y < that.maxScrollY ? that.maxScrollY : that.y;

		that.scroller.style[vendor + 'TransitionDuration'] = time + 'ms';
		that.scroller.style[vendor + 'Transform'] = trnOpen + that.x + 'px,' + that.y + 'px' + trnClose + ' scale(' + scale + ')';
		that.zoomed = false;
	},
	
	isReady: function () {
		return !this.moved && !this.zoomed && !this.animating;
	}
};

if (typeof exports !== 'undefined') exports.iScroll = iScroll;
else window.iScroll = iScroll;

})();
// file: onshake.iphone.js
tersus.handleShake = function()
{
	tersus.invokeAction('<On Shake>');
}
// file: settings.js
// Overload // file: spinning_wheels.iphone.non-simulator.js
/**
 * 
 * Find more about the Spinning Wheel function at
 * http://cubiq.org/spinning-wheel-on-webkit-for-iphone-ipod-touch/11
 *
 * Copyright (c) 2009 Matteo Spinelli, http://cubiq.org/
 * Released under MIT license
 * http://cubiq.org/dropbox/mit-license.txt
 * 
 * Version 1.4 - Last updated: 2009.07.09
 * 
 */

var SpinningWheel = {
	cellHeight: 44,
	friction: 0.003,
	slotData: [],


	/**
	 *
	 * Event handler
	 *
	 */

	handleEvent: function (e) {
		if (e.type == 'touchstart') {
			this.lockScreen(e);
			if (e.currentTarget.id == 'sw-cancel' || e.currentTarget.id == 'sw-done') {
				this.tapDown(e);
			} else if (e.currentTarget.id == 'sw-frame') {
				this.scrollStart(e);
			}
		} else if (e.type == 'touchmove') {
			this.lockScreen(e);
			
			if (e.currentTarget.id == 'sw-cancel' || e.currentTarget.id == 'sw-done') {
				this.tapCancel(e);
			} else if (e.currentTarget.id == 'sw-frame') {
				this.scrollMove(e);
			}
		} else if (e.type == 'touchend' || e.type == 'click') {
			if (e.currentTarget.id == 'sw-cancel' || e.currentTarget.id == 'sw-done') {
				this.tapUp(e);
			} else if (e.currentTarget.id == 'sw-frame') {
				this.scrollEnd(e);
			}
		} else if (e.type == 'webkitTransitionEnd') {
			if (e.target.id == 'sw-wrapper') {
				this.destroy();
			} else {
				this.backWithinBoundaries(e);
			}
		} else if (e.type == 'orientationchange') {
			this.onOrientationChange(e);
		} else if (e.type == 'scroll') {
			this.onScroll(e);
		}
	},


	/**
	 *
	 * Global events
	 *
	 */

	onOrientationChange: function (e) {
		window.scrollTo(0, 0);
		this.swWrapper.style.top = window.innerHeight + window.pageYOffset + 'px';
		this.calculateSlotsWidth();
	},
	
	onScroll: function (e) {
		this.swWrapper.style.top = window.innerHeight + window.pageYOffset + 'px';
	},

	lockScreen: function (e) {
		e.preventDefault();
		e.stopPropagation();
	},


	/**
	 *
	 * Initialization
	 *
	 */

	reset: function () {
		this.slotEl = [];

		this.activeSlot = null;
		
		this.swWrapper = undefined;
		this.swSlotWrapper = undefined;
		this.swSlots = undefined;
		this.swFrame = undefined;
	},

	calculateSlotsWidth: function () {
		var div = this.swSlots.getElementsByTagName('div');
		for (var i = 0; i < div.length; i += 1) {
			this.slotEl[i].slotWidth = div[i].offsetWidth;
		}
	},

	create: function () {
		var i, l, out, ul, div;

		this.reset();	// Initialize object variables

		// Create the Spinning Wheel main wrapper
		div = document.createElement('div');
		div.id = 'sw-wrapper';
		div.style.top = window.innerHeight + window.pageYOffset + 'px';		// Place the SW down the actual viewing screen
		div.style.webkitTransitionProperty = '-webkit-transform';
		div.innerHTML = '<div id="sw-header"><div id="sw-cancel">Cancel</' + 'div><div id="sw-done">Done</' + 'div></' + 'div><div id="sw-slots-wrapper"><div id="sw-slots"></' + 'div></' + 'div><div id="sw-frame"></' + 'div>';

		document.body.appendChild(div);

		this.swWrapper = div;													// The SW wrapper
		this.swSlotWrapper = document.getElementById('sw-slots-wrapper');		// Slots visible area
		this.swSlots = document.getElementById('sw-slots');						// Pseudo table element (inner wrapper)
		this.swFrame = document.getElementById('sw-frame');						// The scrolling controller

		// Create HTML slot elements
		for (l = 0; l < this.slotData.length; l += 1) {
			// Create the slot
			ul = document.createElement('ul');
			out = '';
			for (i in this.slotData[l].values) {
				out += '<li>' + this.slotData[l].values[i] + '<' + '/li>';
			}
			ul.innerHTML = out;

			div = document.createElement('div');		// Create slot container
			div.className = this.slotData[l].style;		// Add styles to the container
			div.appendChild(ul);
	
			// Append the slot to the wrapper
			this.swSlots.appendChild(div);
			
			ul.slotPosition = l;			// Save the slot position inside the wrapper
			ul.slotYPosition = 0;
			ul.slotWidth = 0;
			ul.slotMaxScroll = this.swSlotWrapper.clientHeight - ul.clientHeight - 86;
			ul.style.webkitTransitionTimingFunction = 'cubic-bezier(0, 0, 0.2, 1)';		// Add default transition
			
			this.slotEl.push(ul);			// Save the slot for later use
			
			// Place the slot to its default position (if other than 0)
			if (this.slotData[l].defaultValue) {
				this.scrollToValue(l, this.slotData[l].defaultValue);	
			}
		}
		
		this.calculateSlotsWidth();
		
		// Global events
		document.addEventListener('touchstart', this, false);			// Prevent page scrolling
		document.addEventListener('touchmove', this, false);			// Prevent page scrolling
		window.addEventListener('orientationchange', this, true);		// Optimize SW on orientation change
		window.addEventListener('scroll', this, true);				// Reposition SW on page scroll

		// Cancel/Done buttons events
		document.getElementById('sw-cancel').addEventListener('click', this, false);
		document.getElementById('sw-done').addEventListener('click', this, false);
		document.getElementById('sw-cancel').addEventListener('touchstart', this, false);
		document.getElementById('sw-done').addEventListener('touchstart', this, false);

		// Add scrolling to the slots
		this.swFrame.addEventListener('touchstart', this, false);
	},

	open: function () {
		this.create();

		this.swWrapper.style.webkitTransitionTimingFunction = 'ease-out';
		this.swWrapper.style.webkitTransitionDuration = '400ms';
		this.swWrapper.style.webkitTransform = 'translate3d(0, -260px, 0)';
	},
	
	
	/**
	 *
	 * Unload
	 *
	 */

	destroy: function () {
		this.swWrapper.removeEventListener('webkitTransitionEnd', this, false);

		this.swFrame.removeEventListener('touchstart', this, false);

		document.getElementById('sw-cancel').removeEventListener('touchstart', this, false);
		document.getElementById('sw-done').removeEventListener('touchstart', this, false);

		document.removeEventListener('touchstart', this, false);
		document.removeEventListener('touchmove', this, false);
		window.removeEventListener('orientationchange', this, true);
		window.removeEventListener('scroll', this, true);
		
		this.slotData = [];
		this.cancelAction = function () {
			return false;
		};
		
		this.cancelDone = function () {
			return true;
		};
		
		this.reset();
		
		document.body.removeChild(document.getElementById('sw-wrapper'));
	},
	
	close: function () {
		this.swWrapper.style.webkitTransitionTimingFunction = 'ease-in';
		this.swWrapper.style.webkitTransitionDuration = '400ms';
		this.swWrapper.style.webkitTransform = 'translate3d(0, 0, 0)';
		
		this.swWrapper.addEventListener('webkitTransitionEnd', this, false);
	},


	/**
	 *
	 * Generic methods
	 *
	 */

	addSlot: function (values, style, defaultValue) {
		if (!style) {
			style = '';
		}
		
		style = style.split(' ');

		for (var i = 0; i < style.length; i += 1) {
			style[i] = 'sw-' + style[i];
		}
		
		style = style.join(' ');

		var obj = { 'values': values, 'style': style, 'defaultValue': defaultValue };
		this.slotData.push(obj);
	},

	getSelectedValues: function () {
		var index, count,
		    i, l,
			keys = [], values = [];
		for (var i = 0; i < this.slotEl.length; i += 1) {
			// Remove any residual animation
			this.slotEl[i].removeEventListener('webkitTransitionEnd', this, false);
			this.slotEl[i].style.webkitTransitionDuration = '0';

			if (this.slotEl[i].slotYPosition > 0) {
				this.setPosition(i, 0);
			} else if (this.slotEl[i].slotYPosition < this.slotEl[i].slotMaxScroll) {
				this.setPosition(i, this.slotEl[i].slotMaxScroll);
			}

			index = -Math.round(this.slotEl[i].slotYPosition / this.cellHeight);

			count = 0;
			for (l in this.slotData[i].values) {
				if (count == index) {
					keys.push(l);
					values.push(this.slotData[i].values[l]);
					break;
				}
				
				count += 1;
			}
		}

		return { 'keys': keys, 'values': values };
	},


	/**
	 *
	 * Rolling slots
	 *
	 */

	setPosition: function (slot, pos) {
		this.slotEl[slot].slotYPosition = pos;
		this.slotEl[slot].style.webkitTransform = 'translate3d(0, ' + pos + 'px, 0)';
	},
	
	scrollStart: function (e) {
		// Find the clicked slot
		var xPos = e.targetTouches[0].clientX - this.swSlots.offsetLeft;	// Clicked position minus left offset (should be 11px)

		// Find tapped slot
		var slot = 0;
		for (var i = 0; i < this.slotEl.length; i += 1) {
			slot += this.slotEl[i].slotWidth;
			
			if (xPos < slot) {
				this.activeSlot = i;
				break;
			}
		}

		// If slot is readonly do nothing
		if (this.slotData[this.activeSlot].style.match('readonly')) {
			this.swFrame.removeEventListener('touchmove', this, false);
			this.swFrame.removeEventListener('touchend', this, false);
			return false;
		}

		this.slotEl[this.activeSlot].removeEventListener('webkitTransitionEnd', this, false);	// Remove transition event (if any)
		this.slotEl[this.activeSlot].style.webkitTransitionDuration = '0';		// Remove any residual transition
		
		// Stop and hold slot position
		var theTransform = window.getComputedStyle(this.slotEl[this.activeSlot]).webkitTransform;
		theTransform = new WebKitCSSMatrix(theTransform).m42;
		if (theTransform != this.slotEl[this.activeSlot].slotYPosition) {
			this.setPosition(this.activeSlot, theTransform);
		}
		
		this.startY = e.targetTouches[0].clientY;
		this.scrollStartY = this.slotEl[this.activeSlot].slotYPosition;
		this.scrollStartTime = e.timeStamp;

		this.swFrame.addEventListener('touchmove', this, false);
		this.swFrame.addEventListener('touchend', this, false);
		
		return true;
	},

	scrollMove: function (e) {
		var topDelta = e.targetTouches[0].clientY - this.startY;

		if (this.slotEl[this.activeSlot].slotYPosition > 0 || this.slotEl[this.activeSlot].slotYPosition < this.slotEl[this.activeSlot].slotMaxScroll) {
			topDelta /= 2;
		}
		
		this.setPosition(this.activeSlot, this.slotEl[this.activeSlot].slotYPosition + topDelta);
		this.startY = e.targetTouches[0].clientY;

		// Prevent slingshot effect
		if (e.timeStamp - this.scrollStartTime > 80) {
			this.scrollStartY = this.slotEl[this.activeSlot].slotYPosition;
			this.scrollStartTime = e.timeStamp;
		}
	},
	
	scrollEnd: function (e) {
		this.swFrame.removeEventListener('touchmove', this, false);
		this.swFrame.removeEventListener('touchend', this, false);

		// If we are outside of the boundaries, let's go back to the sheepfold
		if (this.slotEl[this.activeSlot].slotYPosition > 0 || this.slotEl[this.activeSlot].slotYPosition < this.slotEl[this.activeSlot].slotMaxScroll) {
			this.scrollTo(this.activeSlot, this.slotEl[this.activeSlot].slotYPosition > 0 ? 0 : this.slotEl[this.activeSlot].slotMaxScroll);
			return false;
		}

		// Lame formula to calculate a fake deceleration
		var scrollDistance = this.slotEl[this.activeSlot].slotYPosition - this.scrollStartY;

		// The drag session was too short
		if (scrollDistance < this.cellHeight / 1.5 && scrollDistance > -this.cellHeight / 1.5) {
			if (this.slotEl[this.activeSlot].slotYPosition % this.cellHeight) {
				this.scrollTo(this.activeSlot, Math.round(this.slotEl[this.activeSlot].slotYPosition / this.cellHeight) * this.cellHeight, '100ms');
			}

			return false;
		}

		var scrollDuration = e.timeStamp - this.scrollStartTime;

		var newDuration = (2 * scrollDistance / scrollDuration) / this.friction;
		var newScrollDistance = (this.friction / 2) * (newDuration * newDuration);
		
		if (newDuration < 0) {
			newDuration = -newDuration;
			newScrollDistance = -newScrollDistance;
		}

		var newPosition = this.slotEl[this.activeSlot].slotYPosition + newScrollDistance;

		if (newPosition > 0) {
			// Prevent the slot to be dragged outside the visible area (top margin)
			newPosition /= 2;
			newDuration /= 3;

			if (newPosition > this.swSlotWrapper.clientHeight / 4) {
				newPosition = this.swSlotWrapper.clientHeight / 4;
			}
		} else if (newPosition < this.slotEl[this.activeSlot].slotMaxScroll) {
			// Prevent the slot to be dragged outside the visible area (bottom margin)
			newPosition = (newPosition - this.slotEl[this.activeSlot].slotMaxScroll) / 2 + this.slotEl[this.activeSlot].slotMaxScroll;
			newDuration /= 3;
			
			if (newPosition < this.slotEl[this.activeSlot].slotMaxScroll - this.swSlotWrapper.clientHeight / 4) {
				newPosition = this.slotEl[this.activeSlot].slotMaxScroll - this.swSlotWrapper.clientHeight / 4;
			}
		} else {
			newPosition = Math.round(newPosition / this.cellHeight) * this.cellHeight;
		}

		this.scrollTo(this.activeSlot, Math.round(newPosition), Math.round(newDuration) + 'ms');
 
		return true;
	},

	scrollTo: function (slotNum, dest, runtime) {
		this.slotEl[slotNum].style.webkitTransitionDuration = runtime ? runtime : '100ms';
		this.setPosition(slotNum, dest ? dest : 0);

		// If we are outside of the boundaries go back to the sheepfold
		if (this.slotEl[slotNum].slotYPosition > 0 || this.slotEl[slotNum].slotYPosition < this.slotEl[slotNum].slotMaxScroll) {
			this.slotEl[slotNum].addEventListener('webkitTransitionEnd', this, false);
		}
	},
	
	scrollToValue: function (slot, value) {
		var yPos, count, i;

		this.slotEl[slot].removeEventListener('webkitTransitionEnd', this, false);
		this.slotEl[slot].style.webkitTransitionDuration = '0';
		
		count = 0;
		for (i in this.slotData[slot].values) {
			if (i == value) {
				yPos = count * this.cellHeight;
				this.setPosition(slot, yPos);
				break;
			}
			
			count -= 1;
		}
	},
	
	backWithinBoundaries: function (e) {
		e.target.removeEventListener('webkitTransitionEnd', this, false);

		this.scrollTo(e.target.slotPosition, e.target.slotYPosition > 0 ? 0 : e.target.slotMaxScroll, '150ms');
		return false;
	},


	/**
	 *
	 * Buttons
	 *
	 */

	tapDown: function (e) {
		e.currentTarget.addEventListener('touchmove', this, false);
		e.currentTarget.addEventListener('touchend', this, false);
		e.currentTarget.className = 'sw-pressed';
	},

	tapCancel: function (e) {
		e.currentTarget.removeEventListener('touchmove', this, false);
		e.currentTarget.removeEventListener('touchend', this, false);
		e.currentTarget.className = '';
	},
	
	tapUp: function (e) {
		this.tapCancel(e);

		if (e.currentTarget.id == 'sw-cancel') {
			this.cancelAction();
		} else {
			this.doneAction();
		}
		
		this.close();
	},

	setCancelAction: function (action) {
		this.cancelAction = action;
	},

	setDoneAction: function (action) {
		this.doneAction = action;
	},
	
	cancelAction: function () {
		return false;
	},

	cancelDone: function () {
		return true;
	}
};
